////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : AbstractCommand.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 10:36:59 2000
//  Update count     : 2
//  CVS Info         : $Id: AbstractCommand.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  26.02.01        vigelius    enhanced documentation
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


/**
 * AbstractCommand is the base class for all commands. Commands are processed
 * by algorithms and should perform only one logical step.
 * @see visalg.basics.Algorithm
 */
public class AbstractCommand implements Command {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Does nothing at default. Can be overwritten in derived classes.
     */
    public AbstractCommand() {
    }


    // -------- from interface Command  --------------------

    /**
     * This method is called to proceed the command. Must be overwritten in
     * derived classes.
     */
    public void doIt() {
    }

    /**
     * This function should return a string describing the command. Must be
     * overwritten in derived classes.
     * @return Short "one word" description of the command.
     **/
    public String toString() {
	    return "";
    }
} // class AbstractCommand
