////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ModuleManager.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:36 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 11:00:38 2000
//  Update count     : 10
//  CVS Info         : $Id: ModuleManager.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	        Name		Changes/Extensions
//  ----	        ----		------------------
//  26.02.01        vigelius    Enhanced documentation
//  04.04.01        vigelius    added member function getMenuBarContainer
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import visalg.graphics.ModuleWindowContainer;
import visalg.graphics.MenuBarContainer;

import java.util.Vector;


/**
 * The ModuleManager provides an interface for every class, that manages other
 * modules. This is by default the project, but it can be a recursive algorithm
 * or something else, too. The modules need this functionality to display itself
 * or to look for other modules. The ModuleManager can also invoke a step.
**/
public interface ModuleManager {
    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Adds a module on the same recursion level.
     *
     * @param m The module, that has to be added
     *
     * @return True at success
     **/
    public boolean addModule(Module m);

    /**
     * Removes a certain module.
     *
     * @param m The module, that has to be removed
     *
     * @return True at success
     **/
    public boolean removeModule(Module m);

    /**
     * Swaps to modules in the list.
     *
     * @param a,b Positions of the modules to be swapped
     */
    public void swapModules(int a, int b);

    /**
     * Returns a list of all modules on this recursion level.
     *
     * @return All modules
     */
    public Vector getModules();

    /**
     * Returns a list of all data modules on this recursion level.
     *
     * @return All data modules
     */
    public Vector getDataModules();

    /**
     * Returns the Container for the modules. This might be the
     * ApplicationWindow, but it could also be a module window, that
     * shows its submodules.
     *
     * @see visalg.graphics.ApplicationWindow
     *
     * @return The container
     */
    public ModuleWindowContainer getModuleWindowContainer();

    /**
     * Must initiate a new step for all modules.
     */
    public void historyStep();

    /**
     * Is called, if the list of modules has been changed, and thererfore a
     * repaint is necessary.
     */
    public void repaint();

    /**
     * Gives back the MenuBarContainer for this manager. This might be the
     * project window, but could be anything else.
     *
     * @return The MenuBarContainer
     */
    public MenuBarContainer getMenuBarContainer();
} // interface ModuleManager
