////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ObjectReader.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:31 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 11:02:28 2000
//  Update count     : 3
//  CVS Info         : $Id: ObjectReader.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  26.02.01    vigelius    enhanced documentation
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.io.File;


/**
 * ObjectReader reads in a binary object, and provides the object in a public
 * member. If an error occurred, this member is set to null.
*/
public class ObjectReader {
    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * This string is the prefix, that indicates, what converter should be
     * used for an import file.
     */
    public static final String converterMagicNumber = "!";

    /**
     * If the serialisation process succeeded, this member state contains the
     * object, otherwise null.
     */
    public Object object;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Reads in an object out of the file, that is declared by a filename.
     *
     * @param fileName  The name of the file
     */
    public ObjectReader(String fileName) {
    	object = null;
	    try {
	        ObjectInputStream in = new ObjectInputStream(new FileInputStream(fileName));
    	    object = in.readObject();
	        in.close();
    	} catch (Exception e) {
	        object = null;
	    //System.out.println("Fehler beim Lesen der Datei " + fileName);
    	}
    }//ObjectReader(String)

    /**
     * Reads in an object out of the file, that is declared by its reference.
     *
     * @param f Reference to the file
     */
    public ObjectReader(File f) {
    	this(f.getAbsolutePath());
    }//ObjectReader(File)
} // class ObjectReader
