////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : OtherModule.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:27 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 11:03:06 2000
//  Update count     : 3
//  CVS Info         : $Id: OtherModule.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	        Name		Changes/Extensions
//  ----	        ----		------------------
//  26.02.01        vigelius    enhanced documentation
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import java.io.Serializable;


/**
 * OtherModule provides the interface for the interaction between modules. This
 * means especially the possibility of change notification. Many methods of this
 * interface are implemented in the class AbstractModule.
 *
 * @see visalg.basics.AbstractModule
*/
public interface OtherModule extends Serializable {
    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Adds a new ModuleListener, that is notified, if the module has changed.
     *
     * @param a The ModuleListener that wants to be notified
     */
    public void addModuleListener(ModuleListener a);

    /**
     * Removes a ModuleListenr of the notification list.
     *
     * @param a The ModuleListener, that does not want to be notified anymore
     */
    public void removeModuleListener(ModuleListener a);

    /**
     * Returns the number of performed steps.
     *
     * @return Number of steps performed
     */
    public int getTime();

    /**
     * Returns the name of the module.
     *
     * @return Name of the module
     */
    public String toString();
} // interface OtherModule