////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : Project.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:20 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 14:32:25 2000
//  Update count     : 32
//  CVS Info         : $Id: Project.java,v 1.2 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----        ------------    ------------------
//  28.12.00	Vigelius        Added new Modules :
//                              - MergeSort (incl. Viewers)
//                              - BubbleSort (incl. Viewers)
//  27.02.01    vigelius        Enhanced documentation
//  17.05.01    vigelius        added function call afterDeserialisation()
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.modules.*;

import java.io.Serializable;

import java.util.Vector;
import java.util.Enumeration;

import javax.swing.event.ChangeEvent;

import java.awt.Component;


/**
 * A Project manages a list of modules and is in this sense a ModuleManager.
 * One can add or remove modules, activate, deactivate or swap modules. The
 * step() method, that is invoked by the History, will call step() of every
 * module, if it is activated and steppable.
 *
 * @see visalg.basics.ModuleManager
 * @see visalg.basics.History
 * @see visalg.basics.Module#step()
*/
public class Project implements ModuleManager, Serializable {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * The description of the project, shown in the HistoryWindow.
     * @serial Description
     */
    private String m_comment;

    /**
     * Reference to the History, that this project belongs to.
     * @serial History
     */
    private History m_history;

    /**
     * List of all conncted modules.
     * @serial List of all modules
     */
    private Vector m_modules;

    /**
     * Reference to the ProjectWindow (the view).
     */
    private transient ProjectWindow m_projectWindow;

    /**
     * Number of all performed steps.
     * @serial Number of performed steps
     */
    private int m_time;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Besides initialising member states, the Project adds all modules
     * statically. It must know the History in order to provide the
     * WindowContainers.
     *
     * @TODO Add GUI-elements, so that the user can add or remove modules by
     * himself.
     *
     * @param h Reference to the History
     * @see visalg.graphics.ModuleWindowContainer
     * @see visalg.basics.History
     */
    public Project(History h) {

	m_history = h;
	m_time = 0;
	m_modules = new Vector(0,1);
	m_comment = "Bisher nur Testprojekt..."; //test-init

	// TODO: Hier wird bisher eine feste Konfiguration von Modulen
	// vorgegeben. Ziel ist, dass einem Projekt ueber einen Dialog
	// beliebige Module hinzugefuegt werden koennen.
	//
	// Ferner macht es wohl Sinn, dass die Viewer in dem zugehoerigen Modul
	// geoeffnet werden (z.B. SelectionSortViewer Modul im SelectionSort
	// Modul). Somit ist dann auch direkt die Zugehoerigkeit SelectionSort -
	// SelectionSortViewer klar und muss nicht ueber eine zweifelhafte
	// Funktion "findSelectionSort" bestimmt werden.

	//addModule(new DummyAlgorithm2("Dummy verwaltet Sub-Dummies", this));
	//addModule(new DummyAlgorithm("Dummy1", this));
	//addModule(new DummyAlgorithm("Dummy2", this));
	//addModule(new DummyAlgorithm("Dummy3", this));
	//addModule(new DataModule(this));

	// !!! zu Testzwecken einige Module auskommentiert !!!

        //addModule(new SelectionSort(this));
	//addModule(new SelectionSortViewer(this));
	//addModule(new SelectionSortMatrixViewer(this));

	//addModule(new PrimitiveStatistics(this));
	//addModule(new PrimitiveTrigger(this));
	//addModule(new PrimitiveObserver(this));

	//addModule(new MatrixViewer(this));

  	//addModule(new MergeSort(this));
  	//addModule(new MergeSortViewer(this));
  	//addModule(new MergeSortMatrixViewer(this));
 	//addModule(new QuickSort(this));
  	//addModule(new QuickSortViewer(this));
  	//addModule(new QuickSortMatrixViewer(this));
  	//addModule(new BubbleSort(this));
  	//addModule(new BubbleSortViewer(this));
  	//addModule(new BubbleSortMatrixViewer(this));

     //addModule(new DummyTester(this));
    } // Project(History h)


    /**
     * Deletes a module out of the list.
     *
     * @param  m The module to be deleted

     */
    public void deleteModule(Module m) {
	m_modules.removeElement(m);
	notifyProjectWindow();
    }//deleteModule(Module)

    /**
     * For each module contained in the project the corresponding
     * window will be removed.
     */
    public void discardWindows() {

	// If a ProjectWindow has been loaded already, remove its menu
        if (m_projectWindow!=null) {
            m_projectWindow.unloadMenu();
        }

	// Traverse all contained modules and call discardWindows on
	// them; afterwards remove them

	for (Enumeration e = m_modules.elements() ; e.hasMoreElements() ;) {

	    Object o = e.nextElement();

	    if (o instanceof java.awt.Component) {
		m_modules.remove(o);
	    }

	    // ??? Warum nicht erst diese Anweisung und dann loeschen?
	    ((Module)o).discardWindows();

	} // for

    } // discardWindows()

    /**
     * Returns the comment of the project.
     * @return The comment
     */
    public String getComment() {
	return new String(m_comment);
    }//getComment()

    /**
     * Provides the MenuBarContainer.
     * @return The conatainer
     * Gibt den Container an, der das Men darstellt.
     */
    public MenuBarContainer getMenuBarContainer() {
	return m_history.getMenuBarContainer();
    }//getMenuBarContainer()

    /**
     * Gives back the number of performed steps (the project time)
     *
     * @return The project time
     */
    public int getTime() {
	return m_time;
    }//getTime()

    /**
     * Opens a new ProjectWindow as well as all ModuleWindows. It goes through
     * the module list and opens a window, if its state is currently set to
     * "visible".
     *
     * @see visalg.graphics.ProjectWindow
     * @see visalg.graphics.ModuleWindow
     * @see visalg.basics.AbstractModule#isShowing()
     * @see visalg.basics.AbstractModule#setShowing(boolean)
     */
    public void newWindows() {

	ProjectWindowContainer pwc = m_history.getProjectWindowContainer();

        if (pwc != null) {
	    try {
		m_projectWindow = new ProjectWindow(this, pwc);
	    } catch (Exception e) {}
	}

	for (Enumeration e = m_modules.elements() ; e.hasMoreElements() ;) {
	    Module m = (Module)e.nextElement();
        m.afterDeserialisation();
	    if (m.isShowing()) {
		m.newModuleWindow();
	    }
	}
    }//newWindows()

    /**
     * Sets a new comment string for this project.
     *
     * @param s The new comment
     */
    public void setComment(String s) {
	m_comment = s;
	notifyProjectWindow();
    }//setComment()

    /**
     * Sets a new History for this project. This is needed after deserialisation
     * in order to restore the references.
     *
     * @param h The new History
     */
    public void setHistory(History h) {
	m_history = h;
    }//setHistory(History)

    /**
     * This method is called by the History in order to request a global step.
     * It goes through the list of the modules, and calls in every active
     * module the step() method. Additionally, the project time is increased
     * by one and the ProjectWindow is notified of a change (if necessary).
     *
     * @see visalg.basics.Module
     * @see visalg.basics.Module#step()
     * @see visalg.graphics.ProjectWindow
     */
    public void step() {
	projectStep();
    }//step()

    /**
     * Returns the comment string of this project by calling getComment().
     *
     * @return The comment
     *
     * @see visalg.basics.Project#getComment()
     */
    public String toString() {
	return getComment();
    }//toString()


    // ----------- from Interface ModuleManager ------------------

    /**
     * Adds the given module to the internal list and notifies the
     * ProjectWindow of a change. This method is up to now just called by
     * the constructor.
     *
     * @param m The module to be added
     * @see visalg.basics.Project#Project(visalg.basics.History)
     * @see visalg.graphics.ProjectWindow
     */
    public boolean addModule(Module m) {
	m_modules.add(m);
	notifyProjectWindow();
	return true;
    }//addModule(Module)

    /**
     * Gives back a list of all DataModules. This is usefull for all viewers,
     * for example.
     *
     * @return All DataModules
     *
     * @see visalg.basics.DataModule
     */
    public Vector getDataModules() {
	Vector collecting = new Vector();
	for (Enumeration e = m_modules.elements() ; e.hasMoreElements() ;) {
	    Object o = e.nextElement();
	    if (o instanceof DataModule) collecting.add(o);
	}
	return collecting;
    }//getDataModules()

    /**
     * Returns all modules in the list. This function is used by ProjectWindow
     * and ApplicationWindow in order to open the windows, but it can be usefull
     * for other modules, too.
     *
     * @return All modules
     *
     * @see visalg.graphics.ProjectWindow
     * @see visalg.graphics.ApplicationWindow
     */
    public Vector getModules() {
	Vector collecting = new Vector();
	for (Enumeration e = m_modules.elements() ; e.hasMoreElements() ;)
	    collecting.add(e.nextElement());
	return collecting;
    }//getModules()

    /**
     * Gives back the container for the ModuleWindow.
     *
     * @return The container
     *
     * @see visalg.graphics.ModuleWindow
     */
    public ModuleWindowContainer getModuleWindowContainer() {
	return m_history.getModuleWindowContainer();
    }//getModuleWindowContainer()

    /**
     * Requests a global step of the History. This call is simply passed to
     * the step() method of the History, if it exists (what should be the case
     * ...)
     *
     * @see visalg.basics.History#step()
     */
    public void historyStep() {
	if (m_history != null) {
	    m_history.step();
	}
    }//historyStep()

    /**
     * Removes the specified module out of the list. This does exactly the
     * same as deleteModule(), but the ProjectWindow is not notified. This
     * function is implemented of the interface ModuleManager and should be
     * used rather than deleteModule().
     *
     * @see visalg.basics.Project#deleteModule(visalg.basics.Module)
     *
     * @return True, if the specified Module was found and could be removed
     */
    public boolean removeModule(Module m) {
	return m_modules.remove(m);
    }//removeModule(Module)

    /**
     * Swaps the position of the two specified Modules.
     *
     * @param a,b The modules, that are to be swapped
     */
    public void swapModules(int a, int b) {
	int size = m_modules.size();
	if ((a >= 0)&&(a < size)&&(b >= 0)&&(b < size)&&(a != b)) {
	    m_modules.set(a, m_modules.set(b, m_modules.get(a)));
	    notifyProjectWindow();
	}
    }//swapModules(int,int)

    /**
     * Repaints its window by simply passing the repaint() to the window.
     */
    public void repaint() {
        m_projectWindow.repaint();
    }//repaint()

    // -----------------------------------------------------
    // ------- M e t h o d s ( p r o t e c t e d ) ---------
    // -----------------------------------------------------

    /**
     * Notifies the corresponding project window of changes taken place in the
     * model by calling projectChanged(ChangeEvent)
     *
     * @see visalg.graphics.ProjectWindow
     * @see visalg.graphics.ProjectWindow#projectChanged(ChangeEvent)
     */
    protected void notifyProjectWindow() {

	if (m_projectWindow != null) {
	    m_projectWindow.projectChanged(new ChangeEvent(this));
	}

    }//notifyProjectWindow()


    /**
     * Will be called after a deserialisation has taken place. The call will be
     * passed to the modules.
     */
    public void afterDeserialisation() {
        Enumeration enum = m_modules.elements();
        while (enum.hasMoreElements()) {
            Module module = (Module) enum.nextElement();
            module.afterDeserialisation();
        }
    }//afterDeserialisation()

    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

    /**
     * Steps all modules in order of appearance by calling step()
     *
     * @see visalg.basics.Module#step()
     *
     * @return True, if at least one Module steps
     */
    private boolean projectStep() {
	boolean help = false;
	for (Enumeration e = m_modules.elements() ; e.hasMoreElements() ;) {
	    Module m = (Module)e.nextElement();
	    if (m.isStepping()) {
		m.step();
		help = true;
	    }
	}
	m_time += 1;
	if (help==true) notifyProjectWindow(); //wenn sich eben etwas gendert hat...
	return help;
    }//projectStep()
} // class Project
