////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PropertyManager.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Thu May 17 15:07:20 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////

package visalg.basics;

import visalg.types.PropertyObject;
import java.util.Vector;

/**
 * This interface defines methods, that are required to manage properties. The
 * most common PropertyManager is the DataModule, but other modules might also
 * make use of this.
 *
 * @see visalg.basics.DataModule
 */
public interface PropertyManager {

    /**
     * Adds a new property to the list of all properties.
     *
     * @param property The new property
     */
    public void addProperty(PropertyObject property);

    /**
     * Removes a property of the list.
     *
     * @param property The property to be removed
     */
    public void removeProperty(PropertyObject property);

    /**
     * Looks for all properties, that are identified by the given name.
     *
     * @param name The name of the queried properties
     * @return All properties, that are called "name"
     */
    public Vector queryPropertyObjects(String name);

    /**
     * Is called, if one of the registered properties has been changed.
     */
    public void propertyChanged();
}//class PropertyManager