////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : AddModuleDialog.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            : mvigelius@gmx.de
//  Copyright        : Uni Heidelberg
//  Created on       : Wed Mar 07 2001
//  Last modified by : -
//  Last modified on : -
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  06.06.01    vigelius    removed unnecessary viewer modules
//  18.06.01    vigelius    Modules are provided by a list object
//
///////////////////////////////////////////////////////////////////////////////

package visalg.components;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
//import com.borland.jbcl.layout.*;
import java.util.Vector;

import visalg.basics.*;
import visalg.modules.*;

/**
 * This class provides a dialog, where the user can dynamically add modules.
 * The available modules have to be set in the methods
 * getAvailableModules() and addSelectedModule().
 *
 * @todo It might be usefull, to let the user give a path to a module
 */
public class AddModuleDialog extends JDialog {
    /**
     * The panel for the "OK" and "Cancel" Buttons
     */
    JPanel m_finishPanel = new JPanel();

    /**
     * The "OK" button
     */
    JButton m_okButton = new JButton();

    /**
     * The Panel for the module chooser ComboBox and an explaining label
     */
    JPanel m_modulePanel = new JPanel();

    /**
     * The border for the modulePanel
     */
    TitledBorder titledBorder1;

    /**
     * The "Cancel" Button
     */
    JButton m_cancelButton = new JButton();

    /**
     * GridLayout for the modulePanel
     */
    GridLayout gridLayout2 = new GridLayout();

    /**
     * BorderLayout for the mainPanel
     */
    BorderLayout borderLayout1 = new BorderLayout();

    /**
     * The ComboBox, allowing the user to choose a module.
     */

    /**
     * Label for the text explaining the comboBox.
     */
    JLabel m_moduleLabel = new JLabel();

    /**
     * Reference to the project.
     */
    private Project m_project=null;

    /**
     * The list of all available modules.
     */
    private JList m_list;

    /**
     * Creates the dialog window and stores a reference to the project.
     * @param p The associated Project
     */
    public AddModuleDialog(Project p) {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }

        m_project = p;

        show();
    }//AddModuleDialog(Project)

    /**
     * Creates all GUI-Element.
     */
    private void jbInit() throws Exception {
        titledBorder1 = new TitledBorder("");
        this.setModal(true);
        this.setTitle("Add module");
        this.getContentPane().setLayout(borderLayout1);
        m_finishPanel.setPreferredSize(new Dimension(200, 30));
        m_okButton.setText("OK");
        m_okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_okButton_actionPerformed(e);
            }
        });
        m_cancelButton.setText("Cancel");
        m_cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_cancelButton_actionPerformed(e);
            }
        });
        m_modulePanel.setLayout(gridLayout2);
        gridLayout2.setRows(2);
        m_moduleLabel.setText("Module to add :");
        this.getContentPane().add(m_finishPanel, BorderLayout.SOUTH);
        m_finishPanel.add(m_okButton, null);
        m_finishPanel.add(m_cancelButton, null);
        this.getContentPane().add(m_modulePanel, BorderLayout.CENTER);
        m_modulePanel.add(m_moduleLabel, null);
        m_list = new JList(getAvailableModules());
        m_list.setVisibleRowCount(10);
        m_modulePanel.add(new JScrollPane(m_list));
        setBounds(50,50,500,300);
    }//jbInit()

    /**
     * If the user chooses OK, the selected module will be added to the module
     * list using ModuleManager.addModule(Module).
     *
     * @see ModuleManager#addModule(Module)
     */
    void m_okButton_actionPerformed(ActionEvent e) {
        addSelectedModule();
        dispose();
    }//m_okButton_actionPerformed(ActionEvent)

    /**
     * Just closes the dialog without changing anything.
     */
    void m_cancelButton_actionPerformed(ActionEvent e) {
        dispose();
    }//m_cancelButton_actionPerformed(ActionEvent)

    /**
     * Returns am array containing the names of all available modules.
     *
     * @return All available modules
     */
    private String[] getAvailableModules() {
        Vector moduleArray=new Vector();
        moduleArray.add("DataModule");
        moduleArray.add("BubbleSort");
        moduleArray.add("QuickSort");
        moduleArray.add("SelectionSort");
        moduleArray.add("MergeSort");
        moduleArray.add("PrimitiveObserver");
        moduleArray.add("PrimitiveStatistics");
        moduleArray.add("PrimitiveTrigger");
        moduleArray.add("DummyAlgorithm");
        moduleArray.add("DummyAlgorithm2");
        moduleArray.add("TuringMachine");
        String[] rvalue = new String[moduleArray.size()];
        moduleArray.copyInto(rvalue);
        return rvalue;
    }//getAvailableModules()

    /**
     * Creates a new instance of the selected module and adds it to the list
     * of loaded modules of the ModuleManager
     */
    private void addSelectedModule()
    {
        Object[] items = m_list.getSelectedValues();
        String module;

        for (int i=0; i<items.length; i++) {
            module = (String) items[i];

            if (module=="BubbleSort")
                m_project.addModule(new BubbleSort(m_project));
            if (module=="DummyAlgorithm")
                m_project.addModule(new DummyAlgorithm("Dummy",m_project));
            if (module=="DummyAlgorithm2")
                m_project.addModule(new DummyAlgorithm2("Dummy2",m_project));
            if (module=="MergeSort")
                m_project.addModule(new MergeSort(m_project));
            if (module=="PrimitiveObserver")
                m_project.addModule(new PrimitiveObserver(m_project));
            if (module=="PrimitiveStatistics")
                m_project.addModule(new PrimitiveStatistics(m_project));
            if (module=="PrimitiveTrigger")
                m_project.addModule(new PrimitiveTrigger(m_project));
            if (module=="QuickSort")
                m_project.addModule(new QuickSort(m_project));
            if (module=="SelectionSort")
                m_project.addModule(new SelectionSort(m_project));
            if (module=="DataModule")
                m_project.addModule(new DataModule(m_project));
            if (module=="TuringMachine")
                m_project.addModule(new TuringMachine("Turing Machine", m_project));
        }
    }////addSelectedModule
 }//class AddModuleDialog
