////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ClassTypeDialog.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Mon Nov 12 12:00:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
///////////////////////////////////////////////////////////////////////////////

package visalg.components;


import visalg.basics.DataModule;
import visalg.types.*;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * This class provides a dialog, where the user can choose an empty data set,
 * that will be created for the DataModule. After the user has chosen a type,
 * this object will be created and set to the specified DataModule.
 */
public class ClassTypeDialog extends JDialog {

    /**
     * All possible data types that can be created empty.
     */
    private static String[] m_types = {"Integer Array", "Matrix"};

    /**
     * Reference to the DataModule.
     */
    private DataModule m_dataModule;

    // Visual Elements
    public BorderLayout m_borderLayout = new BorderLayout();
    public JPanel m_finishPanel = new JPanel();
    public JButton m_okButton = new JButton();
    public JButton m_cancelButton = new JButton();
    public JPanel m_listPanel = new JPanel();
    public JList m_classTypeList = new JList(m_types);
    public BorderLayout borderLayout1 = new BorderLayout();
    public JLabel m_message = new JLabel();
    public FlowLayout flowLayout1 = new FlowLayout();
    public BorderLayout borderLayout2 = new BorderLayout();

    /**
     * Constructor. A DataModule must be given, since this class sets the
     * data automatically.
     *
     * @param dm The DataModule where the data will be put in
     */
    public ClassTypeDialog(DataModule dm) {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        m_dataModule = dm;
        show();
    }//ClassTypeDialog

    /**
     * Creates the visual elements for this dialog.
     */
    private void jbInit() throws Exception {
        this.setResizable(false);
        this.setModal(true);
        this.setTitle("Choose new Class Type ...");
        this.getContentPane().setLayout(borderLayout1);
        m_finishPanel.setLayout(flowLayout1);
        m_okButton.setMaximumSize(new Dimension(32767, 32767));
        m_okButton.setMinimumSize(new Dimension(20, 10));
        m_okButton.setPreferredSize(new Dimension(80, 30));
        m_okButton.setText("OK");
        m_okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okActionPerformed(e);
            }
        });
        m_finishPanel.setBorder(BorderFactory.createRaisedBevelBorder());
        m_finishPanel.setPreferredSize(new Dimension(350, 50));
        m_cancelButton.setMaximumSize(new Dimension(32767, 32767));
        m_cancelButton.setMinimumSize(new Dimension(20, 10));
        m_cancelButton.setPreferredSize(new Dimension(80, 30));
        m_cancelButton.setText("Cancel");
        m_cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelActionPerformed(e);
            }
        });
        m_listPanel.setLayout(borderLayout2);
        m_classTypeList.setBackground(Color.white);
        m_classTypeList.setBorder(BorderFactory.createEtchedBorder());
        m_classTypeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        m_listPanel.setBorder(BorderFactory.createRaisedBevelBorder());
        m_listPanel.setMinimumSize(new Dimension(150, 78));
        m_listPanel.setPreferredSize(new Dimension(350, 150));
        m_message.setText("Please choose a new Data Type :");
        this.getContentPane().add(m_listPanel, BorderLayout.NORTH);
        m_listPanel.add(m_message, BorderLayout.NORTH);
        m_listPanel.add(m_classTypeList, BorderLayout.CENTER);
        this.getContentPane().add(m_finishPanel, BorderLayout.SOUTH);
        m_finishPanel.add(m_okButton, null);
        m_finishPanel.add(m_cancelButton, null);
        setBounds(50,50,350,235);
    }//jbInit()

    /**
     * Just closes the window.
     */
    void cancelActionPerformed(ActionEvent e) {
        dispose();
    }//cancelActionPerformed(ActionEvent)

    /**
     * Creates the specified Data object and sets it in the DataModule. After
     * that, the dialog will be disposed.
     */
    void okActionPerformed(ActionEvent e) {
        JOptionPane confirmDialog = new JOptionPane();
        int result;

        if (m_classTypeList.getSelectedIndex()!=-1) {
            result = confirmDialog.showConfirmDialog(this,
                "Warning ! This will delete existing data ! Continue ?",
                "Confirmation ...",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE);
            if (result==JOptionPane.YES_OPTION) {
               createSelectedData();
            }
        }
        dispose();
    }//okActionPerformed(ActionEvent)

    /**
     * Creates the selected Data object and sets it in the DataModule.
     */
    private void createSelectedData() {
        switch (m_classTypeList.getSelectedIndex()) {

        // Create a new int vector object
        case 0 :
            VisAlgVector intVector = new VisAlgVector(new VisAlgInt());
            m_dataModule.setNewData(intVector);
            break;

        // Create new VisAlgMatrix
        case 1 :
            NewMatrixDialog nmd = new NewMatrixDialog();
            nmd.show();
            VisAlgMatrix matrix = nmd.getMatrix();

            if (matrix!=null) {
                m_dataModule.setNewData(matrix);
            }

             break;

        default :
        }
    }///createSelectedData()
}//class ClassTypeDialog