////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : NewMatrixDialog.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Nov 13 16:30:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
///////////////////////////////////////////////////////////////////////////////

package visalg.components;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import visalg.types.VisAlgMatrix;

/**
 * This class provides a dialog, where the user can specify the properties
 * of a matrix, that has to be created new. The user chooses the type of the
 * matrix elements (integer or double) as well as the number of rows and
 * columns. If the user finishes this dialog using the ok button, an empty
 * matrix meeting the conditions above is created an can be obtained by calling
 * getMatrix().
 */
public class NewMatrixDialog extends JDialog {

    /**
     * The matrix object that will be created.
     */
    private VisAlgMatrix m_matrix = null;

    /**
     * The available data types.
     */
    private static String[] m_dataTypeOptions = {"Integer", "Double"};

    // Visual Elements of the dialog
    public BorderLayout m_mainLayout = new BorderLayout();
    public JPanel m_finishPanel = new JPanel();
    public FlowLayout m_finishPanelLayout = new FlowLayout();
    public JButton m_okButton = new JButton();
    public JButton m_cancelButton = new JButton();
    public JPanel m_mainPanel = new JPanel();
    public JLabel m_dataTypeLabel = new JLabel();
    public JComboBox m_dataTypeBox = new JComboBox(m_dataTypeOptions);
    public JLabel m_rowsLabel = new JLabel();
    public JTextField m_rowsInput = new JTextField();
    public GridLayout m_mainPanelLayout = new GridLayout();
    public JLabel m_columnsLabel = new JLabel();
    public JTextField m_columnsInput = new JTextField();

    /**
     * Creates a new dialog.
     */
    public NewMatrixDialog() {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Returns the matrix or null, if none has been created.
     *
     * @return The matrix or null
     */
    public VisAlgMatrix getMatrix() {
        return m_matrix;
    }//getMatrix()

    /**
     * Creates all elements of this dialog.
     */
    private void jbInit() throws Exception {
        this.getContentPane().setLayout(m_mainLayout);
        m_finishPanel.setLayout(m_finishPanelLayout);
        m_okButton.setText("OK");
        m_okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_okButton_actionPerformed(e);
            }
        });
        m_cancelButton.setText("Cancel");
        m_cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                m_cancelButton_actionPerformed(e);
            }
        });
        m_finishPanel.setBorder(BorderFactory.createRaisedBevelBorder());
        m_mainPanel.setBorder(BorderFactory.createRaisedBevelBorder());
        m_mainPanel.setLayout(m_mainPanelLayout);
        m_dataTypeLabel.setText("Type of Matrix Data :");
        m_rowsLabel.setText("Number of Rows :");
        m_mainPanelLayout.setRows(3);
        m_mainPanelLayout.setColumns(2);
        m_columnsLabel.setText("Number of Columns :");
        this.setModal(true);
        this.getContentPane().add(m_finishPanel, BorderLayout.SOUTH);
        m_finishPanel.add(m_okButton, null);
        m_finishPanel.add(m_cancelButton, null);
        this.getContentPane().add(m_mainPanel, BorderLayout.CENTER);
        m_mainPanel.add(m_dataTypeLabel, null);
        m_mainPanel.add(m_dataTypeBox, null);
        m_mainPanel.add(m_rowsLabel, null);
        m_mainPanel.add(m_rowsInput, null);
        m_mainPanel.add(m_columnsLabel, null);
        m_mainPanel.add(m_columnsInput, null);
        setBounds(100,100,350,175);
    }//jbInit()

    /**
     * Will be called if the user presses ok. This method checks, if the
     * values are correct, and returns, if not. If all values are correct,
     * it creates an empty matrix meeting the specified conditions. After
     * that, the dialog window will be hidden by calling dispose().
     */
    void m_okButton_actionPerformed(ActionEvent e) {
        Integer rows, columns;
        String input;
        JOptionPane errorPane = new JOptionPane();

       input = m_rowsInput.getText();

        try {
            rows = new Integer(input);
        } catch (NumberFormatException nfe) {
            errorPane.showMessageDialog(this,
                    "Please enter a correct row value !",
                    "Wrong row value",
                    JOptionPane.ERROR_MESSAGE);

            return;
        }

        input = m_columnsInput.getText();
        try {
            columns = new Integer(input);
        } catch (NumberFormatException nfe) {
            errorPane.showMessageDialog(this,
                    "Please enter a correct column value !",
                    "Wrong column value",
                    JOptionPane.ERROR_MESSAGE);

            return;
        }

        m_matrix = new VisAlgMatrix(rows.intValue(), columns.intValue());
        for (int i=0; i<rows.intValue(); i++) {
            for (int j=0; j<columns.intValue(); j++) {
                if (m_dataTypeBox.getSelectedIndex() == 0) {
                    m_matrix.setValue(i, j, new Integer(0));
                } else if (m_dataTypeBox.getSelectedIndex()==1) {
                    m_matrix.setValue(i, j, new Double(0));
                }
            }
        }

        dispose();
    }//m_okButton_ActionPerformed(ActionEvent)

    /**
     * Just closes the dialog window.
     */
    void m_cancelButton_actionPerformed(ActionEvent e) {
        dispose();
    }///m_cancelButton_ActionPerformed(ActionEvent)
}//class NewMatrixDialog