////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ArrayOfIntConverter.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 11:08:24 2000
//  Update count     : 5
//  CVS Info         : $Id: ArrayOfIntConverter.java,v 1.2 2002/03/18 12:28:45 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  27.02.01    vigelius        Enhanced documentation
//  30.10.01    vigelius        Does now use VisAlgVectorArray
//
///////////////////////////////////////////////////////////////////////////////


package visalg.converter;


import visalg.basics.ObjectReader;
import visalg.basics.Converter;
import visalg.types.VisAlgInt;

import java.util.Vector;
import java.util.StringTokenizer;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

import java.lang.NumberFormatException;

import visalg.types.VisAlgVector;

/**
 * This class converts an ASCII file into an array of VisAlgInt
 *
 * @see visalg.types.VisAlgInt
 */
public class ArrayOfIntConverter implements Converter {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Does nothing.
     */
    public ArrayOfIntConverter() {
    }
    //ArrayOfIntConverter()

    /**
     * Reads in the data out of the specified file and gives back the array.
     * The numbers in the file must be ASCII numbers, seperated by either
     * " ", "\t", "\n" or "\r". The API function Integer.parseInt(...) is used
     * for conversion, exception calls due to errors are ignored. If the access
     * to the file was not successfull, the return value is null.
     *
     * @param f The file
     * @return A VisAlgIntArray, null if a file error occurred
     */
    public Object getData(File f) {

    // Create new VisAlgVector with initial Type VisAlgInt
    VisAlgVector dataVector = new VisAlgVector(new VisAlgInt());
	//VisAlgIntArray visAlgInts = new VisAlgIntArray();

	try {
	    BufferedReader in = new BufferedReader(new FileReader(f));
	    String line;

	    // Scan lines
	    while ((line = in.readLine()) != null) {
		    // If the file contains a magic number this line will
		    // be omitted
		    if (!(line.startsWith(ObjectReader.converterMagicNumber))) {
		        StringTokenizer t = new StringTokenizer(line, " \t\n\r");
		        while (t.hasMoreTokens()) {
			        try {
                        int integer = Integer.parseInt(t.nextToken());
                        dataVector.addData(new VisAlgInt(integer));
			        } catch (NumberFormatException nfe) {
		            } catch (IllegalArgumentException iae) {
                        // Dont know why this occurred
                        System.err.println(iae);
                    }
		        }
	        }
        }
	    in.close();
	} catch (IOException e) {
	    System.out.println("Fehler beim Lesen der Datei " + f.getAbsolutePath());
	    return null;
	}

	return dataVector;
    } // getData(File f)
} // class ArrayOfIntConverter
