////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : MatrixOfIntConverter.java
//  Description      :
//  Author           : Dominik Meidner
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Fri Apr 7 16:21:00 2000
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 11:10:31 2000
//  Update count     : 3
//  CVS Info         : $Id: MatrixOfIntConverter.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//  27.02.01    vigelius        enhanced documentation
//
///////////////////////////////////////////////////////////////////////////////


package visalg.converter;


import visalg.basics.ObjectReader;
import visalg.basics.Converter;
import visalg.types.VisAlgMatrix;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.ArrayList;


/**
 * Converts an ASCII file containing int values into a VisAlgMatrix object.
 *
 * @see visalg.types.VisAlgMatrix
 */
public class MatrixOfIntConverter implements Converter {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Reads in a matrix out of an ASCII file containing int values. The values
     * must be seperated by either " ", "\t", "\n" or "\r". The int values are
     * created using the API class Integer. Any errors are ignored, except file
     * access errors. If a file access error occurred or the matrix does not contain
     * the appropriate number of values, the return value will be null.
     * @param f The specified file
     * @return A VisAlgMatrix object
     * @see visalg.types.VisAlgMatrix
    */
    public Object getData(File f) {
	// matrix ist eine ArrayList, die vorbergehend die
	// Matrix-Daten aufnimmt.

	ArrayList matrix=new ArrayList();
	int m=0, n=0, anz=0;
	try {
	    BufferedReader in = new BufferedReader(new FileReader(f));
	    String line;
	    while ((line = in.readLine()) != null) {
		if (!(line.startsWith(ObjectReader.converterMagicNumber))) {
		    StringTokenizer t = new StringTokenizer(line, " \t\n\r");
		    while (t.hasMoreTokens()) {
			try {
			    matrix.add(new Integer(t.nextToken()));
			} catch (NumberFormatException nfe) {}
		    }
		}
	    }
	    in.close();
	} catch (IOException e) {
	    System.out.println("Fehler beim Lesen der Datei " + f.getAbsolutePath()+"\n");
	    return null;
	}
	anz=matrix.size();
	try {
	    m=((Integer)(matrix.get(0))).intValue();
	    n=((Integer)(matrix.get(1))).intValue();
	    VisAlgMatrix matrixfeld = new VisAlgMatrix(m,n);
	    for(int i=0;i<m;i++)
		for(int j=0;j<n;j++)
		    matrixfeld.setValue(i,j,(Number)matrix.get(i*n+j+2));
	    return matrixfeld;
	} catch (IndexOutOfBoundsException ioobe) {
	    System.out.println("Zuwenig Elemente angegeben bzw. einige Elemente nicht konvertierbar!!\n");
	    return null;
	}
    }


} // class MatrixOfIntConverter
