////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ModuleWindowContainer.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 22 16:31:38 2000
//  Update count     : 7
//  CVS Info         : $Id: ModuleWindowContainer.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  27.02.01    vigelius        enhanced documentation
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import java.awt.Rectangle;


/**
 * This interface must be implemented of any ModuleWindowContainer. The most
 * used container is the ApplicationWindow, but in principle can every
 * ModuleWindow act as a container for other ModuleWindows. The Container must
 * implement some kind of DesktopManager in order to manage the connected
 * windows and to react on user actions, like dragging or resizing.
 *
 * @see ApplicationWindow
 * @see visalg.basics.Module#setModuleWindowBounds(boolean, Rectangle)
*/
public interface ModuleWindowContainer {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Called by the ModuleWindow, if it wants to register itself.
     *
     * @param mw The ModuleWindow that wants to be registered
     * @param bounds The desired initial window bounds
     */
    public void addModuleWindow(ModuleWindow mw, Rectangle bounds);

    /**
     * Called, if a ModuleWindow needs to be removed.
     *
     * @param mw The ModuleWindow that needs to be removed
     */
    public void removeModuleWindow(ModuleWindow mw);

    /**
     * Called, if all ModuleWindows need to be closed.
     */
    public void discardModuleWindows();
} // interface ModuleWindowContainer
