////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : StatusLine.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Wed Oct  4 11:36:39 2000
//  Update count     : 6
//  CVS Info         : $Id: StatusLine.java,v 1.2 2002/03/18 12:28:48 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  27.02.01    vigelius        enhanced documentation
//  03.04.01    vigelius        the stdErr and StdOut is now piped to the line
//  15.11.01    vigelius    stdErr and stdOut is not piped anymore
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import javax.swing.*;
import java.io.*;
import java.awt.Rectangle;

/**
 * This class represents the StatusLine that is shown on the bottom of the
 * ApplicationWindow.
 *
 * @see visalg.graphics.ApplicationWindow
 *
*/
public class StatusLine extends JTextArea {

    /**
     * A specialised output stream derivation, that can be used to pipe output
     * streams to the status line.
     */
    class StatusStream extends OutputStream {
        /**
         * Writes the byte as a char to the status line using the default
         * encoding.
         */
        public void write(int b) {
            byte[] ba = { (byte) b };
            setText(new String(ba));
        }//write(int)
    }//StatusStream

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Creates itself as an empty JLabel.
     */
    public StatusLine() {
	super(" ");
        setRows(3);
        String output = new String();
        output += "Welcome to VisAlg Version 1.2\n";
        output += "Originally designed by : Martin Spoden\n";
        output += "Current developer : Matthias Vigelius\n";
        output += "For any suggestions, flames etc. mail to : mvigelius@gmx.de\n";
        setText(output);
        setEditable(false);
        //System.setOut(new PrintStream(new StatusStream()));
        //System.setErr(new PrintStream(new StatusStream()));
    }//StatusLine()

    /**
     * Creates itself with an initial message and the specified number of lines.
     *
     * @param s The initial string
     * @param n Number of lines of the StatusLine (not implemented yet)
     *
     * @todo Implement the number of lines
     */
    public StatusLine(String s, int n) {
	super(" ");
	setText(s);
    }//StatusLine(String, int)

    /**
     * Sets a new message to the Statusline.
     *
     * @param s The new message
     *
     * does not notify the ApplicationWindow of its change
     */
    public void setText(String s) {
    	if (s.equals("")) {
	        s = " ";
    	}
	    append(s);
        scrollRectToVisible(new Rectangle(getWidth(),getHeight(),getWidth(),getHeight()));
    }//setText(String)
} // class StatusLine
