////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : BubbleSort.java
//  Description      :
//  Author           : Babsi & Jochen
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Nov 16 22:56:28 1999
//  Last modified by : Dino Ahr
//  Last modified on : Tue Nov 16 22:56:28 1999
//  Update count     : 0
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//  28.12.00    vigelius        Angepasst an neue variablennamen
//  29.12.00    vigelius        Abbruchbedingung eingebaut
//  04.01.01    vigelius        Code gemaess den Richtlinien formatiert und
//                              dokumentiert.
//  21.02.01    vigelius        new module window uses correct window bound
//  05.06.01    vigelius        BubbleSort now uses Properties
//  05.11.01    vigelius        BubbleSort now uses VisAlgVector
//  07.11.01    vigelius        Does now support module changes
//
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import java.util.*;
import java.awt.event.*;
import java.io.File;
import java.awt.Color;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.*;

import visalg.converter.*;

/**
 * This class implements the Bubblesort algorithm as a VisAlg module.
 * Bubblesort is a simple, but not very efficient (O(N^2)) sorting algorithm.
 * Starting with the first two values, two number are compared and swapped if
 * necessary. After one swapping, the algorithm starts at the beginning until
 * no swappings are done anymore.
 *
 * Because after the first pass the biggest number is at its place, it does not
 * have to be compared again. The same is after the second pass and so on.
 *
 * The worst case scenario for Bubblesort is, if the numbers are sorted exactly
 * opposite. In this case, the algorithm takes O(n^2) time.
 *
 * The name of this algorithm results of the fact, that the numbers ascend like
 * bubbles.
*/

public class BubbleSort extends Algorithm {
        /**
        * Module containing current data
        * @serial
        */
        private DataModule m_dataModule;
        /**
        * @serial
        * Pointer to already sorted data
        */
        private int m_sorted;

        private IndicatorBarProperty m_sortedProperty =
            new IndicatorBarProperty("Sorted", new Color(0,0,255), -1 , -1);

        /**
        * @serial
        * Pointer to biggest data element
        */
        private int m_biggest;

        private IndicatorBarProperty m_biggestProperty =
            new IndicatorBarProperty("Biggest", new Color(255,255,255), -1, 1);

        /**
        * @serial
        * Pointer to current data element
        */
        private int m_current;

        private IndicatorBarProperty m_currentProperty =
            new IndicatorBarProperty("Current", new Color(128,0,0), -1, 0);

        /**
        * @serial
        * Result of last comparison
        */
        private int m_comparation;
        /**
        * @serial
        * Temporary result whether biggest equals sorted element
        */
        private boolean m_biggestAtSorted;


        /**
        * Initialisation of variables and command list.
        * @param p Reference to the project this algorithm belongs to
        */
        public BubbleSort(Project p) {
                super("BubbleSort", p);
                m_done = false; // unnoetig
                // Kommandoliste hat im Moment keine Funktionalitaet
                // trotzdem sollten die richtigen Klassen, und nicht
                // die superklassen gesetzt werden (siehe unten)

                m_commands = new Command[] {
                        new PointerComparison(),
                        new PointerSetting(),
                        new ValueComparison(),
                        new SwopValues(),
                };
                /*
                commands = new Command[] {
                        new IncreaseCurrent(),
                        new CompareBiggestWithCurrent(),
                        new SetBiggestToCurrent(),
                        new SwopBiggestAndCurrent(),
                        new SetSortedToBiggest(),
                        new IsBiggestAtSorted(),
                };
                */

        /**
        * Zeiger:
        */
                m_sorted = getDataSize();
                m_biggest = 0;
                m_current = 0;
                //Zwischenergebnisse
                m_comparation = 0;
                m_biggestAtSorted = false;
                //Anfangszustand
                setNextCommand(new IsBiggestAtSorted());
        }

        /**
         * Returns the current data module
         * @return reference to current data module
         */
        public DataModule getDataModule() {
                return m_dataModule;
        } //getDataModule

        /**
         * Sets the current data module and notifies change
         * @param dm new data module
         */
        public void setDataModule(DataModule dm) {
                super.setDataModule(dm);
                if (dm==null) return;
                Object o = dm.getData();
                if (o==null) return;
                if (!(o instanceof VisAlgVector)) return;
                if (!((VisAlgVector)o).isComparable()) return;
                VisAlgComparableData[] data = (VisAlgComparableData[])((VisAlgVector) o).toArray();
                m_dataModule = dm;
                reset();
                registerProperties();
                fireEvent();
        }//setDataModule

        /**
        * Calculate size of data
        * @return size of data
        */
        public int getDataSize() { // warum nicht private ?
                if (m_dataModule==null) return 0;
                Object o = m_dataModule.getData();
                if (o==null) return 0;
                if (!(o instanceof VisAlgVector)) return 0;
                if (!((VisAlgVector)o).isComparable()) return 0;
                VisAlgComparableData[] data = (VisAlgComparableData[])((VisAlgVector) o).toArray();
                return data.length;
        }//getDataSize

        /**
        * Get data element at specific position
        * @param index position of required element
        * @return element at position index
        */
        public VisAlgComparableData getDataElementAt(int index) { // warum nicht private ?
                if (m_dataModule==null) return null;
                Object o = m_dataModule.getData();
                if (!(o instanceof VisAlgVector)) return null;
                return (VisAlgComparableData)((VisAlgVector)o).getDataElementAt(index);
        }//getDataElementAt

        /**
        * Get start position of already sorted data
        * @return position
        */
        public int getSorted() { // warum nicht private ??
                return m_sorted;
        }//getSorted

        /**
        * Get position of biggest element
        * @return position
        */
        public int getBiggest() { // warum nicht private ??
                return m_biggest;
        }//getBiggest

        /**
        * Get position of current element
        * @return position
        */
        public int getCurrent() { // warum nicht private ?
                return m_current;
        }

        /**
        * Get result of the last comparison
        * @return result
        */
        protected int getComparation() { // warum nicht private ?
                return m_comparation;
        }// getComparation

        /**
        * Check wether biggest element equals sorted element
        * @return true, if biggest element equals sorted element,
        *         false, else.
        */
        protected boolean getBiggestAtSorted() { // warum nicht private ?
                return m_biggestAtSorted;
        }//getBiggestAtSorted

        /**
         * Sets all member variables to initial state.
         */
        protected void reset() {
                //Zeiger
                m_sorted = getDataSize();
                m_biggest = 0;
                m_current = 0;
                //Zwischenergebnisse
                m_comparation = 0;
                m_biggestAtSorted = false;
                //Anfangszustand
                setNextCommand(new IsBiggestAtSorted());
                m_done = false;
                m_time = 0;
                fireEvent();
        }//reset

        /**
         * Sets next command in the command list
         * @param ac next command to be executed
         */
        protected void setNextCommand(Command ac) {
            super.setNextCommand(ac);
        }//setNextCommand

        /**
         * Sets current state of algorithm
         * @param b algorithm is done or not
         */
        protected void setDone(boolean b) {
            super.setDone(b);
        } //setDone


        //OPERATIONEN (gekapselt) - und selbsterklrend:

        /**
         * Inclusion class for a pointer comparison command.
         */
        private class PointerComparison extends AbstractCommand {

        /**
         * Gives current command in plain text
         * @return current command
         */
            public String toString() {
                return "Zeigervergleich";
            }//toString
        }//class PointerComparison

        /**
         * Inclusion class for a pointer setting command
         */
        private class PointerSetting extends AbstractCommand {

        /**
         * Gives current command in plain text
         * @return current command
         */
            public String toString() {
                return "Zeiger setzen";
            }//toString
        }//class PointerSetting

        /**
         * Inclusion class for a value comparision command
         */
        private class ValueComparison extends AbstractCommand {

        /**
         * Gives current command in plain text
         * @return current command
         */
            public String toString() {
                return "Wertevergleich";
            }//toString
        }//class ValueComparison

        /**
         * Inclusion class for a value swap command
         */
        private class SwopValues extends AbstractCommand {

        /**
         * Gives current command in plain text
         * @return current command
         */
            public String toString() {
                return "Vertauschung";
            }//toString
        }//class SwopValues

        //OPERATIONEN

        /**
        * Command class : Test whether biggest element equals sorted element
        */
        private class IsBiggestAtSorted extends PointerComparison {

                /**
                 * Executes test and saves the result. If the current element
                 * smaller than the biggest up to now, the values are swapped
                 * and the next command will be SetSortedToBiggest. Otherwise
                 * IncreaseCurrent will be executed.
                 */
                public void doIt() {
                    //Fehlerhafte Konstellation abfangen...
                    if (m_dataModule==null) {
                        reset();
                        return;
                    }
                    Object o = m_dataModule.getData();
                    if (o==null) return;
                    if (!(o instanceof VisAlgVector)||(!((VisAlgVector)o).isComparable())) {
                        reset();
                        return;
                    }

                    VisAlgComparableData[] data = (VisAlgComparableData[])((VisAlgVector) o).toArray();

                    /**
                    * Ist der Wert des aktuellen Feldes kleiner als des
                    * derzeit grten ("m_biggest") werden die Werte
                    * vertauscht und der Zeiger "m_biggest" wird auf das neue
                    * Feld gesetzt. Ist der Wert jedoch grer, wird nur der
                    * Zeiger ohne Vertauschung umgesetzt.
                    */
                    if (getBiggest() < getSorted()-1) {
                        m_biggestAtSorted = false;
                        setNextCommand(new IncreaseCurrent());
                    } else {
                        m_biggestAtSorted = true;
                        setNextCommand(new SetSortedToBiggest());
                    }
                }//doIt

                /**
                 * Gives current command in plain text
                 * @return current command
                 */
                public String toString() {
                    return "Zeigervergleich (m_biggest - sorted)";
                }//toString
        }

        /**
         * Command class : Increases pointer to current element
         */
        private class IncreaseCurrent extends PointerSetting {

            /**
            * Executes command. The next command will be
            * CompareBiggestWithCurrent
            */
            public void doIt() {
                m_current += 1;
                setNextCommand(new CompareBiggestWithCurrent());
            }//doIt

            /**
            * Gives current command in plain text.
            * @return current command
            */
            public String toString() {
                return "Zeiger setzen (m_current)";
            }//toString
        }//class IncreaseCurrent

        /**
         * Command class : Copares biggest element with current element.
         */
        private class CompareBiggestWithCurrent extends ValueComparison {

            /**
             * Executes command. If the biggest element is bigger than the
             * current one, they will be swapped by calling
             * SwopBiggestAndCurrent. Otherwise current element will be marked
             * as biggest element (SetBiggestToCurrent).
             */
            public void doIt() {
                //Fehlerhafte Konstellation abfangen...
                if (m_dataModule==null) {
                    reset();
                    return;
                }

                Object o = m_dataModule.getData();
                if (o==null) return;
                if (!(o instanceof VisAlgVector)||(!((VisAlgVector)o).isComparable())) {
                    reset();
                    return;
                }

                VisAlgComparableData[] data = (VisAlgComparableData[])((VisAlgVector) o).toArray();

                m_comparation = data[getBiggest()].compareTo(data[getCurrent()]);
                if (m_comparation < 0) { // m_biggest > m_current
                    setNextCommand(new SwopBiggestAndCurrent());
                } else { // m_biggest <= m_current
                    setNextCommand(new SetBiggestToCurrent());
                }
            }//doIt

            /**
             * Gives current command as plain text.
             * @return current command
             */
            public String toString() {
                return "Wertevergleich (m_biggest - m_current)";
            }//toString
        }//class CompareBiggestWithCurrent

        /**
         * Command class : Marks current element as biggest element
         */
        private class SetBiggestToCurrent extends PointerSetting {

            /**
             * Executes command. Current element will be marked as biggest,
             * and IsBiggestAtSorted will be executed next.
             */
            public void doIt() {
                m_biggest = m_current;
                setNextCommand(new IsBiggestAtSorted());
            }//doIt

            /**
             * Returns current command as plain text.
             * @return current command
             */
            public String toString() {
                return "Zeiger setzen (m_biggest)";
            }//toString
        }//class SetBiggestToCurrent

        /**
         * Command class : Swaps biggest element and current one.
         */
        private class SwopBiggestAndCurrent extends SwopValues {
            /**
             * Executes command. The values are swapped within the data module
             * and then SetBiggestToCurrent will be called.
             */
            public void doIt() {
                //Fehlerhafte Konstellation abfangen...
                if (m_dataModule==null) {
                    reset();
                    return;
                }

                Object o = m_dataModule.getData();
                if (o==null) return;
                if (!(o instanceof VisAlgVector)||(!((VisAlgVector)o).isComparable())) {
                    reset();
                    return;
                }

                ((VisAlgVector) o).swapValues(getCurrent(), getBiggest());

                setNextCommand(new SetBiggestToCurrent());
            }//doIt

            /**
             * Returns current command as plain text.
             * @return current command
             */
            public String toString() {
                return "Vertauschung (m_current - m_biggest)";
            }//toString
        }//class SwopBiggestAndCurrent

        /**
         * Command class : The biggest element will be marked as sorted.
         */
        private class SetSortedToBiggest extends PointerSetting {

            /**
             * Executes command. The biggest element will be marked as sorted
             * and the biggest element and the current element will be set to
             * zero. Then the algorithm will start again with execution of
             * IsBiggestAtSorted.
             */
            public void doIt() {
                m_sorted = m_biggest;
                m_biggest = 0;
                m_current = 0;
                m_biggestAtSorted = false;
                //Wieder zurck zum Anfang
                setNextCommand(new IsBiggestAtSorted());
            }//doIt

            /**
             * Returns current command as plain text.
             * @return current command
             */
            public String toString() {
                return "Zeiger setzen (sorted)";
            }//toString
        }//class SetSortedToBiggest

        /**
        * This is the overriden step method. It first tests whether the current
        * data module is null or not of type VisAlgComparableData. In that case
        * the method reset will be called. It then checks if the algorithm has
        * finished yet (that means if sorted element is the start element) and
        * depending on this sets the state. At the end it will call the
        * superclass method. This method should be overwritten by derived
        * classes.
        */
        public void step() {
            if ((m_dataModule==null)
                || (m_dataModule.getData()==null)
                || !(m_dataModule.getData() instanceof VisAlgVector)
                || !((VisAlgVector)m_dataModule.getData()).isComparable())
                {
                    reset();
                }

            // Abbruchbedingung : m_sorted ist am Anfang (=0)
            // ueberpruefen und setzen

            if (m_sorted==0) {
                setDone(true);
            }

            /*
            //Abbruch-Bedingung: m_sorted steht am Anfang
                if (done) return;

                //Status abfragen
                switch(myNextOperation) {
                        case 0: isBiggestAtSorted();
                                break;
                        case 1: increaseCurrent();
                                break;
                        case 2: compareBiggestWithCurrent();
                                break;
                        case 3: swopBiggestAndCurrent();
                                break;
                        case 4: setBiggestToCurrent();
                                break;
                        case 5: setSortedToBiggest();
                                break;
                        case 6: isBiggestAtSorted();
                                break;
                        case 7: increaseCurrent();
                                break;
                        default://Algorithmus-Ende
                                done = true;
                                break;
                }
                */
                super.step();
        }//step

        /**
         * Creates a new Window of type BubbleSortWindow
         */
        public void newModuleWindow() {
            try {
                m_moduleWindow = new BubbleSortWindow(m_name,
                        this, m_moduleWindowBounds,
                        m_moduleManager.getModuleWindowContainer());
                } catch (Exception e) {}
        }//newModuleWindow

        public void registerProperties() {
             m_dataModule.addProperty(m_biggestProperty);
            m_dataModule.addProperty(m_currentProperty);
            m_dataModule.addProperty(m_sortedProperty);
        }

        public void setProperties() {
           m_biggestProperty.setIndicatorPosition(m_biggest);
            m_currentProperty.setIndicatorPosition(m_current);
            m_sortedProperty.setIndicatorPosition(m_sorted);
            if (m_dataModule!=null) {
                m_dataModule.propertyChanged();
            }
        }
        //protected void fireEvent()

        public void moduleChanged(ModuleChangeEvent evt) {
            reset();
            registerProperties();
            setProperties();
        }
}//class BubbleSort
