////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : DummyAlgorithm2.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 16:44:42 2000
//  Update count     : 4
//  CVS Info         : $Id: DummyAlgorithm2.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.modules.DummyAlgorithm;
import visalg.basics.ModuleManager;
import visalg.basics.Module;
import visalg.graphics.ModuleWindowContainer;
import visalg.graphics.ModuleWindow;
import visalg.graphics.MenuBarContainer;

import java.awt.Rectangle;

import java.util.Vector;
import java.util.Enumeration;


/**
 * DummyAlgorithm2 testet Submodule; 3 Dummy-Algorithmen werden unter
 * ihm aufgebaut, die bei jedem Schritt getaktet werden...  ... dabei
 * bleibt er selbst auch Dummy-Algorithmus und lt sich von weiteren
 * Submodulen analysieren...  (Dabei verzichte ich der Einfachheit
 * halber auf die Kapselung, da ein Submodul nicht das gleiche Modul
 * sein soll.)
 */
public class DummyAlgorithm2 extends DummyAlgorithm
    implements ModuleManager, ModuleWindowContainer {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * @serial
     */
    protected Vector m_subModules;

    /**
     * @serial
     */
    private PrimitiveStatistics m_statistics;

    /**
     * @serial
     */
    private PrimitiveTrigger m_trigger;

    /**
     * @serial
     */
    private PrimitiveObserver m_observer;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Konstruktor
     */
    public DummyAlgorithm2(String s, ModuleManager mm) {
	super(s, mm);
	m_subModules = new Vector();
	//ganz faul den ModuleManager alles machen lassen :-)
	m_subModules.add(new DummyAlgorithm(" - 1. Submodul", mm));
	m_subModules.add(new DummyAlgorithm(" - 2. Submodul", mm));
	m_subModules.add(new DummyAlgorithm(" - 3. Submodul", mm));
	//und um den Rest sich selbst kmmern
	m_subModules.add(this);
	m_statistics = new PrimitiveStatistics(this);
	m_subModules.add(m_statistics);
	m_trigger = new PrimitiveTrigger(this);
	m_subModules.add(m_trigger);
	m_observer = new PrimitiveObserver(this);
	m_subModules.add(m_observer);
    }


    // -------- from interface ModuleWindowContainer  ------------------------

    /**
     * Modul-Fenster auf den Desktop...
     */
    public void addModuleWindow(ModuleWindow mw, Rectangle bounds) {
	if (mw instanceof PrimitiveStatisticsWindow) {
	    ((DummyAlgorithmWindow2)m_moduleWindow).setStatisticsWindow(mw);
	    return;
	}
	if (mw instanceof PrimitiveTriggerWindow) {
	    ((DummyAlgorithmWindow2)m_moduleWindow).setTriggerWindow(mw);
	    return;
	}
	m_moduleManager.getModuleWindowContainer().addModuleWindow(mw,bounds);
    }

    /**
     * Modul-Fenster wegwerfen...
     */
    public void discardModuleWindows() {
	m_moduleManager.getModuleWindowContainer().discardModuleWindows();
    }

    public void removeModuleWindow(ModuleWindow mw) {
	m_moduleManager.getModuleWindowContainer().removeModuleWindow(mw);
    }


    // -------- from interface Module  ------------------------

    public void step() {
	super.step();
	for (Enumeration e = m_subModules.elements() ; e.hasMoreElements() ;) {
	    Object o = e.nextElement();
	    if (o==this) return;
	    if (o instanceof DummyAlgorithm) {
		DummyAlgorithm da = ((DummyAlgorithm)o);
		if (da.isStepping()) da.step();
	    }
	}
    }

    public Vector getSubModules() {
	return m_subModules;
    }

    public void newModuleWindow() {
	try {
	    m_moduleWindow =
		new DummyAlgorithmWindow2("DummyAlgorithmus2" + m_name,
					  this,
					  m_moduleWindowBounds,
					  m_moduleManager.getModuleWindowContainer());
	    m_statistics.newModuleWindow();
	    m_trigger.newModuleWindow();
	} catch (Exception e) {}
    }


    // -------- from interface ModuleManager  ------------------------

    /**
     *
     */
    public ModuleWindowContainer getModuleWindowContainer() {
	return this;
    }

    /**
     * vertauschen zweier Module in der Liste
     */
    public void swapModules(int a, int b) {
	if ((a<0) || (b<0) || (a>=m_subModules.size()) || (b>=m_subModules.size())) return;
	m_subModules.set(a, m_subModules.set(b, m_subModules.get(a)));
    }

    /**
     * historyStep() wird abgeblockt!
     */
    public void historyStep() {
	m_moduleManager.historyStep();
    }

    /**
     * getModules() liefert einen Vector zurck, der die Module auf gleicher Ebene enthlt.
     */
    public Vector getModules() {
	return m_subModules;
    }

    /**
     * getDataModules() liefert alle Datenmodule auf gleicher Ebene zurck.
     */
    public Vector getDataModules() {
	return null;
    }

    /**
     * addModule(Module) wird abgeblockt!
     */
    public boolean addModule(Module m) {
	return false;
    }

    /**
     * removeModule(Module) lscht ein Modul.
     */
    public boolean removeModule(Module m) {
	if (m==m_observer) {
	    m_subModules.remove(m_observer);
	    return true;
	}
	return false;
    }

    public void repaint() {
        m_moduleWindow.repaint();
    }

    public MenuBarContainer getMenuBarContainer() {
        return null;
    }
} // class DummyAlgorithm2
