////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : BubbleSortMatrixViewerWindow.java
//  Description      :
//  Author           : Babsi & Jochen
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  08.01.01    vigelius    Code gemaess den Richtlinien formatiert und
//                          dokumentiert.
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  06.06.01    vigelius    Does work now
//
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import visalg.modules.MatrixArrayViewer;

import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;

import visalg.types.VisAlgComparableData;
import visalg.types.ModuleChangeEvent;
import visalg.types.VisAlgInt;
import visalg.types.VisAlgVector;
import visalg.types.VisAlgComparableData;

import java.awt.*;
import javax.swing.event.*;

/**
 * This class does the visualisation of array data using a matrix scheme.
*/
public class MatrixArrayViewerWindow extends ModuleWindow {

    /**
     * The reference to the according data module.
     */
	private transient MatrixArrayViewer m_matrixArrayViewer;

    /**
     * @serial The window scale offset.
     */
	private static int s_scaleOffset = 30;       //y-Achse

    /**
     * @serial The border Offset.
     */
	private static int s_borderOffset = 30;      //x-Achse

    /**
     * @serial The window font.
     */
	private static Font s_font = new Font("SansSerif", Font.PLAIN, 10);

    /**
     * The window font metrics.
     */
	private transient FontMetrics m_fontMetrics;

    /**
     * The standard color for the window.
     */
	private transient Color m_standardColor;

    /**
     * @serial The color used for the representation of an element.
     */
	private static Color s_barColor = new Color(128, 0, 0);       //rote Kstchen

    /**
     * The dimensions of the window panel.
     */
	private transient Dimension m_dimension;

    /**
     * The height of the window panel.
     */
	private transient int m_panelHeight;

    /**
     * The width of the window panel.
     */
	private transient int m_panelWidth;

    /**
     * The maximum value of all data elements.
     */
	private transient int m_maxOfData;

    /**
     * Initialisation. Creates the actual window by calling the superclass
     * constructor and sets up a reference to the BubbleSortMatrixViewer
     * window.
     * @param s The window title
     * @param ssv Reference to the BubbleSortMatrixViewer module
     * @param bounds Window bounds
     * @param mwc Reference to the module window container
     */
	public MatrixArrayViewerWindow(String s, MatrixArrayViewer ssv,
            Rectangle bounds, ModuleWindowContainer mwc) {
		super(s, ssv, bounds, mwc);
		m_matrixArrayViewer = ssv;

		m_dimension = getSize(new Dimension());
	}//BubbleSortMatrixViewerWindow

    /**
     * This member method is the actual functionality of the viewer. It reads
     * out the properties both of the data module and the BubbleSortMatrixViewer
     * module and does a visual representation.
     * @param g Reference to the graphics object
     */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		m_standardColor = g.getColor();
		g.setFont(s_font);
		m_fontMetrics = g.getFontMetrics(s_font);
		m_dimension = getSize(m_dimension);

		m_panelHeight = m_dimension.height;
		m_panelWidth = m_dimension.width;

        Object o = m_matrixArrayViewer.getDataModule().getData();

        // create an array of VisAlgInt
        VisAlgComparableData[] data = (VisAlgComparableData[])((VisAlgVector) o).toArray();

		int totalNumberOfData = data.length;

		m_maxOfData = 0;

        	for (int i=0; i < data.length; i++) {
                    if ( (data[i]!=null)
                        && (data[i].compareToZero() > m_maxOfData)) {
                        m_maxOfData = data[i].compareToZero();
                    }
		}

		//Achsen beschriften:

		for (int i=0; i < data.length; i++) {
			data[i].paintComponent(g, 5, getY(i, totalNumberOfData)+10);
		}

		//Flche nach rechts lschen und die Achsen zeichnen...
		g.clearRect(s_scaleOffset, 0, m_panelWidth - s_scaleOffset,
                m_panelHeight);
		g.drawLine(s_scaleOffset, s_borderOffset, s_scaleOffset, m_panelHeight);
		g.drawLine(s_scaleOffset, s_borderOffset, m_panelWidth, s_borderOffset);
		//Kstchen zeichnen...
		g.setColor(s_barColor);
		for (int i=0; i < data.length; i++) {
                  g.fill3DRect(getX(data[i].compareToZero()),
                  getY(i, totalNumberOfData)+5, 5, 5, true);
		}

		g.setColor(m_standardColor);
	}//paintComponent

    /**
     * This member function calculates the x-position of a certain element
     * considering its value.
     * @param value The value of this element
     * @return The x-position of this element
     */
	private int getX(int value) {
		float f = (float)(m_panelWidth - s_scaleOffset- s_scaleOffset) * value;
		//return s_scaleOffset + Math.round(f/m_maxOfData);
		return s_scaleOffset + Math.round(f/m_maxOfData);
	}//getX

    /**
     * This member function calculates the y-position of a certain element
     * considering its relative position inside the data and the total amount
     * of data available.
     * @param numberOfData Relative position of the element
     * @param totalNumberOfData Total amount of data
     */
	private int getY(int numberOfData, int totalNumberOfData) {
		float f = (float)(m_panelHeight - s_borderOffset - s_borderOffset)
                * numberOfData;
		return s_borderOffset + Math.round(f/totalNumberOfData);
	}//getY

    /**
     * This member method causes a repaint of the window. It is invoked by the
     * BubbleSortMatrixViewer module if a change occured.
     */
	public void moduleChanged(ModuleChangeEvent cEvt) {
		repaint();
	}//moduleChanged
}//class BubbleSortMatrixViewerWindow