////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : MatrixToolWindow.java
//  Description      :
//  Author           : Tom Kraus, Martin Nowak, Dominik Maidner
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Don Aug 3 22:30:00 2000
//  Last modified by :
//  Last modified on :
//  Update count     :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ---				---
//  27.06.01    vigelius        adjusted to the viewer concept
//                              closes windows on discardWindows()
//
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;
import visalg.basics.DataModule;
import visalg.types.VisAlgMatrix;
import visalg.types.SamplingPoint;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.util.*;
import java.text.*;
import java.io.*;

/**
 * MatrixToolWindow visualizes a matrix and provides the opportunity to edit it.
 */
public class MatrixToolWindow extends ModuleWindow{

    // ---------------------------------
    // ------- V a r i a b l e s -------
    // ---------------------------------

    private transient DataModule m_dataModule;
    private transient VisAlgMatrix m_matrix;
	private transient MatrixWindow m_matrixWindow;
    private transient SamplingPoint[] m_samplingPoints;

    //private transient JComboBox ComboBoxDataModule;
    private transient JComboBox ComboBoxColorMode;
    private transient JLabel DataModuleClassLabel;

	private transient JButton NewColorModeButton, EditColorModeButton;

    private transient boolean
	m_dataLoaded=false,
	m_colorModeEditorActive=false,
	m_colorModeEditable=false,
	m_newColorMode=true,
	m_matrixVisable=false;

	private transient double intervall=1, minimum, maximum;

    private transient String m_colorModeName;


	/**
	* Instance of the MatrixEditor
	*/
	private transient MatrixEditor m_matrixEditor;

    /**
     * Instance of the ColorModeEditor
     */
    private transient ColorModeEditor m_colorModeEditor;

	/**
	* Button for the "Editor"
	*/
	private transient JButton MatrixEditorButton;

	/**
	* "case" variable to switch to another editor mode
	*/
	private transient int m_Switch = 1;

	/**
	* current row position (editor navigation)
	*/
	private transient int m_currentrow = 0;

	/**
	* current column position (editor navigation)
	*/
	private transient int m_currentcolumn = 0;

	/**
	* "true" when "Editor" is switched on
	*/
	private transient boolean m_matrixEditorActive=false;

	/**
	*tells if a row/column/value is selected in the "Editor"
	*/
	private transient boolean m_elementSelected=false;



    // -------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) ---------
    // -------------------------------------------------

    /**
     * Constructor, grouping the window elements and generating the MatrixWindow
     */
    public MatrixToolWindow(String s, MatrixTool mv, Rectangle bounds,
			      ModuleWindowContainer mwc, DataModule dm) {

	super(s, mv, bounds, mwc);
	final MatrixTool mvInt=mv;
	final ModuleWindowContainer mwcInt=mwc;

    m_dataModule = dm;

	JPanel northPanel=new JPanel(new GridLayout(2,1,5,5));
	JPanel panel=new JPanel(new GridLayout(1,2,5,5));
	panel.add(new JLabel("Data Selection:",JLabel.LEFT));
	//panel.add(ComboBoxDataModule = new JComboBox(mv.getDataModules()));

	northPanel.add(panel);
	northPanel.add(DataModuleClassLabel=new JLabel(dm.getDataClass().toString(),
					       JLabel.CENTER));
	northPanel.setBorder(new TitledBorder("Data"));
	m_contentPane.add(northPanel,BorderLayout.NORTH);

	JPanel centerPanel=new JPanel(new BorderLayout());
	panel=new JPanel(new GridLayout(1,2,5,5));
	panel.add(new JLabel("Color Mode Selection:",JLabel.LEFT));
	panel.add(ComboBoxColorMode=new JComboBox(findDirectoryItems()));
	ComboBoxColorMode.setSelectedIndex(-1);
	centerPanel.add(panel, BorderLayout.NORTH);
	panel=new JPanel(new GridLayout(1,2,5,5));
	panel.add(NewColorModeButton=new JButton("New Color Mode ..."));
	panel.add(EditColorModeButton=new JButton("Edit Color Mode ..."));
	centerPanel.add(panel, BorderLayout.SOUTH);
	centerPanel.setBorder(new TitledBorder("Color Mode"));
	m_contentPane.add(centerPanel,BorderLayout.CENTER);
	m_contentPane.add(MatrixEditorButton=new JButton("Editor"),BorderLayout.SOUTH);

	m_matrixWindow=new MatrixWindow("Matrix",mv,new Rectangle(0,271,240,255),mwc);


	validate();
	m_matrixVisable=false;
	m_matrixEditorActive=false;
	setMatrixToolDisplay();

	/**
	 *ActionListener for the button "New Color Mode".
	 */
	NewColorModeButton.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent evt) {
		m_newColorMode=true;
		m_colorModeEditor = new ColorModeEditor("New Color Mode",mvInt,new Rectangle(0,0,300,450),mwcInt);
		m_colorModeEditorActive=true;
		m_matrixVisable=false;
		setMatrixToolDisplay();
	    }
	});

	/**
	 *ActionListener for the button "Edit Color Mode".
	 */
	EditColorModeButton.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent evt) {
		m_newColorMode=false;
		new ColorModeEditor("Edit Color Mode",mvInt,new Rectangle(0,230,300,450),mwcInt);
		m_colorModeEditorActive=true;
		m_matrixVisable=false;
		setMatrixToolDisplay();
	    }
	});
	/**
	 *ActionListener for the "Editor"-Button.
	*/
	MatrixEditorButton.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent evt) {
		m_matrixEditor=new MatrixEditor("Editor("+m_matrix.rows+"x"+m_matrix.columns+" - "
							+m_matrix.getValue(0,0).getClass().getName()+")",
							mvInt,new Rectangle(0,0,260,270),mwcInt);
		m_matrixEditorActive=true;
		m_matrixVisable=true;
		setMatrixToolDisplay();
	    }
	});

	/**
	 *ActionListener for the Selection of the DataModule (ComboBox).
	 */
	/**ComboBoxDataModule.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent evt) {
		DataModule dm = (DataModule)ComboBoxDataModule.getSelectedItem();
		if (dm != m_dataModule)
		    setDataModule(dm);
		ComboBoxDataModule.invalidate();
		m_matrixVisable=false;
		setMatrixToolDisplay();
	    }
	});**/

	/**
	 * ActionListener for the Selection of the Color Mode (ComboBox).
	 */
	ComboBoxColorMode.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent evt) {
		loadColorMode((String)ComboBoxColorMode.getSelectedItem());
		ComboBoxColorMode.invalidate();
		m_matrixVisable=true;
		setMatrixToolDisplay();
	    }
	});

		// Set Matrix
    	m_matrix=(VisAlgMatrix) m_dataModule.getData();
    	calculateMinMaxIntervall();
	    m_dataLoaded=true;
		m_matrixVisable=false;
		setMatrixToolDisplay();

    }


    // ---------------------------------------------------
    // ------- M e t h o d s   ( p r i v a t e ) ---------
    // ---------------------------------------------------


    /**
     * findDirectoryItems looks for files ending with "*.cm" in the colorModes
     * directory and returns the found files as a String array
     */
    private String[] findDirectoryItems() {
	File directory=new File("."+File.separator+"colorModes");
	if(!(directory.exists()))
	    directory.mkdir();
	String[] content=directory.list(), nameList;
	int newlength=0;
	for(int i=0;i<content.length;i++)
	    if(content[i].endsWith(".cm"))
		newlength++;
	nameList=new String[newlength];
	newlength=0;
	for(int i=0;i<content.length;i++)
	    if(content[i].endsWith(".cm")){
		nameList[newlength]=content[i].substring(0,content[i].length()-3);
		newlength++;
	    }
	return nameList;
    }

    /**
     * If the transfered DataModule contains a matrix setDataModule
     * sets it as the current valid DataModule and its content as the
     * current valid matrix and calls up the method calculateMinMaxIntervall
     */
/**    private void setDataModule(DataModule dm) {
	if (dm==null) return;
	Object o = dm.getData();
	if (!(o instanceof VisAlgMatrix)) return;
	m_dataModule = dm;
	DataModuleClassLabel.setText("Matrix loaded.");
    }
**/
    /**
     * calculateMinMaxIntervall calculates the smallest and the biggest
     * matrix element and the range of the values, if a new DataModule
     * is selected.
     */
    private void calculateMinMaxIntervall() {
	double min=m_matrix.getValue(0,0).doubleValue(), max=min, temp;
	for(int i=0;i<m_matrix.rows;i++)
	    for(int j=0;j<m_matrix.columns;j++){
		temp=m_matrix.getValue(i,j).doubleValue();
		if(min>temp) min=temp;
		if(max<temp) max=temp;
	    }
	intervall=(maximum=max)-(minimum=min);
	if(intervall==0) intervall=1;
    }

    /**
     * setMatrixToolDisplay sets the Enabled-Flags of the window elements,
     * depending from the Status-Flags m_dataLoaded, m_colorModeEditorActive,
     * m_colorModeEditable, m_matrixEditorActive.
     */
    private void setMatrixToolDisplay(){
	m_matrixWindow.repaint();
	//ComboBoxDataModule.setEnabled(!m_colorModeEditorActive && !m_matrixEditorActive);
	ComboBoxColorMode.setEnabled(m_dataLoaded && !m_colorModeEditorActive);
	NewColorModeButton.setEnabled(m_dataLoaded && !m_colorModeEditorActive && !m_matrixEditorActive);
	MatrixEditorButton.setEnabled(m_dataLoaded && !m_colorModeEditorActive && m_colorModeEditable && !m_matrixEditorActive);
	EditColorModeButton.setEnabled(m_dataLoaded && !m_colorModeEditorActive && m_colorModeEditable &&
									!m_matrixEditorActive);
	repaint();
	}

    /**
     * loadColorMode loads the file with the file name that is selected in
     * the ComboBox and sets its content as the current valid SamplingPoint
     * field.
     */
    private void loadColorMode(String name){
	File newfile=new File("."+File.separator+"colorModes"+File.separator+
			    name+".cm");
	Object object=null;
	try {
	    ObjectInputStream in = new ObjectInputStream(new FileInputStream(newfile));
	    object = in.readObject();
	    in.close();
	} catch (Exception e) {
	    System.out.println(e);
	    JOptionPane.showMessageDialog(null, "Error Opening File!",
					  "Open", JOptionPane.ERROR_MESSAGE);
	    m_colorModeEditable=false;
	    return;
	}
	if(!(object instanceof SamplingPoint[])){
	    JOptionPane.showMessageDialog(null, "No Valid SamplingPoint File!",
					  "Open", JOptionPane.ERROR_MESSAGE);
	    m_colorModeEditable=false;
	    return;
	}
	m_samplingPoints=(SamplingPoint[])object;
	m_colorModeName=name;
	m_colorModeEditable=true;
    }

    // -----------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------
    // -----------------------------------------------


/*
/ The private class MatrixEditor was added to realize the "Editor"
*/
	private class MatrixEditor extends ModuleWindow {

    // ---------------------------------
    // ------- V a r i a b l e s -------
    // ---------------------------------
	/**
     * needed to cancel all changes made to the matrix
     */
	private transient VisAlgMatrix matrixcopy=new VisAlgMatrix(m_matrix.rows,m_matrix.columns);;

	/**
     * needed to save the last of the "row" transformations
     */
	private transient int historyrow1=-1, historyrow2=-1;

	/**
     * needed to save the last of the "column" transformations
     */
	private transient int historycolumn1=-1, historycolumn2=-1;

	/**
     * needed to change single values of the matrix
     */
	Number valueold, valuenew;

	/**
     * makes an "Undo" possible
     */
	private transient boolean history=false;

	/**
     * makes a "Redo" possible
     */
	private transient boolean future=false;

	/**
     * tells you the mode of last/future transformation (useless right now but could be useful
     * to integrate the history window functionality).
     */
	private transient boolean history_mode_dragged;

	/*JTextField
		mposition,                             //tells you the current row position
		nposition,                             //tells you the current column position
		value;                                  //displays a single value in the matrix
	*/
	/**
     * ComboBox to choose the "Editor" mode
     */
	JComboBox modecombo;

	/*JButton
		prev, next, up, down,                  //Buttons to navigate in the MatrixWindow
		select,                                //Button to select a row/column/single value
		back,                                  //Button to cancel or undo a transformation
		ok,                                    //Button to finish or redo a transformation
		cancel,                                //Button to cancel all changes made to the matrix and close the "Editor"
		done;                                  //Button to accept all changes and close the "Editor"
	*/

    // -------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) ---------
    // -------------------------------------------------

    /**
     * GridBagLayout-Constructor
    */
    public void concept(GridBagConstraints gbc, int gx, int gy,
				int gw, int gh, int wx, int wy){
		gbc.gridx = gx;
		gbc.gridy = gy;
		gbc.gridwidth = gw;
		gbc.gridheight = gh;
		gbc.weightx = wx;
		gbc.weighty = wy;
	}

	public void con(GridBagConstraints gb,
					int hx, int hy,
					int hw, int hh, int vx, int vy){
		gb.gridx = hx;
		gb.gridy = hy;
		gb.gridwidth = hw;
		gb.gridheight = hh;
		gb.weightx = vx;
		gb.weighty = vy;
	}
		JPanel frame = new JPanel(new BorderLayout());
		JPanel northPanel = new JPanel();

		JPanel opPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 3, 20));
		JPanel exrowPanel = new JPanel(new GridLayout(2, 3, 5, 5));
		JPanel moverowPanel = new JPanel(new GridLayout(2, 4, 5, 5));
		JPanel excolPanel = new JPanel(new GridLayout(2, 3, 5, 5));
		JPanel movecolPanel = new JPanel(new GridLayout(2, 4, 5, 5));
		JPanel entryPanel = new JPanel(new GridLayout(3, 4, 5, 5));
		JPanel southPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		GridBagConstraints constraints = new GridBagConstraints();
		GridBagConstraints conts = new GridBagConstraints();
		GridBagLayout lay = new GridBagLayout();
		GridBagLayout layup = new GridBagLayout();

		JPanel cardPanel = new JPanel();
		CardLayout cardLayout = new CardLayout();


		JPanel[] card = new JPanel[5];

		JLabel rilabel = new JLabel("row i");
		JTextField mposition = new JTextField("Pos",3);
		JLabel rjlabel = new JLabel("row j");
		JTextField mtargetposition = new JTextField("Pos",3);
		JLabel cklabel = new JLabel("column k");
		JTextField nposition = new JTextField("Pos",3);
		JLabel cllabel = new JLabel("column l");
		JTextField ntargetposition = new JTextField("Pos",3);
		JLabel valuelabel = new JLabel("value:");
		JTextField value = new JTextField("Value");
		JButton prev =new JButton("<");
		JButton next =new JButton(">");
		JButton up =new JButton("^");
		JButton down =new JButton("v");
		JButton nextentry =new JButton(">>");
		JButton select =new JButton("Select");
		JButton back =new JButton("Back");
		JButton ok = new JButton("OK");
		JButton cancel = new JButton("Cancel");
		JButton done = new JButton("Done");
		JLabel equalizer1 = new JLabel("");
		JLabel equalizer2 = new JLabel("");
		JLabel equalizer3 = new JLabel("");
		JLabel equalizer4 = new JLabel("");

    public void moduleChanged(ModuleChangeEvent cEvt) {};

    /**
     * Constructor of the "Editor" Window
     */
	public MatrixEditor(String s, MatrixTool mv, Rectangle bounds,
			    ModuleWindowContainer mwc){
	    super(s, mv, bounds, mwc);
		final ModuleWindowContainer mwcInt=mwc;

		frame.add(northPanel, BorderLayout.NORTH);
		frame.add(cardPanel, BorderLayout.CENTER);
		frame.add(southPanel, BorderLayout.SOUTH);

		cardPanel.setBorder(new TitledBorder("Action"));

		card[0] = new JPanel(new GridLayout(2, 1, 5, 5));
		card[1] = new JPanel(new GridLayout(2, 1, 5, 5));
		card[2] = new JPanel(new GridLayout(2, 1, 5, 5));
		card[3] = new JPanel(new GridLayout(2, 1, 5, 5));
		card[4] = new JPanel(new GridLayout(2, 1, 5, 5));
		cardPanel.setLayout(cardLayout);
		cardPanel.add("Card 1", card[0]);
		cardPanel.add("Card 2", card[1]);
		cardPanel.add("Card 3", card[2]);
		cardPanel.add("Card 4", card[3]);
		cardPanel.add("Card 5", card[4]);

		modecombo = new JComboBox();
		modecombo.setToolTipText("Editor mode");
		modecombo.addItem("exchange rows i and j");
		modecombo.addItem("move row i");
		modecombo.addItem("exchange columns k and l");
		modecombo.addItem("move column k");
		modecombo.addItem("modify entry (i, k)");

		JLabel comboboxlabel = new JLabel("Task:");

		con(conts, 0, 0, 1, 1, 30, 100);
		conts.fill = GridBagConstraints.HORIZONTAL;
		layup.setConstraints(comboboxlabel, conts);
		northPanel.add(comboboxlabel);

		con(conts, 0, 0, 1, 1, 70, 0);
		conts.fill = GridBagConstraints.HORIZONTAL;
		layup.setConstraints(modecombo, conts);
		northPanel.add(modecombo);

		select.setToolTipText("Pick a row/column");
		opPanel.add(select);
		back.setToolTipText("Cancel/Undo action");
		opPanel.add(back);
		ok.setToolTipText("Finish/Redo action");
		opPanel.add(ok);
		southPanel.add(cancel);
		southPanel.add(done);

		m_Switch =1;
		cardLayout.show(cardPanel, "Card 1");
		layout1();
		ok.setText("Exchange");
		m_currentrow=0;
		m_currentcolumn=0;

		m_contentPane.add(frame);
		validate();
		setTextFields();
		setMatrixEditorDisplay();
		m_matrixWindow.repaint();

		/**
		* Makes a copy of the original matrix
		*/
		copy(m_matrix,matrixcopy);

		/**
		*  ActionListener for the Selection of the editor mode(ComboBox)
		*/
		modecombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				String Modus = (String)cb.getSelectedItem();
				m_currentrow=0;
				m_currentcolumn=0;
				history=false;
				future=false;
				if (Modus=="exchange rows i and j") {
					m_Switch=1;
					cardLayout.show(cardPanel, "Card 1");
					layout1();
					ok.setText("Exchange");
				}
				if (Modus=="move row i") {
					m_Switch=2;
					cardLayout.show(cardPanel, "Card 2");
					layout2();
					ok.setText("OK");
				}
				if (Modus=="exchange columns k and l") {
					m_Switch=3;
					cardLayout.show(cardPanel, "Card 3");
					layout3();
					ok.setText("Exchange");
				}
				if (Modus=="move column k") {
					m_Switch=4;
					cardLayout.show(cardPanel, "Card 4");
					layout4();
					ok.setText("OK");
				}
				if (Modus=="modify entry (i, k)") {
					m_Switch=5;
					cardLayout.show(cardPanel, "Card 5");
					layout5();
					ok.setText("OK");
				}
				setTextFields();
				setMatrixEditorDisplay();
				m_matrixWindow.repaint();
				}
	});
	/**
	 * ActionListener for the "Row i" Textfield
	 */
		mposition.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				String row_new;
				try{
					row_new=mposition.getText();
					int newrow=Integer.parseInt((String)row_new);
					if(newrow<0||newrow>=m_matrix.rows){
						setTextFields();
						return;
					}
					if (m_elementSelected && m_Switch==2){
						DragRow(m_currentrow,newrow);
					}
					m_currentrow=newrow;
					if(!m_elementSelected && (m_Switch==1 || m_Switch==2)){
						SelectRowColumnOrValue();
					}
					setTextFields();
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				} catch(Exception e){
					return;
					}
			}
		});

		/**
		*  ActionListener for the "Row j" Textfield
		*/
		mtargetposition.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				String row_new;
				try{
					row_new=mtargetposition.getText();
					int newrow=Integer.parseInt((String)row_new);
					if(newrow<0||newrow>=m_matrix.rows){
						setTextFields();
						return;
					}
					m_currentrow=newrow;
					setTextFields();
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				} catch(Exception e){
					return;
					}
			}
		});

		/**
		*  ActionListener for the "Column k" Textfield
		*/
		nposition.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				String column_new;
				try{
					column_new=nposition.getText();
					int newcolumn=Integer.parseInt((String)column_new);
					if(newcolumn<0||newcolumn>=m_matrix.columns){
						setTextFields();
						return;
					}
					if (m_elementSelected && m_Switch==4){
						DragColumn(m_currentcolumn,newcolumn);
					}
					m_currentcolumn=newcolumn;
					if(!m_elementSelected && (m_Switch==3 || m_Switch==4)){
						SelectRowColumnOrValue();
					}
					setTextFields();
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				} catch(Exception e){
					return;
					}
			}
		});

		/**
		*  ActionListener for the "Column l" Textfield
		*/
		ntargetposition.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				String column_new;
				try{
					column_new=ntargetposition.getText();
					int newcolumn=Integer.parseInt((String)column_new);
					if(newcolumn<0||newcolumn>=m_matrix.columns){
						setTextFields();
						return;
					}
					m_currentcolumn=newcolumn;
					setTextFields();
					m_matrixWindow.repaint();
					setTextFields();
				} catch(Exception e){
					return;
					}
			}
		});

		/**
		*  ActionListener for the "Value" Textfield
		*/
		value.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				String value_new;
				String valuetype;
				try{
					valuetype=valueold.getClass().getName();
					value_new=value.getText();
					if (valuetype.endsWith(".Integer")) valuenew=new Integer((String) value_new);
					if (valuetype.endsWith(".Double")) valuenew=new Double((String) value_new);
					if (valuetype.endsWith(".Long")) valuenew=new Long((String) value_new);
					if (valuetype.endsWith(".Short")) valuenew=new Short((String) value_new);
					if (valuetype.endsWith(".Float")) valuenew=new Float((String) value_new);
					if (valuetype.endsWith(".Byte")) valuenew=new Byte((String) value_new);
					if (valuenew.floatValue()==m_matrix.getValue(m_currentrow,m_currentcolumn).floatValue()){
						skipToNextEntry();
					}
					else {
						m_matrix.setValue(m_currentrow,m_currentcolumn,valuenew);
					}
				} catch(Exception e){
					//m_matrix.setValue(m_currentrow,m_currentcolumn,valueold);
					setTextFields();
					return;
				}
				setTextFields();
				calculateMinMaxIntervall();
				m_matrixWindow.repaint();
			}
		});

		/**
		*  ActionListener for the "previous" button
		*/
		prev.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(m_currentcolumn<=0){
						return;
					}
					switch(m_Switch){
						case 4: if(m_elementSelected){
									m_matrix.SwapColumns(m_currentcolumn,m_currentcolumn-1);
								}
						case 3:	m_currentcolumn--;
								break;
						case 5: m_currentcolumn--;
								break;
					}
					setTextFields();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "next" button
		*/
		next.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(m_currentcolumn>=m_matrix.columns-1){
						return;
					}
					switch(m_Switch){
						case 4: if(m_elementSelected){
									m_matrix.SwapColumns(m_currentcolumn,m_currentcolumn+1);
								}
						case 3:	m_currentcolumn++;
								break;
						case 5: m_currentcolumn++;
								break;
					}
					setTextFields();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "up" button
		*/
		up.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(m_currentrow<=0){
						return;
					}
					switch(m_Switch){
						case 2: if(m_elementSelected){
									m_matrix.SwapRows(m_currentrow,m_currentrow-1);
								}
						case 1:	m_currentrow--;
								break;
						case 5: m_currentrow--;
								break;
					}
					setTextFields();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "down" button
		*/
		down.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(m_currentrow>=m_matrix.rows-1){
						return;
					}
					switch(m_Switch){
						case 2: if(m_elementSelected){
									m_matrix.SwapRows(m_currentrow,m_currentrow+1);
								}
						case 1:	m_currentrow++;
								break;
						case 5: m_currentrow++;
								break;
					}
					setTextFields();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "next entry (>>)" button
		*/
		nextentry.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					skipToNextEntry();
					setTextFields();
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "select" button
		*/
		select.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					SelectRowColumnOrValue();
					setTextFields();
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "back/undo" button
		*/
		back.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(m_elementSelected){
						CancelAction();
					}
					if(history){
						UndoRedoAction();
						history=false;
						future=true;
					}
					setMatrixEditorDisplay();
					setTextFields();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "OK/Exchange/Redo" button
		*/
		ok.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					if(future){
						UndoRedoAction();
						history=true;
						future=false;
					}
					else{ switch(m_Switch){
							case 1: historyrow2=m_currentrow;
									if(historyrow1!=historyrow2){
										m_matrix.SwapRows(historyrow1,historyrow2);
										history=true;
										history_mode_dragged=false;
									}
									break;
							case 2:	historyrow2=m_currentrow;
									if(historyrow1!=historyrow2){
										history=true;
										history_mode_dragged=true;
									}
									break;
							case 3: historycolumn2=m_currentcolumn;
									if(historycolumn1!=historycolumn2){
										m_matrix.SwapColumns(historycolumn1,historycolumn2);
										history=true;
										history_mode_dragged=false;
									}
									break;
							case 4: historycolumn2=m_currentcolumn;
									if(historycolumn1!=historycolumn2){
										history=true;
										history_mode_dragged=true;
									}
									break;
							case 5: if(m_matrix.getValue(historyrow1,historycolumn1)!=valueold) history=true;
									break;
							}
					}
					m_elementSelected=false;
					setMatrixEditorDisplay();
					m_matrixWindow.repaint();
				}
		});

		/**
		*  ActionListener for the "Cancel" button - closes window and abandons all changes
		*/
		cancel.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					copy(matrixcopy,m_matrix);
					calculateMinMaxIntervall();
					mwcInt.removeModuleWindow(MatrixEditor.this);
					m_matrixEditorActive=false;
					m_matrixVisable=true;
					setMatrixToolDisplay();
					m_matrixWindow.repaint();
				}
			});
		/**
		*  ActionListener for the "Done" button - closes window and sets the matrix as valid DataModule
		*/
		done.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent ae){
					mwcInt.removeModuleWindow(MatrixEditor.this);
					m_matrixEditorActive=false;
					m_matrixVisable=true;
					setMatrixToolDisplay();
				}
		});

	}

	/**
	* method sets the Enabled-Flags of the window elements
    */
	public void setMatrixEditorDisplay(){
		mposition.setEnabled(!((m_Switch==5 && m_elementSelected)||(m_Switch==1 && m_elementSelected)));
		mtargetposition.setEnabled(!(m_Switch==1 && !m_elementSelected));
		nposition.setEnabled(!((m_Switch==5 && m_elementSelected)||(m_Switch==3 && m_elementSelected)));
		ntargetposition.setEnabled(!(m_Switch==3 && !m_elementSelected));
		value.setEnabled(m_Switch==5 && m_elementSelected);
		prev.setEnabled(!(m_Switch==5 && m_elementSelected));
		next.setEnabled(!(m_Switch==5 && m_elementSelected));
		up.setEnabled(!(m_Switch==5 && m_elementSelected));
		down.setEnabled(!(m_Switch==5 && m_elementSelected));
		select.setEnabled(!m_elementSelected);
		if(m_elementSelected){
			back.setText("Back");
			switch(m_Switch){
				case 1:
				case 3:	ok.setText("Exchange");
						break;
				case 2:
				case 4:
				case 5:	ok.setText("OK");
						break;
			}
		}
		if(history){
			back.setText("Undo");
		}
		back.setEnabled(m_elementSelected||history);
		if(future){
			ok.setText("Redo");
		}
		ok.setEnabled(m_elementSelected||future);
		modecombo.setEnabled(!m_elementSelected);
		cancel.setEnabled(!m_elementSelected);
		done.setEnabled(!m_elementSelected);
	}

	/**
	* method creates a copy of a matrix (original -> target)
	*/
	public void copy(VisAlgMatrix m, VisAlgMatrix mcopy){
		for(int i=0;i<m.rows;i++){
			for(int j=0;j<m.columns;j++){
				mcopy.setValue(i,j,m.getValue(i,j));
			}
		}
	}

	/**
     * method to skip to the next entry in the matrix
    */
	public void skipToNextEntry(){
		if(m_currentcolumn<m_matrix.columns-1){
				m_currentcolumn++;
		}
		else {
			m_currentcolumn=0;
			if(m_currentrow<m_matrix.rows-1){
				m_currentrow++;
			}
			else {
				m_currentrow=0;
			}
		}
		if(m_elementSelected){
			SelectRowColumnOrValue();
			}
	}

	/**
     * method sets the TextFields depending from the editor mode
    */
	public void setTextFields(){
		switch(m_Switch){
			case 1:	if(m_elementSelected){
						mtargetposition.setText(""+m_currentrow);
					}
					else{
					mposition.setText(""+m_currentrow);
					mtargetposition.setText("");
					}
					break;
			case 2: mposition.setText(""+m_currentrow);
					break;
			case 3:	if(m_elementSelected){
						ntargetposition.setText(""+m_currentcolumn);
					}
					else{
					nposition.setText(""+m_currentcolumn);
					ntargetposition.setText("");
					}
					break;
			case 4: nposition.setText(""+m_currentcolumn);
					break;
			case 5: mposition.setText(""+m_currentrow);
					nposition.setText(""+m_currentcolumn);
					value.setText(""+m_matrix.getValue(m_currentrow,m_currentcolumn));
					break;
			}
	}

	/**
     * method to "drag" rows
    */
	public void DragRow(int z1, int z2){
		if (z2<z1){
			for(int i=z1;i>z2;i--){
				m_matrix.SwapRows(i,i-1);
			}
		}
		else for(int i=z1;i<z2;i++){
			m_matrix.SwapRows(i,i+1);
		}
	}

	/**
     * method to "drag" columns
    */
	public void DragColumn(int s1, int s2){
		if (s2<s1){
			for(int i=s1;i>s2;i--){
				m_matrix.SwapColumns(i,i-1);
			}
		}
		else for(int i=s1;i<s2;i++){
			m_matrix.SwapColumns(i,i+1);
		}
	}

	/**
     * method to select a row column or a value (called by ActionListener "select", "nextentry"
     * and by a RETURN in TextField in Exchange-Mode)
    */
	public void SelectRowColumnOrValue(){
		switch(m_Switch){
			case 1:
			case 2: historyrow1=m_currentrow;
					historyrow2=-1;
					break;
			case 3:
			case 4: historycolumn1=m_currentcolumn;
					historycolumn2=-1;
					break;
			case 5: historyrow1=m_currentrow;
					historycolumn1=m_currentcolumn;
					valueold=m_matrix.getValue(m_currentrow,m_currentcolumn);
					break;
			}
		history=false;
		future=false;
		m_elementSelected=true;
	}

	/**
     * method to cancel actions (called by ActionListener "back")
    */
	public void CancelAction(){
		switch(m_Switch){
			case 2: if (m_elementSelected){
						DragRow(m_currentrow,historyrow1);
					}
			case 1:	m_currentrow=historyrow1;
					historyrow1=-1;
					break;

			case 4: if (m_elementSelected){
						DragColumn(m_currentcolumn,historycolumn1);
					}
			case 3:	m_currentcolumn=historycolumn1;
					historycolumn1=-1;
					break;
			case 5: m_matrix.setValue(historyrow1, historycolumn1, valueold);
					if(valuenew.doubleValue()==minimum||valuenew.doubleValue()==maximum
						||valueold.doubleValue()<minimum||valueold.doubleValue()>maximum)
						calculateMinMaxIntervall();
					break;
		}
		m_elementSelected=false;
		history=false;
		future=false;
	}

	/**
     * method to undo or redo the last action (called by the ActionListener "back" and "ok")
    */
	public void UndoRedoAction(){
		switch(m_Switch){
			case 1:	m_matrix.SwapRows(historyrow1,historyrow2);
					break;
			case 2:	if(history_mode_dragged){
						if(history){DragRow(historyrow2,historyrow1);}
						if(future){DragRow(historyrow1,historyrow2);}
					}
					break;
			case 3:	m_matrix.SwapColumns(historycolumn1,historycolumn2);
					break;
			case 4:	if(history_mode_dragged){
						if(history){DragColumn(historycolumn2,historycolumn1);}
						if(future){DragColumn(historycolumn1,historycolumn2);}
					}
					break;
			case 5: if(history){
						m_matrix.setValue(historyrow1,historycolumn1,valueold);
						if(valuenew.doubleValue()==minimum||valuenew.doubleValue()==maximum
						||valueold.doubleValue()<minimum||valueold.doubleValue()>maximum)
						calculateMinMaxIntervall();
						}
					if(future){
						m_matrix.setValue(historyrow1,historycolumn1,valuenew);
						if(valueold.doubleValue()==minimum||valueold.doubleValue()==maximum
						||valuenew.doubleValue()<minimum||valuenew.doubleValue()>maximum)
						calculateMinMaxIntervall();
						}
					break;
		}
	}

	/**
	* layout for "exchange rows" mode
	*/
	public void layout1(){

	//------Exchange rows i and j-------------------
		exrowPanel.add(rilabel);
		exrowPanel.add(mposition);
		mposition.setToolTipText("current row i");
		up.setToolTipText("up");
		exrowPanel.add(up);
		exrowPanel.add(rjlabel);
		mtargetposition.setToolTipText("current row j");
		exrowPanel.add(mtargetposition);
		down.setToolTipText("down");
		exrowPanel.add(down);

		card[0].add(exrowPanel);
		card[0].add(opPanel);

		m_contentPane.add(frame);
		validate();
		setMatrixEditorDisplay();
		setTextFields();
	}

	/**
	* layout for "move row" mode
	*/
	public void layout2() {

		//-----Move row i-------------------------
		moverowPanel.add(equalizer1);
		moverowPanel.add(rilabel);
		mposition.setToolTipText("current row");
		moverowPanel.add(mposition);
		moverowPanel.add(equalizer2);
		moverowPanel.add(equalizer3);
		up.setToolTipText("up");
		moverowPanel.add(up);
		down.setToolTipText("down");
		moverowPanel.add(down);
		moverowPanel.add(equalizer4);

		card[1].add(moverowPanel);
		card[1].add(opPanel);

		m_contentPane.add(frame);
		validate();
		setMatrixEditorDisplay();
		setTextFields();

	}

	/**
	* layout for "exchange columns" mode
	*/
	public void layout3(){

	//------Exchange columns k and l------------------
		excolPanel.add(cklabel);
		excolPanel.add(nposition);
		nposition.setToolTipText("current column k");
		prev.setToolTipText("previous");
		excolPanel.add(prev);
		excolPanel.add(cllabel);
		ntargetposition.setToolTipText("current column l");
		excolPanel.add(ntargetposition);
		next.setToolTipText("next");
		excolPanel.add(next);

		card[2].add(excolPanel);
		card[2].add(opPanel);

		m_contentPane.add(frame);
		validate();
		setMatrixEditorDisplay();
		setTextFields();

	}

	/**
	* layout for "move column" mode
	*/
	public void layout4() {

		//-----Move column k-------------------------
		movecolPanel.add(equalizer1);
		movecolPanel.add(cklabel);
		nposition.setToolTipText("current column");
		movecolPanel.add(nposition);
		movecolPanel.add(equalizer2);
		movecolPanel.add(equalizer3);
		prev.setToolTipText("previous");
		movecolPanel.add(prev);
		next.setToolTipText("next");
		movecolPanel.add(next);
		movecolPanel.add(equalizer4);

		card[3].add(movecolPanel);
		card[3].add(opPanel);

		m_contentPane.add(frame);
		validate();
		setMatrixEditorDisplay();
		setTextFields();
	}

	/**
	* layout for "modify entry" mode
	*/
	public void layout5() {

		entryPanel.add(rilabel);
		mposition.setToolTipText("current row");
		entryPanel.add(mposition);
		entryPanel.add(cklabel);
		nposition.setToolTipText("current column");
		entryPanel.add(nposition);
		entryPanel.add(valuelabel);
		value.setToolTipText("current value");
		entryPanel.add(value);
		entryPanel.add(equalizer1);
		nextentry.setToolTipText("skip to next entry");
		entryPanel.add(nextentry);
		prev.setToolTipText("previous");
		entryPanel.add(prev);
		next.setToolTipText("next");
		entryPanel.add(next);
		up.setToolTipText("up");
		entryPanel.add(up);
		down.setToolTipText("down");
		lay.setConstraints(down, constraints);
		entryPanel.add(down);

		card[4].add(entryPanel);
		card[4].add(opPanel);

		m_contentPane.add(frame);
		validate();
		setMatrixEditorDisplay();
		setTextFields();
	}

}// class MatrixEditor

/**
 * class MyMouseListener was added to integrate mouse functionality
*/
class MyMouseListener extends MouseAdapter{
	private MatrixWindow matrixW;
	private transient int mousecolumn, mouserow;
	public MyMouseListener(MatrixWindow m) {matrixW=m;}
	public void mousePressed(MouseEvent e) {
		matrixW.mouseX=e.getX();
		mousecolumn=(int)((float)(matrixW.mouseX-matrixW.border)/matrixW.rectangleWidth);
		if(mousecolumn<0||mousecolumn>=m_matrix.columns){
			return;
		}
		matrixW.mouseY=e.getY();
		mouserow=(int)((float)(matrixW.mouseY-matrixW.border)/matrixW.rectangleHeight);
		if(mouserow<0||mouserow>=m_matrix.rows){
			return;
		}
		if(m_elementSelected){
			switch(m_Switch){
			case 2:	m_matrixEditor.DragRow(m_currentrow,mouserow);
					break;
			case 4: m_matrixEditor.DragColumn(m_currentcolumn,mousecolumn);
					break;
			case 5: return;
			}
		}
		m_currentcolumn=mousecolumn;
		m_currentrow=mouserow;
		m_matrixEditor.setTextFields();
		m_matrixWindow.repaint();
	}
}//class MyMouseListener

    /**
     * The internal class ColorModeEditor provides the opportunity to create a new
     * Color Mode or to edit a Color mode
     */
    private class ColorModeEditor extends ModuleWindow{

	private transient JSlider slider;
	private transient JTextField point, red, green, blue, fileName;
	private transient JButton addPoint, colorChooser, removePoint, savePoint, cancel;
	private transient JList newlist;
	private transient DefaultListModel listenModel;

	private transient boolean isMin=false, isMax=false;

        public void moduleChanged(ModuleChangeEvent cEvt) {};

        public ColorModeEditor(String s, MatrixTool mv, Rectangle bounds,
			     ModuleWindowContainer mwc){

	    super(s, mv, bounds, mwc);
	    final ModuleWindowContainer mwcInt=mwc;

	    JPanel northPanel=new JPanel(new GridLayout(1,2,10,10));
	    northPanel.add(new JLabel("Color Mode Name:"));
	    northPanel.add(fileName=new JTextField());
	    m_contentPane.add(northPanel, BorderLayout.NORTH);
	    fileName.setEditable(m_newColorMode);

	    JPanel centerPanel=new JPanel(new GridLayout(2,1,5,5));
	    JPanel centerPanelUp=new JPanel(new BorderLayout());
	    JPanel panel1=new JPanel(new GridLayout(1,2,5,5));
	    JPanel panel2=new JPanel(new GridLayout(2,1,5,5));
	    panel1.add(new JLabel("Sampling Point:"));
	    panel1.add(point=new JTextField());
	    point.setText(""+(minimum+intervall/2));
	    panel2.add(panel1);
	    panel2.add(slider=new JSlider());
	    slider.setMaximum(10000);
	    slider.setValue(5000);
	    centerPanelUp.add(panel2,BorderLayout.NORTH);

	    panel1=new JPanel(new GridLayout(3,2,5,5));
	    panel1.add(new JLabel("Red (0 - 255):",JLabel.RIGHT));
	    panel1.add(red=new JTextField());
	    panel1.add(new JLabel("Green (0 - 255):",JLabel.RIGHT));
	    panel1.add(green=new JTextField());
	    panel1.add(new JLabel("Blue (0 - 255):",JLabel.RIGHT));
	    panel1.add(blue=new JTextField());
	    centerPanelUp.add(panel1, BorderLayout.CENTER);
	    panel1=new JPanel(new GridLayout(1,2,5,5));
	    panel1.add(colorChooser=new JButton("ColorChooser ..."));
	    panel1.add(addPoint=new JButton("Add"));
	    centerPanelUp.add(panel1, BorderLayout.SOUTH);
	    centerPanelUp.setBorder(new TitledBorder("Sampling Point Definition"));
	    centerPanel.add(centerPanelUp);

	    JPanel centerPanelDown=new JPanel(new BorderLayout());
	    listenModel=new DefaultListModel();
	    newlist=new JList(listenModel);
	    JScrollPane pane=new JScrollPane(newlist);
	    centerPanelDown.add(pane,BorderLayout.CENTER);
	    centerPanelDown.add(removePoint=new JButton("Remove"), BorderLayout.SOUTH);
	    centerPanelDown.setBorder(new TitledBorder("Sampling Points"));
	    centerPanel.add(centerPanelDown);
	    m_contentPane.add(centerPanel,BorderLayout.CENTER);

	    JPanel southPanel=new JPanel(new GridLayout(1,2,5,5));
	    southPanel.add(savePoint=new JButton("Save"));
	    southPanel.add(cancel=new JButton("Cancel"));
	    m_contentPane.add(southPanel,BorderLayout.SOUTH);

	    if(!m_newColorMode){
		for(int i=0;i<m_samplingPoints.length;i++)
		    listenModel.addElement(m_samplingPoints[i]);
		fileName.setText(m_colorModeName);
		isMin=true;
		isMax=true;
	    }

	    savePoint.setEnabled(isMin && isMax);
	    validate();

	    /**
	     * ActionListener for JButton "Save".
	    */
		savePoint.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    boolean ok=saveColorMode();
		    if(ok){
			mwcInt.removeModuleWindow(ColorModeEditor.this);
			m_colorModeEditorActive=false;
			m_matrixVisable=true;
			setMatrixToolDisplay();
		    }
		}
	    });

	    /**
	     * ActionListener for JButton "Cancel".
	    */
		cancel.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    mwcInt.removeModuleWindow(ColorModeEditor.this);
		    m_colorModeEditorActive=false;
			m_matrixVisable=true;
		    setMatrixToolDisplay();
		}
	    });

	    /**
	     * ActionListener for Button "Remove Sampling Point".
	    */
		removePoint.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    int[] indices=newlist.getSelectedIndices();
		    for(int i=indices.length-1;i>=0;i--)
			if(indices[i]>-1)
			    listenModel.removeElementAt(indices[i]);
		    boolean maxFound=false, minFound=false;
		    for(int i=0;i<listenModel.size();i++){
			if(((SamplingPoint)listenModel.get(i)).point==0) minFound=true;
			if(((SamplingPoint)listenModel.get(i)).point==10000) maxFound=true;
		    }
		    isMin=minFound;
		    isMax=maxFound;
		    savePoint.setEnabled(isMin && isMax);
		}
	    });

	    /**
	     * ActionListener for JButton "ColorChooser".
	    */
		colorChooser.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    Color c;
		    c=JColorChooser.showDialog(((Component)evt.getSource()),"ColorChooser",
					       Color.white);
		    if(c!=null){
			red.setText(""+c.getRed());
			green.setText(""+c.getGreen());
			blue.setText(""+c.getBlue());
		    }
		}
	    });

	    /**
		* ActionListener for JButton "Hinzufgen".
	    */
		addPoint.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    addSamplingPoint();
		    ;
		}
	    });

	    /**
		* ActionListener for JComponent Slider.
	    */
		slider.addChangeListener(new ChangeListener(){
		public void stateChanged(ChangeEvent evt) {
		    point.setText(""+(minimum+intervall/10000*(slider.getValue())));
		}
	    });

	    /**
	     * ActionListener for the TextField to display and insert
	     * a Sampling Point.
	     */
	    point.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent evt) {
		    double value;
		    try{
			value=Double.valueOf(point.getText()).doubleValue();
		    } catch(Exception e){
			return;
		    }
		    value=(value-minimum)/intervall*10000;
		    value=Math.max(0,value);
		    value=Math.min(value,10000);
		    slider.setValue((int)value);
		}
	    });
	}

	/**
	 * addSamplingPoint deals with the action that is launched
	 * when you click on the button with the same name. It reads
	 * the values from the four TextFields and writes them if no
	 * error occured into a new Sampling Point object and adds
	 * this one sorted into the list.
	 */
	private void addSamplingPoint() {
	    int tempred, tempgreen, tempblue, temppoint;
	    try{
		tempred=Integer.parseInt(red.getText());
		tempgreen=Integer.parseInt(green.getText());
		tempblue=Integer.parseInt(blue.getText());
	    }
	    catch(Exception e){return;}
	    if(tempred<0 || tempred>255) return;
	    if(tempgreen<0 || tempgreen>255) return;
	    if(tempblue<0 || tempblue>255) return;

	    temppoint=slider.getValue();
	    if(temppoint==0) isMin=true;
	    if(temppoint==10000) isMax=true;
	    SamplingPoint newSamplingPoint=
		new SamplingPoint(temppoint,
				  tempred,
				  tempgreen,
				  tempblue);
	    int index=0;
	    while((index<listenModel.getSize()) &&
		  (newSamplingPoint.compareTo(listenModel.get(index))>0))
		index++;

	    listenModel.insertElementAt(newSamplingPoint, index);
	    red.setText("");
	    green.setText("");
	    blue.setText("");
	    savePoint.setEnabled(isMin && isMax);
	}

	/**
	 * saveColorMode writes a SamplingPoint array generated from
	 * the list in a file named after the TextField entry and sets
	 * the array as the current valid Sampling Point array
	 */
	private boolean saveColorMode() {
	    String name;
	    Object[] array=listenModel.toArray();
	    SamplingPoint[] SamplingPointArray=new SamplingPoint[array.length];
	    for(int i=0;i<array.length;i++)
		SamplingPointArray[i]=(SamplingPoint)array[i];
	    if((name=fileName.getText()).equals("")){
		JOptionPane.showMessageDialog(null, "No File Name!",
					      "Save", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	    File file=new File("."+File.separator+"colorModes"+File.separator+name+".cm");
	    if(file.exists()&&m_newColorMode){
		JOptionPane.showMessageDialog(null, "File already exists!",
					      "Save", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	    try {
		ObjectOutputStream out=new ObjectOutputStream(
							      new FileOutputStream(file));
		out.writeObject(SamplingPointArray);
		out.close();
	    } catch (IOException e) {
		System.out.println(e);
		JOptionPane.showMessageDialog(null, "Error occured while saving file!",
					      "Save", JOptionPane.ERROR_MESSAGE);
		return false;
	    }
	    m_colorModeName=name;
	    m_samplingPoints=SamplingPointArray;
	    if(m_newColorMode)
		ComboBoxColorMode.addItem(name);
	    ComboBoxColorMode.setSelectedItem(name);
	    m_colorModeEditable=true;
	    setMatrixToolDisplay();
	    return true;
	}

    } // class ColorModeEditor

    /**
     * The class MatrixWindow opens a second window in which the matrix
     * is displayed as a color pattern
     */
    private class MatrixWindow extends ModuleWindow{
	private transient int panelHeight;
	private transient int panelWidth;
	private transient int border = 20;
	private transient int rectangleHeight, rectangleWidth;
	private transient int mouseX, mouseY;

        public void moduleChanged(ModuleChangeEvent cEvt) {};

        /**
	 * Constructor of the MatrixWindow.
	 */
	public MatrixWindow(String s, MatrixTool mv, Rectangle bounds,
			    ModuleWindowContainer mwc){
	    super(s, mv, bounds, mwc);
	}

	/**
	 * paintComponent is the overwritten method to draw the window.
	 * Depending from the height and width of the Panel and the number
	 * of rows and columns of the matrix the height and width of a single
	 * rectangle is calculated. Also every single value gets his color
	 * in this method.
	 * In the editor mode the blue/red rectangles are drawn and the
	 * MouseListener switched on
	 */
	public void paintComponent(Graphics g) {
	    super.paintComponent(g);
	    if(m_matrixVisable){
		double value, factor;
		int oberGrenze, colorR, colorG, colorB;
		panelHeight=getSize().height;
		panelWidth=getSize().width;
		rectangleHeight=Math.round(((float)(panelHeight-2*border))/m_matrix.rows);
		rectangleWidth=Math.round(((float)(panelWidth-2*border))/m_matrix.columns);
		for(int i=0;i<m_matrix.rows;i++)
		    for(int j=0;j<m_matrix.columns;j++){
			value=(m_matrix.getValue(i,j).doubleValue()-minimum)/intervall*10000;
			for(oberGrenze=1; (oberGrenze<m_samplingPoints.length-1) &&
				(value>=m_samplingPoints[oberGrenze].point); oberGrenze++);
			factor=(value-m_samplingPoints[oberGrenze-1].point)/
			    (m_samplingPoints[oberGrenze].point-
			     m_samplingPoints[oberGrenze-1].point);
			colorR=(int)(factor*m_samplingPoints[oberGrenze].R+
				     (1-factor)*m_samplingPoints[oberGrenze-1].R);
			colorG=(int)(factor*m_samplingPoints[oberGrenze].G+
				     (1-factor)*m_samplingPoints[oberGrenze-1].G);
			colorB=(int)(factor*m_samplingPoints[oberGrenze].B+
				     (1-factor)*m_samplingPoints[oberGrenze-1].B);
			int editorFactor;
			if(m_matrixEditorActive){
				editorFactor=1;
			}
			else editorFactor=0;
			g.setColor(new Color(colorR, colorG, colorB));
			g.fillRect(border+j*rectangleWidth, border+i*rectangleHeight,
				   rectangleWidth-1*editorFactor, rectangleHeight-1*editorFactor);
		    }
	    }
		if (m_matrixEditorActive){
			if(m_elementSelected){
				g.setColor(Color.red);
			}
			else g.setColor(Color.blue);
			MouseListener m1= new MyMouseListener(this);
			this.addMouseListener(m1);
			switch(m_Switch){
				case 1:	if(m_elementSelected){
							g.drawRect(border-1,m_matrixEditor.historyrow1*rectangleHeight+border-1,
							m_matrix.columns*rectangleWidth,rectangleHeight);
						}
				case 2: g.drawRect(border-1,m_currentrow*rectangleHeight+border-1,
						m_matrix.columns*rectangleWidth,rectangleHeight);
						break;
				case 3:	if(m_elementSelected){
							g.drawRect(m_matrixEditor.historycolumn1*rectangleWidth+border-1,border-1,
							rectangleWidth,m_matrix.rows*rectangleHeight);
						}
				case 4: g.drawRect(m_currentcolumn*rectangleWidth+border-1,border-1,
						rectangleWidth,m_matrix.rows*rectangleHeight);
						break;
				case 5: g.drawRect(m_currentcolumn*rectangleWidth+border-1,m_currentrow*rectangleHeight+border-1,
						rectangleWidth,rectangleHeight);
						break;
						}
		}
	}

    } // class MatrixWindow

    /**
     * This module does not react on change events. Warum nicht (koennte repainten) ?
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
        invalidate();
        repaint();
    };

    /**
     * Removes all open windows.
     */
    public void discardWindows() {
        if (m_matrixWindow instanceof MatrixWindow) {
            getModule().getModuleManager().getModuleWindowContainer().removeModuleWindow(m_matrixWindow);
        }
        if (m_matrixEditorActive) {
            getModule().getModuleManager().getModuleWindowContainer().removeModuleWindow(m_matrixEditor);
        }
        if (m_colorModeEditorActive) {
            getModule().getModuleManager().getModuleWindowContainer().removeModuleWindow(m_colorModeEditor);
        }

    }
} // class MatrixToolWindow