////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : MergeSortNichtRekursiv.java
//  Description      :
//  Author           : Babsi & Jochen
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Fri Feb 18
//  Last modified by : vigelius
//  Last modified on : Thu Dec 28 2001
//  Update count     : 1
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//  28.12.00    Vigelius        Angepasst an neue Variablennamen in Algorithm
//  03.01.01    Vigelius        Abbruchbedingung eingebaut
//  05.01.01    Vigelius        Code gemaess den Richtlinien dokumentiert und
//                              formatiert.
//  04.04.01    vigelius        Algorithm now uses AlgorithmWindow
//  17.05.01    vigelius        added the property concept
//  06.11.01    vigelius        Does now use VisAlgVector
//  07.11.01    vigelius        Does now support module changes
//
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import java.util.*;
import java.awt.event.*;
import java.awt.Color;

import java.lang.*;
import java.io.File;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.*;

import visalg.converter.*;

/**
 * This class implements the "Merge Sort" algorith. MergeSort provides the
 * possibility of sorting data sets, that are much bigger, than the available
 * workspace. The set of data is divide in two or more seperate sets, that are
 * sorted by a random algorithm. If all sets have been sorted, they have to be
 * merged again. For this, the first two elements of each set are compared, and
 * the smaller one will be written into the target file. If the end of one set
 * has been reached, the other set has just to be appended on the target file.
 * For this algorithm, it is just necessary to keep two datas in the workspace.
 *
 * The average as well as the maximum time of the MergeSort algorithm is
 * O(n log n).
 *
*/

public class MergeSort extends Algorithm {

    /**
     * @serial The data module for this algorithm.
     */
    private DataModule m_dataModule;

    /**
     * @serial The number of recursion loops executed up to now.
     */
    private int m_loopNumber;

    /**
     * @serial The actual size of the treated part.
     */
     private int m_partSize;

    /**
     * @serial Current position in the left part
     */
    private int m_left;

    private IndicatorBarProperty m_leftProperty =
        new IndicatorBarProperty("Left", new Color(0, 0, 255), -1, 1);


    /**
     * @serial Beginning of the current part
     */
    private int m_leftBeginning;

    /**
     * @serial End of the current part
     */
    private int m_right;

    private IndicatorBarProperty m_rightProperty =
        new IndicatorBarProperty("Right", new Color(128, 0, 0), -1, 2);

    /**
     * @serial Current number to be sorted.
     */
    private int m_current;

    private IndicatorBarProperty m_currentProperty =
        new IndicatorBarProperty("Current", new Color(255, 255, 255), -1, -1);

    /**
     * @serial Temporary result of the last comparison.
     */
    private int m_comparation;

    /**
     * @serial Indicates whether the left element is the only one.
     */
    private boolean m_leftTheOnlyElement;

    /**
     * @serial Indicates whether the left pointer is at the end of data.
     */
    private boolean m_leftAtEnd;

    /**
     * @serial Indicates if the left pointer is at the end minus loop counter.
     */
    private boolean m_leftAtEndMinusI;

    /**
     * @serial Indicates if the left pointer is at the end minus two times the
     * loop counter.
     */
    private boolean m_leftAtEndMinusII;

    /**
     * @serial Indicates if the current pointer at the right end of this part.
     */
    private boolean m_currentAtRight;

    /**
     * @serial Indicates if the left pointer is at the current position minus
     * one.
     */
    private boolean m_leftAtCurrentMinusOne;

    /**
     * @serial Indicates if the left pointer is at the current position.
     */
    private boolean m_leftAtCurrent;


    /**
    * Initialisation.
    * Sets the current project and does some init stuff. The superclass
    * constructor is called first.
    * @param p The current project.
    */
    public MergeSort(Project p) {
        super("MergeSort", p);

        m_done = false;
        // ??? Warum zweimal dieselbe Variable setzen ???
        m_commands = new Command[] {
            new PointerComparison(),
            new PointerSetting(),
            new ValueComparison(),
            new SwopValues(),
        };

                /*m_commands = new Command[] {
                        new SetPointer(),
                        new SetCurrent(),
                        new SetRight(),
                        new SetLeft(),
                        new SetCurrentAndRight(),
                        new SetPointerToEnd(),
                        new IsLeftTheOnlyElement(),
                        new IsLeftAtEnd(),
                        new IsLeftAtEndMinusI(),
                        new IsLeftAtEndMinusII(),
                        new IsLeftAtCurrentMinusOne(),
                        new IsLeftAtCurrent(),
                        new IsCurrentAtRight(),
                        new IncreaseLeftAndRight(),
                        new IncreaseCurrent(),
                        new CompareLeftWithCurrent(),
                        new CompareCurrentWithLeft(),
                        new SwopLeftAndCurrent(),
                        new SwopCurrentAndLeft(),

                };*/

            //Zeiger
            m_loopNumber = 1;
            m_partSize=1;
            m_left = 0;
            m_leftBeginning = 0;
            m_right = -1;
            m_current = -1;
            //Zwischenergebnisse
            m_comparation = 0;
            m_leftTheOnlyElement = false;
            //Anfangszustand
            setNextCommand(new IsLeftTheOnlyElement());

    }//MergeSort

    //ZUGRIFFSMETHODEN

    /**
     * Returns the current data module
     * @return Current data module.
     */
    public DataModule getDataModule() {
        return m_dataModule;
    }//getDataModule

    /**
     * Sets a new data module. It is first checked if the data module is valid
     * that means not null and of type VisAlgComparableData. If this is the case
     * it is set, the other modules are
     * notified by executing fireEvent().
     */
    public void setDataModule(DataModule dm) {
        super.setDataModule(dm);
        if (dm==null) return;
        Object o = dm.getData();
        if (o==null) return;
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector) o).isComparable()) return;

       m_dataModule = dm;

        reset();
        registerProperties();

        m_done = false;

        fireEvent();
    }//setDataModule

    /**
     * Registers all necessary properties.
     */
    public void registerProperties() {
        // Properties registrieren
        m_dataModule.addProperty(m_leftProperty);
        m_dataModule.addProperty(m_rightProperty);
        m_dataModule.addProperty(m_currentProperty);
   }
    /**
     * Calculates the size of the data.
     * @return Size of data.
     */
    public int getDataSize() {
        if (m_dataModule==null) return 0;

        Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector) o).isComparable()) {
                reset();
                return 0;
            }
        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

        return data.length;
    }//getDataSize

    /**
     * Returns a specific element.
     * @param index Index of the desired element.
     * @return The element if found, else null.
     */
    public VisAlgComparableData getDataElementAt(int index) {
        if (m_dataModule==null) return null;
        Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector) o).isComparable()) return null;
        return (VisAlgComparableData) ((VisAlgVector) o).getDataElementAt(index);
    }//getDataElementAt

    /**
     * Returns the number of loops executed up to now.
     * @return Number of executed loops
     */
    public int getLoopNumber() {
        return m_loopNumber;
    }//getLoopNumber

    /**
     * Returns the current position in the left part.
     * @return Position in the left part
     */
    public int getLeft() {
        return m_left;
    }//getLeft

    /**
     * Returns the position of the beginning of the left part.
     * @return Beginning Position
     */
    public int getLeftBeginning() {
        return m_leftBeginning;
    }//getLeftBeginning

    /**
     * Returns the position of the right element.
     * @return Position of right element
     */
    public int getRight() {
        return m_right;
    }//getRight

    /**
     * Returns the position of the current element.
     * @return Position of current element
     */
    public int getCurrent() {
        return m_current;
    }//getCurrent

    /**
     * Returns the result of the last comparison.
     * @return Result of the last comparison
     */
    protected int getComparation() {
        return m_comparation;
    }//getComparation

    /**
     * Gives back the state of the left element.
     * @return true, if the left element is the only one, false else
     */
    protected boolean getLeftTheOnlyElement() {
        return m_leftTheOnlyElement;
    }//getLeftTheOnlyElement

    /**
     * Gives back the state of the left element.
     * @return true, if the left element is at the end of data, else false
     */
    protected boolean getLeftAtEnd() {
        return m_leftAtEnd;
    }

    /**
     * Returns the state of the left element.
     * @return True, if the left pointer is at the end minus the
     * loop counter. False, if not.
     */
    protected boolean getLeftAtEndMinusI() {
        return m_leftAtEndMinusI;
    }//getLeftAtEndMinusI

    /**
     * Returns the state of the left element.
     * @return True, if the left pointer is at the end minus two times the
     * loop counter. False, if not.
     */
    protected boolean getLeftAtEndMinusII() {
        return m_leftAtEndMinusII;
    }//getLeftAtEndMinusII

    /**
     * Returns the state of the current element.
     * @return true, if the position of the current element equals the position
     * of the right element. False, if not.
     */
    protected boolean getCurrentAtRight() {
        return m_currentAtRight;
    }//getCurrentAtRight

    /**
     * Returns the state of the left element.
     * @return True, if the position of the left element equals the position of
     * the current element minus one. False, if not.
     */
    protected boolean getLeftAtCurrentMinusOne() {
        return m_leftAtCurrentMinusOne;
    }//getLeftAtCurrentMinusOne

    /**
     * Returns the state of the left element.
     * @return True, if the position of the left element equals the position of
     * the current element. False, if not.
     */
    protected boolean getLeftAtCurrent() {
        return m_leftAtCurrent;
    }//getLeftAtCurrent

    /**
     * This method will reset all pointer values.
     * All pointers are set to initial value. Then the next command will be set
     * to IsLeftTheOnlyElement.
     */
    protected void reset() {
        //Zeiger
        m_loopNumber = 1;
        m_left = 0;
        m_leftBeginning = 0;
        m_right = -1;
        m_current = -1;
        //Zwischenergebnisse
        m_comparation = 0;
        m_leftTheOnlyElement = false;
        //Anfangszustand
        setNextCommand(new IsLeftTheOnlyElement());
        m_done = false;
        m_time = 0;
        fireEvent();
    }//reset

    /**
     * This method is used to set the next command in the command list.
     * Up to now, it just calls the superclass method.
     * @param ac The next command to be executed.
     */
    protected void setNextCommand(Command ac) {
        super.setNextCommand(ac);
    }//setNextCommand

    /**
     * This sets the state of the algorithm.
     * @param b True, if algorithm has finished. If not, false.
     */
    protected void setDone(boolean b) {
        super.setDone(b);
    }//setDone


    //OPERATIONEN
    /**
     * This is a class, where all pointer comparison commands should be derived
     * from. It implements just the method toString.
     */
    private class PointerComparison extends AbstractCommand {

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich";
        }//toSring
    }//class PointerComparison

    /**
     * All pointer setting commands should be derived from this class. The only
     * implementation is the method toString.
     */
    private class PointerSetting extends AbstractCommand {
        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen";
        }//toString
    }//class PointerSetting

    /**
     * All value comparing commands should be derived from this class. The only
     * implementation is the method toString.
     */
    private class ValueComparison extends AbstractCommand {

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Wertevergleich";
        }//toString
    }//class ValueComparison

    /**
     * All value swapping commands should be derived from this class. The only
     * implementation is the method to String.
     */
    private class SwopValues extends AbstractCommand {

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Vertauschung";
        }
    }//class SwopValues


        //OPERATIONEN

    /**
     * This class checks, if the left element is the only one in the data.
     * It first examines the data on validity and resets the algorithm if this
     * is not the case. Otherwise it checks if there is just one element at all
     * which does not have to be sorted. If this is the case, the algorithm
     * state will be set to be done. If not, the next command will be
     * IsLeftAtEnd.
     * @see visalg.modules.MergeSort.IsLeftAtEnd
     */
    private class IsLeftTheOnlyElement extends PointerComparison {

        /**
         * The method executes the command described above.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }
            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < data.length-1) {
                m_leftTheOnlyElement = false;
                setNextCommand(new IsLeftAtEnd());
            } else {
                m_leftTheOnlyElement = true;
                setNextCommand(null);
                setDone(true);
            }
        }//doIt

        /**
         * This method returns the designation of this command as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_left - Einziges Element des Arrays)";
        }//toString
    }//class IsLeftTheOnlyElement

    /**
     * This class checks if the left element is at the end of data.
     * If this is the case the next command will be SetPointer. If not,
     * IsLeftAtEndMinusII will be executed next.
     * @see MergeSort.IsLeftAtEndMinusII
     * @see MergeSort.SetPointer
     */
    private class IsLeftAtEnd extends PointerComparison {

    /**
     * This method executes the command described in the class header.
     */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }
            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < data.length-1) {
                m_leftAtEnd = false;
                setNextCommand(new IsLeftAtEndMinusII());
            } else {
                m_leftAtEnd = true;
                setNextCommand(new SetPointer());
            }
        }//doIt

        public String toString() {
            return "Zeigervergleich (m_left - Ende des Arrays)";
        }//toString
    }//class IsLeftAtEnd

    /**
     * This class tests if the position of the left element equals the end
     * position minus two times the loop counter. If this is not the case, the
     * next command executed will be SetCurrentAndRight. If it is, the next
     * command will be IsLeftAtEndMinusI.
     * @see MergeSort.SetCurrentAndRight
     * @see MergeSort.IsLeftAtEndMinusI
     */
    private class IsLeftAtEndMinusII extends PointerComparison {
        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }
            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < data.length-m_loopNumber-m_loopNumber+1) {
                m_leftAtEndMinusII = false;
                setNextCommand(new SetCurrentAndRight());
            } else {
                m_leftAtEndMinusII = true;
                setNextCommand(new IsLeftAtEndMinusI());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_left - 2 I vom Ende des Arrays entfernt)";
        }
    }

    /**
     * This class checks if the position of the left element equals the end
     * position minus the loop counter. If this is the case, the next command
     * executed will be SetPointerToEnd. If not, the next command will be
     * SetRightToEnd.
     * @see MergeSort.SetRightToEnd
     * @see MergeSort.SetPointerToEnd
     */
    private class IsLeftAtEndMinusI extends PointerComparison {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }
            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < data.length-m_loopNumber-1) {
                m_leftAtEndMinusI = false;
                setNextCommand(new SetRightToEnd());
            } else {
                m_leftAtEndMinusI = true;
                setNextCommand(new SetPointerToEnd());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_left - I vom Ende des Arrays entfernt)";
        }//toString
    }//class IsLeftAtEndMinusI

    /**
     * This class contains the command, that is executed in the beginning of
     * a new loop. The loop counter will be increased and the other pointers
     * will be set accordingly. The next command will be set to
     * IsLeftAtEndMinusII. The condition of truncation is verified here.
     * @see MergeSort.IsLeftAtEndMinusII
     */
    private class SetPointer extends PointerSetting {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            m_left = 0;
            m_leftBeginning = m_left;
            m_loopNumber = m_loopNumber+m_loopNumber;
            // zwei hoch m_loopNumber
            m_partSize=m_partSize*2;
            // Ueberpruefen, ob 2 hoch m_loopNumber groesser als laenge
            if (m_partSize>getDataSize()) {
                setDone(true);
            }
            setNextCommand(new IsLeftAtEndMinusII());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (marker)";
        }//toString
    }//class SetPointer

    /**
     * This class sets the right pointer according to the left one and the
     * loop number. The next command executed will be CompareLeftWithCurrent
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class SetRight extends PointerSetting {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            m_right = m_left+m_loopNumber;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (marker)";
        }//toString
    }//class setRight

    /**
     * This class sets the current pointer and the right pointer accordingly.
     * The next command executed will be CompareLeftWithCurrent
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class SetCurrentAndRight extends PointerSetting {
        public void doIt() {
            m_current = m_left+m_loopNumber;
            m_right = m_left+m_loopNumber+m_loopNumber-1;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (marker)";
        }//toString
    }//class SetCurrentAndRight

    /**
     * This class sets the right pointer to the end of the data. The next
     * command executed will be CompareLeftWithCurrent.
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class SetRightToEnd extends PointerSetting {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            m_right = getDataSize()-1;
            m_current = m_left+m_loopNumber;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (marker)";
        }//toString
    }//class SetRightToEnd

    /**
     * This class set both the right pointer and the current pointer to the end
     * of data. The next command executed will be CompareLeftWithCurrent.
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class SetPointerToEnd extends PointerSetting {
        public void doIt() {
            m_right = getDataSize()-1;
            m_current = getDataSize()-1;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (marker)";
        }//toString
    }//class SetPointerToEnd

    /**
     * This class compares the left element with the current one. If the
     * current element is bigger than the left one the next command will be
     * IsLeftAtCurrentMinusOne. If not, SwopLeftAndCurrent will be executed.
     * @see MergeSort.IsLeftAtCurrentMinusOne
     * @see MergeSort.SwopLeftAndCurrent
     */
    private class CompareLeftWithCurrent extends ValueComparison {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            m_comparation = data[getLeft()].compareTo(data[getCurrent()]);
            if (m_comparation > 0) {
                setNextCommand(new IsLeftAtCurrentMinusOne());
            } else {
                setNextCommand(new SwopLeftAndCurrent());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Wertevergleich (m_left - m_current)";
        }//toString
    }//class CompareLeftWithCurrent

    /**
     * This class compares the current element with the left one. If the current
     * element is bigger than the left one, the next command will be SetLeft.
     * If not, SweopCurrentAndLeft will be executed.
     * @see MergeSort.SwopCurrentAndLeft
     * @see MergeSort.SetLeft
     */
    private class CompareCurrentWithLeft extends ValueComparison {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            m_comparation = data[getLeft()].compareTo(data[getCurrent()]);
            if (m_comparation > 0) {
                setNextCommand(new SetLeft());
            } else {
                setNextCommand(new SwopCurrentAndLeft());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Wertevergleich (m_left - m_current)";
        }//toString
    }//class CompareCurrentWithLeft

    /**
     * This class checks if the position of the current element equals the
     * position of the right one. If this is the case, SetCurrent will be
     * executed. If not, the next command will be set to IncreaseCurrent.
     * @see MergeSort.IncreaseCurrent
     * @see MergeSort.SetCurrent
     */
    private class IsCurrentAtRight extends PointerComparison {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getCurrent() < getRight()) {
                m_currentAtRight = false;
                setNextCommand(new IncreaseCurrent());
            } else {
                m_currentAtRight = true;
                setNextCommand(new SetCurrent());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_current - m_right)";
        }//toString
    }//class IsCurrentAtRight

    /**
     * This class checks if the position of the left element equals the
     * position of the current one. If this is the case, the next command
     * executed will be SetLeft. If not, IncreaseLeft will be executed next.
     * @see MergeSort.SetLeft
     * @see MergeSort.IncreaseLeft
     */
    private class IsLeftAtCurrentMinusOne extends PointerComparison {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < getCurrent()-1) {
                m_leftAtCurrentMinusOne = false;
                setNextCommand(new IncreaseLeft());
            } else {
                m_leftAtCurrentMinusOne = true;
                setNextCommand(new SetLeft());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_current - m_right)";
        }//toString
    }//class IsLeftAtCurrentMinusOne

    /**
     * This class checks if the position of the left element equals the position
     * of the right one. If this is the case, IsCurrentAtRight will be executed.
     * If not, the next command will be IncreaseLeft.
     * @see MergeSort.IncreaseLeft
     * @see MergeSort.IsCurrentAtRight
     */
    private class IsLeftAtCurrent extends PointerComparison {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }
            VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

            if (getLeft() < getCurrent()-1) {
                m_leftAtCurrent = false;
                setNextCommand(new IncreaseLeft());
            } else {
                m_leftAtCurrent = true;
                setNextCommand(new IsCurrentAtRight());
            }
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeigervergleich (m_left - m_current)";
        }//toString
    }//class IsLeftAtCurrent


    /**
     * This class increases the pointer to the current element. Also, the
     * pointer to the left element will be set to the beginning of the left
     * part. The next command executed will be CompareLeftWithCurrent
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class IncreaseCurrent extends PointerSetting {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            m_current += 1;
            m_left = m_leftBeginning;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (m_current)";
        }//toString
    }//class IncreaseCurrent

    /**
     * This class sets the position of the current element accordingly. Then
     * the command IsLeftAtCurrentMinusOne will be executed next.
     * @see MergeSort.IsLeftAtCurrentMinusOne
     */
    private class SetCurrent extends PointerSetting {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            m_current = m_leftBeginning+m_loopNumber;
            setNextCommand(new IsLeftAtCurrentMinusOne());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (m_current)";
        }//toString
    }//SetCurrent

    /**
     * This class sets the left pointer to the next part. The next command
     * will be IsLeftAtEnd.
     * @see MergeSort.IsLeftAtEnd
     */
    private class SetLeft extends PointerSetting {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            m_left = m_right+1;
            m_leftBeginning = m_left;
            setNextCommand(new IsLeftAtEnd());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (m_left)";
        }//toString
    }//class SetLeft

    /**
     * This class increases the position of the left element. The next command
     * will be CompareLeftWithCurrent
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class IncreaseLeft extends PointerSetting {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            m_left += 1;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (left)";
        }//toString
    }//class IncreaseLeft

    /**
     * This class increases both the position of the left element and the
     * position of the right one. The next command will be
     * CompareLeftWithCurrent.
     * @see MergeSort.CompareLeftWithCurrent
     */
    private class IncreaseLeftAndRight extends PointerSetting {

        /**
         * This method executes the command described in the class header.
         */
        public void doIt() {
            m_left = m_right+1;
            m_current = m_left+m_loopNumber;
            m_right = m_left+m_loopNumber;
            setNextCommand(new CompareLeftWithCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Zeiger setzen (m_current)";
        }//toString
    }//class IncreaseLeftAndRight

    /**
     * This class swaps the left element and the current one. The next command
     * executed will be IsLeftAtCurrent.
     * @see MergeSort.IsLeftAtCurrent
     */
    private class SwopLeftAndCurrent extends SwopValues {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }

            VisAlgVector v = (VisAlgVector) o;

            v.swapValues(getLeft(),getCurrent());

            setNextCommand(new IsLeftAtCurrent());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Vertauschung (left - m_current)";
        }//toString
    }//class SwopLeftAndCurrent

    /**
     * This class swaps the current element and the left one. The next command
     * executed will be IsCurrentAtRight.
     * @see MergeSort.IsCurrentAtRight
     */
    private class SwopCurrentAndLeft extends SwopValues {

        /**
         * This method executes the command as described in the class header.
         */
        public void doIt() {
            //Fehlerhafte Konstellation abfangen...
            if (m_dataModule==null) {
                reset();
                return;
            }

            Object o = m_dataModule.getData();
            if (!(o instanceof VisAlgVector)
                ||!((VisAlgVector) o).isComparable()) {
                reset();
                return;
            }

            VisAlgVector v = (VisAlgVector) o;

            v.swapValues(getLeft(), getCurrent());

            setNextCommand(new IsCurrentAtRight());
        }//doIt

        /**
         * This method returns its designation as plain text.
         * @return A string containing the command name.
         */
        public String toString() {
            return "Vertauschung (left - m_current)";
        }//toString
    }//class SwopCurrentAndLeft

    /**
     * This method is called by the module manager. It verifies the validity of
     * the current data module and if this is not given, the algorithm will be
     * reset. The method of the superclass is called at the end and thus all
     * connected modules are notified.
     */
    public void step() {
        if ((m_dataModule==null) ||
            (m_dataModule.getData()==null) ||
            !(m_dataModule.getData() instanceof VisAlgVector)
            || !((VisAlgVector) m_dataModule.getData()).isComparable()) {
                reset();
        }

        //Abbruch-Bedingung:
        if (m_done) return;

        //Status abfragen

        // ??? Was soll das ???
        // Auf alle Faelle bricht der Algorithmus nicht ab !

        /*switch(myNextOperation) {

                        case 0: isLeftAtEndMinusI();
                                break;
                        case 1: setPointer();
                                break;
                        case 2: compareLeftWithCurrent();
                                break;
                        case 3: isCurrentAtRight();
                                break;
                        case 4: increaseCurrent();
                                break;
                        case 5: isLeftAtCurrentMinusOne();
                                break;
                        case 6: setCurrent();
                                break;
                        case 7: increaseLeft();
                                break;
                        case 8: isRightAtEnd();
                                break;
                        case 9: increaseLeftAndRight();
                                break;
                        case 10: swopLeftAndCurrent();
                                break;
                        case 11: setPointerToEnd();
                                break;
                        default://Algorithmus-Ende
                                m_done = true;
                                break;
                }*/
            super.step();
    }//step

    /**
     * This method creates a new element of type MergeSortWindow.
     * @see MergeSortWindow
    */
    public void newModuleWindow() {
        try {
            m_moduleWindow = new MergeSortWindow(m_name, this,
                m_moduleWindowBounds, m_moduleManager.getModuleWindowContainer());
            }
        catch (Exception ex) {

        }

        super.newModuleWindow();
            /*try {
                m_moduleWindow = new MergeSortWindow(m_name, this,
                        m_moduleWindowBounds,
                        m_moduleManager.getModuleWindowContainer());
                } catch (Exception e) {}*/
    }//newModuleWindow

    public void setProperties() {
        m_leftProperty.setIndicatorPosition(m_left);
        m_rightProperty.setIndicatorPosition(m_right);
        m_currentProperty.setIndicatorPosition(m_current);
        if (m_dataModule != null) {
            m_dataModule.propertyChanged();
        }
    }

    public void moduleChanged(ModuleChangeEvent evt) {
        reset();
        registerProperties();
        setProperties();
    }//moduleChanged(ModuleChangeEvent)
}//class MergeSort