////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TuringMachineWindow.java
//  Description      : Visualization of module TuringMachine
//  Author           : Heiner Violet
//  Email            : hviolet@ix.urz.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : 30.09.01
//  Last modified by :
//  Last modified on :
//  Update count     : V 1.0
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.basics.Module;
import visalg.basics.ModuleListener;
import visalg.basics.*;
import visalg.graphics.AlgorithmWindow;
import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

/*
 * This class provides the Window belonging to the TuringMachine
 * module. A data module for the Turingmachine (i.e. a Turing
 * Machine program) can be selected.
 * @see visalg.modules.TuringMachine
 */
public class TuringMachineWindow extends AlgorithmWindow {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     *
     */
    private transient JLabel m_commandTesterLabel;

    /**
     *
     */
    private transient TuringMachine m_turingMachine;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Einteilung des Hauptfensters, Initialisierung
     */
    public TuringMachineWindow(String s, TuringMachine tm, Rectangle bounds,
				ModuleWindowContainer mwc) {
	super(s, tm, bounds, mwc);
	m_turingMachine = tm;

	//Initialisierung der (lokalen) Variablen
	m_commandTesterLabel = new JLabel("last command: " + m_turingMachine.getLastCommand());
	m_contentPane.add(m_commandTesterLabel);
	revalidate();
    }


    // -------- from interface ModuleListener  -------------

    public void moduleChanged(ModuleChangeEvent cEvt) {
    super.moduleChanged(cEvt);
	m_commandTesterLabel.setText("last Command: " + m_turingMachine.getLastCommand());
	invalidate();
	repaint();
    }


} // class TuringMachineWindow
