////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : Intervalconfiguration.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.modules.tspGraphViewerWindow;


import visalg.types.VisAlgGraph.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * This class opens a window and enables the user to enter the number of
 * intervals to be configured.
 */
public class Intervalconfiguration extends JFrame
                                   implements ActionListener, KeyListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     *
     */
    private Object m_visAlgGraph;
    private JTextField m_number;
    private JButton m_okButton;
    private IntervalconfigurationFrame m_icf;
    private TSPGraphViewerWindow m_tgvw;


    // -----------------------------------------------------
    // ------- M e t h o d s  ( p u b l i c ) --------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public Intervalconfiguration(Object obj, TSPGraphViewerWindow t,
                                 IntervalconfigurationFrame icf) {
        m_visAlgGraph = obj;
        m_tgvw        = t;
        m_icf         = icf;

        m_tgvw.setIntervalconfiguration(this);

        setTitle("Number of intervals");

        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension d = tk.getScreenSize();
        int screenHeight = d.height;
        int screenWidth  = d.width;
        setSize(200, 100);
        setLocation( (screenWidth / 2) - 100, (screenHeight / 2) - 50 );
        setResizable(false);

	addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    dispose();
                    m_icf.show();
                }//end public void windowClosing(WindowEvent e)
        });//end WindowAdapter()

        addKeyListener(this);

	Container contentPane = getContentPane();
	GridBagLayout gbl = new GridBagLayout();
	contentPane.setLayout(gbl);

	JLabel label     = new JLabel("Please enter number of intervals:");
	m_number         = new JTextField(5);

	m_okButton = new JButton("OK");
	m_okButton.addActionListener(this);

	GridBagConstraints gbc = new GridBagConstraints();
	gbc.fill    = GridBagConstraints.NONE;
	gbc.anchor  = GridBagConstraints.WEST;
	gbc.weightx = 100;
	gbc.weighty = 100;
	add(label, gbc, 0, 0, 2, 1);
	gbc.anchor  = GridBagConstraints.CENTER;
	add(m_number, gbc, 0, 1, 1, 1);
	add(m_okButton, gbc, 1, 2, 1, 1);
    }//end public Intervalconfiguration()

    /**
     * This function adds a component into the frame using a GridBagLayout.
     * @param c   Component to be added.
     * @param g   GridBagConstraints.
     * @param x   Used to configure the GridBagConstraints.
     * @param y   Used to configure the GridBagConstraints.
     * @param w   Used to configure the GridBagConstraints.
     * @param h   Used to configure the GridBagConstraints.
     */
    public void add(Component c, GridBagConstraints g,
                    int x, int y, int w, int h) {
        g.gridx      = x;
	g.gridy      = y;
	g.gridwidth  = w;
	g.gridheight = h;
	getContentPane().add(c, g);
    }//end public void add(Component c, GridBagConstraints g,
     //                    int x, int y, int w, int h)

    /**
     * This function returns the parent of this class.
     *
     * @return m_icf Parent
     */
    public IntervalconfigurationFrame getParentOfIC() {
        return m_icf;
    }//end public IntervalconfigurationFrame getParent()


    // ------- from interface KeyListener ------------------

    /**
     * This function checks whether the ENTER- or the ESCAPE- key was pressed
     * or not.
     * @param evt   KeyEvent to be checked.
     */
    public void keyPressed(KeyEvent evt) {
        int keyCode = evt.getKeyCode();

	if (keyCode == KeyEvent.VK_ENTER) {
            try {
                if (Integer.parseInt(m_number.getText().trim()) == 0) {
                    JOptionPane op = new JOptionPane();
		    op.showMessageDialog(this,
                                         "Invalid input! Only unsigned integers allowed.",
		                         "Error",
                                         JOptionPane.WARNING_MESSAGE);
		}//end if (Integer.parseInt(number.getText().trim()) == 0)
		else {
                    if (m_visAlgGraph instanceof VisAlgGraph) {
		        JFrame icf = new IntervalconfigurationFrame(Integer.parseInt(m_number.getText().trim()),
                                                                    (VisAlgGraph) m_visAlgGraph, this, m_tgvw);
		        dispose();
                        icf.show();
                    }//end if (visAlgGraph instanceof VisAlgGraph)
                    else {
                        JOptionPane op = new JOptionPane();
		        op.showMessageDialog(this,
                                             "No VisAlgGraph Object available!",
		                             "Error",
                                             JOptionPane.WARNING_MESSAGE);
                    }//end else
		}//end else
	    }//end try
	    catch (Exception e) {
                JOptionPane op = new JOptionPane();
		op.showMessageDialog(this,
                                     "Invalid input! Only unsigned integers allowed.",
		                     "Error",
                                     JOptionPane.WARNING_MESSAGE);
	    }//end catch
	}//end if (keyCode == KeyEvent.VK_ENTER)

        else if (keyCode == KeyEvent.VK_ESCAPE) {
            dispose();
            m_icf.show();
	}//end else if (keyCode == KeyEvent.VK_ESCAPE)
    }//end public void keyPressed(KeyEvent evt)

    /**
     * Dud- method
     */
    public void keyReleased(KeyEvent evt) {
    }

    /**
     * Dud- method
     */
    public void keyTyped(KeyEvent evt) {
    }


    // ------- from interface ActionListener ---------------

    /**
     * This function checks whether the OK- Button was pressed or not.
     * @param evt   ActionEvent to be checked.
     */
    public void actionPerformed(ActionEvent evt) {
        try {
            if (evt.getSource() == m_okButton) {
                if (Integer.parseInt(m_number.getText().trim()) == 0) {
                    JOptionPane op = new JOptionPane();
		    op.showMessageDialog(this,
                                         "Invalid input! Only unsigned integers allowed.",
		                         "Error",
                                         JOptionPane.WARNING_MESSAGE);
		}//end if (Integer.parseInt(number.getText().trim()) == 0)
		else {
                    if (m_visAlgGraph instanceof VisAlgGraph) {
                        JFrame icf = new IntervalconfigurationFrame(Integer.parseInt(m_number.getText().trim()),
                                                                    (VisAlgGraph) m_visAlgGraph, this, m_tgvw);
		        dispose();
                        icf.show();
                    }//end if (visAlgGraph instanceof VisAlgGraph)
                    else {
                        JOptionPane op = new JOptionPane();
		        op.showMessageDialog(this,
                                             "No VisAlgGraph Object available!",
		                             "Error",
                                             JOptionPane.WARNING_MESSAGE);
                    }//end else
		}//end else
	    }//end if (evt.getSource() == okButton)
	}//end try
	catch (Exception e) {
            JOptionPane op = new JOptionPane();
	    op.showMessageDialog(this,
                                 "Invalid input! Only unsigned integers allowed.",
	                         "Error",
                                 JOptionPane.WARNING_MESSAGE);
	}//end catch
    }//end public void actionPerformed(ActionEvent evt)

}//end public class Intervalconfiguration extends JFrame
 //                                       implements ActionListener, KeyListener