////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : IntervalconfigurationFrame.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.modules.tspGraphViewerWindow;


import visalg.types.VisAlgGraph.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * This class opens a window for the user to link userdefined intervals with
 * different colors. Several tests are made in order to assure the correctness
 * of the given intervals.
 */
public class IntervalconfigurationFrame extends JFrame
                                        implements ActionListener, KeyListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     *
     */
    private Object m_parent;
    private VisAlgGraph m_visAlgGraph;
    private JPanel m_panel;
    private JTextField[] m_from;
    private JTextField[] m_to;
    private JButton[] m_colorButton;
    private JLabel[] m_number;
    private JPanel[] m_showColor;
    private JButton m_okButton;
    private JButton m_cancelButton;
    private JButton m_changeButton;
    private JCheckBox m_left;
    private JCheckBox m_right;
    private VisAlgInterval[] m_interval;
    private int m_numberOfIntervals;
    private boolean m_noProblem;
    private Color[] m_tempColor;
    private TSPGraphViewerWindow m_tgvw;


    // -----------------------------------------------------
    // ------- M e t h o d s  ( p u b l i c ) --------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public IntervalconfigurationFrame(int n, VisAlgGraph vag,
                                      Object p, TSPGraphViewerWindow t) {
        m_numberOfIntervals = n;
        m_visAlgGraph = vag;
        m_parent = p;
        m_tempColor = new Color[n];
        m_tgvw = t;

        m_tgvw.setIntervalconfigurationFrame(this);

	setTitle("Intervalmodification");

        if (m_parent instanceof Intervalconfiguration) {
            setTitle("Intervalconfiguration");
        } // if (m_parent instanceof Intervalconfiguration)

        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension d = tk.getScreenSize();
        int screenHeight = d.height;
        int screenWidth  = d.width;
	setSize(400, 200);
	setLocation( (screenWidth / 2) - 200, (screenHeight / 2) - 100);
	setResizable(false);

	addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    if (m_parent instanceof Intervalconfiguration) {
                        Intervalconfiguration tmp = (Intervalconfiguration) m_parent;
                        dispose();
                        tmp.show();
                    }//end if (m_parent instanceof Intervalconfiguration)
                    else {
                        dispose();
                    }//end else
		}//end public void windowClosing(WindowEvent e)
        });//end WindowAdapter()

        addComponentListener(new ComponentAdapter() {
                public void componentShown(ComponentEvent e) {
                    requestFocus();

                    // initializes the temporary Color[]- object with black
                    if (m_parent instanceof Intervalconfiguration) {
                        for (int i = 0; i < m_numberOfIntervals; i++) {
                            m_tempColor[i] = new Color(0, 0, 0);
                            m_showColor[i].setBackground(m_tempColor[i]);
                        } // for (int i = 0; i < m_numberOfIntervals; i++)
                    } //end if (m_parent instanceof Intervalconfiguration)
                    else {
                        VisAlgInterval[] vai = m_tgvw.getIntervals();

                        for (int i = 0; i < m_numberOfIntervals; i++) {
                            m_tempColor[i] = new Color(vai[i].getColor().getRGB());
                            m_showColor[i].setBackground(m_tempColor[i]);
                        } // for (int i = 0; i < m_numberOfIntervals; i++)
                    } //end else
                }//end public void componentShown(ComponentEvent e)
        });//end ComponentAdapter()

	addKeyListener(this);

	GridBagLayout gbl = new GridBagLayout();
	m_panel = new JPanel();
	m_panel.setLayout(gbl);

	JLabel fromlabel  = new JLabel("From:");
	JLabel tolabel    = new JLabel("To:");
        JLabel colorlabel = new JLabel("Actual color:");

	m_from           = new JTextField[m_numberOfIntervals];
	m_to             = new JTextField[m_numberOfIntervals];
	m_colorButton    = new JButton[m_numberOfIntervals];
        m_number         = new JLabel[m_numberOfIntervals];
	m_interval       = new VisAlgInterval[m_numberOfIntervals];
        m_showColor      = new JPanel[m_numberOfIntervals];

	for (int i = 0; i < m_numberOfIntervals; i++) {
            m_from[i] = new JTextField(5);
	    m_to[i]   = new JTextField(5);

	    m_colorButton[i] = new JButton("Color...");
	    m_colorButton[i].addActionListener(this);

            m_number[i] = new JLabel( (i + 1) + "." );

            if (m_parent instanceof Intervalconfiguration) {
                m_interval[i] = new VisAlgInterval(0, 0, Color.black);
            } //end if (m_parent instanceof Intervalconfiguration)
            else {
                VisAlgInterval[] vai = m_tgvw.getIntervals();
                m_interval[i] = new VisAlgInterval(0, 0, new Color(vai[i].getColor().getRGB()));
            } //end else

            m_showColor[i]   = new JPanel();
	}//end for (int i = 0; i < m_numberOfIntervals; i++)

	m_okButton     = new JButton("OK");
	m_okButton.addActionListener(this);

        m_cancelButton = new JButton("Cancel");
        m_cancelButton.addActionListener(this);

	m_left   = new JCheckBox("left open");
	m_right  = new JCheckBox("right open");

	GridBagConstraints gbc = new GridBagConstraints();
	gbc.fill    = GridBagConstraints.NONE;
	gbc.anchor  = GridBagConstraints.CENTER;
	gbc.weightx = 100;
	gbc.weighty = 100;

	add(m_panel, fromlabel, gbc, 1, 0, 1, 1);
	add(m_panel, tolabel, gbc, 2, 0, 1, 1);
        add(m_panel, colorlabel, gbc, 3, 0, 2, 1);

	for (int i = 0; i < m_numberOfIntervals; i++) {
            if (!(m_parent instanceof Intervalconfiguration)) {
                m_from[i].setText("" + m_visAlgGraph.getIntervalAt(i).getLowerLimit());
                m_to[i].setText("" + m_visAlgGraph.getIntervalAt(i).getUpperLimit());
            } // if (m_parent instanceof Intervalconfiguration)

            add(m_panel, m_number[i], gbc, 0, (i + 1), 1, 1);
            add(m_panel, m_from[i], gbc, 1, (i + 1), 1, 1);
            add(m_panel, m_to[i], gbc, 2, (i + 1), 1, 1);
            add(m_panel, m_showColor[i], gbc, 3, (i + 1), 1, 1);
            add(m_panel, m_colorButton[i], gbc, 4, (i + 1), 1, 1);
	}//end for (int i = 0; i < m_numberOfIntervals; i++)

        add(m_panel, m_okButton, gbc, 4, (m_numberOfIntervals + 1), 1, 1);

        if (!(m_parent instanceof Intervalconfiguration)) {
            m_left.setSelected(m_visAlgGraph.isLeftOpen());
            m_right.setSelected(m_visAlgGraph.isRightOpen());
        } // if (m_parent instanceof Intervalconfiguration)

	add(m_panel, m_left, gbc, 1, (m_numberOfIntervals + 1), 1, 1);
	add(m_panel, m_right, gbc, 2, (m_numberOfIntervals + 1), 1, 1);
        add(m_panel, m_cancelButton, gbc, 4, (m_numberOfIntervals + 2), 1, 1);

        if (!(m_parent instanceof Intervalconfiguration)) {
            m_changeButton = new JButton("New intervals...");
            m_changeButton.addActionListener(this);

            add(m_panel, m_changeButton, gbc, 0, (m_numberOfIntervals + 2), 4, 1);
        } // if (m_parent instanceof Intervalconfiguration)

	JScrollPane sp = new JScrollPane(m_panel);
	sp.getVerticalScrollBar().setUnitIncrement(15);
	Container contentPane = getContentPane();
	contentPane.add(sp, "Center");
    }//end public IntervalconfigurationFrame(int n, VisAlgGraph vag)

    /**
     * This function adds a component into the panel using a GridBagLayout.
     * @param p   Panel, into which the component will be added.
     * @param c   Component to be added.
     * @param g   GridBagConstraints.
     * @param x   Used to configure the GridBagConstraints.
     * @param y   Used to configure the GridBagConstraints.
     * @param w   Used to configure the GridBagConstraints.
     * @param h   Used to configure the GridBagConstraints.
     */
    public void add(JPanel p, Component c, GridBagConstraints g,
                    int x, int y, int w, int h) {
        g.gridx      = x;
	g.gridy      = y;
	g.gridwidth  = w;
	g.gridheight = h;
	p.add(c, g);
    }//end public void add(JPanel p, Component c, GridBagConstraints g,
     //                    int x, int y, int w, int h)

    /**
     * This function sets the m_parent- variable.
     * @param obj   Parent component
     */
    public void setParent(Object obj) {
        m_parent = obj;
    }//end public void setParent(Object obj)


    // ------- from interface KeyListener ------------------

    /**
     * This function checks whether the ENTER- or the ESCAPE- key was pressed
     * or not.
     * 'ENTER'- key calls several tests in order to assure the correctness
     * of the given intervals. 'ESCAPE'- key calls dispose().
     * @param evt   KeyEvent.
     */
    public void keyPressed(KeyEvent evt) {
        int keyCode = evt.getKeyCode();

	if (keyCode == KeyEvent.VK_ENTER) {
            try {
                //Compact interval
		if (    (m_left.isSelected() == false)
                        && (m_right.isSelected() == false) ) {
                    m_noProblem = true;

		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        double f = Double.parseDouble(m_from[i].getText().trim());
                        double t = Double.parseDouble(m_to[i].getText().trim());

                        if (f > t || (f < 0 || t > 1 )) {
                            JOptionPane op = new JOptionPane();
			    op.showMessageDialog(this,
                                                 "Invalid interval- limits! ("
                                                 + (i + 1) + ")",
                                                 "Error",
			                         JOptionPane.WARNING_MESSAGE);
			    return;
			}//end if (f > t || (f < 0 || t > 1 ))
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)

		    intervalcheck:
		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        for (int j = 0; j < m_numberOfIntervals; j++) {
                            if (j != i) {
                                double fI = Double.parseDouble(m_from[i].getText().trim());
                                double fJ = Double.parseDouble(m_from[j].getText().trim());
                                double tJ = Double.parseDouble(m_to[j].getText().trim());

                                if ((fI >= fJ) && (fI <= tJ)) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Interval- intersection! ("
                                                         + (i + 1) + "," + (j + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
                                    m_noProblem = false;
				    break intervalcheck;
				}//end if ((fI >= fJ) && (fI <= tJ))
			    }//end if (j != i)
			}//end for (int j = 0; j < m_numberOfIntervals; j++)
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)
		}//end if (    (m_left.isSelected() == false)
                 //            && (m_right.isSelected() == false))

		//Open or semi- open interval
		else if (    ((m_left.isSelected() == true) && (m_right.isSelected() == false))
		             || ((m_left.isSelected() == false) && (m_right.isSelected() == true))
		             || ((m_left.isSelected() == true) && (m_right.isSelected() == true)) ) {

                    //Open and semi- open intervals are both covered by the
		    //following code. But the differentiation is important
		    //for the colorize- method.

		    m_noProblem = true;

		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        double f = Double.parseDouble(m_from[i].getText().trim());
                        double t = Double.parseDouble(m_to[i].getText().trim());

                        if (f > t || (f < 0 || t > 1 )) {
                            JOptionPane op = new JOptionPane();
			    op.showMessageDialog(this,
                                                 "Invalid interval- limits! ("
                                                 + (i + 1) + ")",
                                                 "Error",
			                         JOptionPane.WARNING_MESSAGE);
			    return;
			}//end if (f > t || (f < 0 || t > 1 ))
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)

		    intervalcheck:
		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        for (int j = 0; j < m_numberOfIntervals; j++) {
                            double fI = Double.parseDouble(m_from[i].getText().trim());
                            double fJ = Double.parseDouble(m_from[j].getText().trim());
                            double tJ = Double.parseDouble(m_to[j].getText().trim());

                            if (j == i) {
                                if (fJ == tJ) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Empty interval! ("
                                                         + (i + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
				    m_noProblem = false;
			            break intervalcheck;
				}//end if (fJ == tJ)
			    }//end if (j == i)

			    else if (j != i) {
                                if ((fI >= fJ) && (fI < tJ)) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Interval- intersection! ("
                                                         + (i + 1) + "," + (j + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
				    m_noProblem = false;
				    break intervalcheck;
				}//end if ((fI >= fJ) && (fI < tJ))
			    }//end else if (j != i)
			}//end for (int j = 0; j < m_numberOfIntervals; j++)
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)
		}//end else if (    ((m_left.isSelected() == true) && (m_right.isSelected() == false))
		 //                 || ((m_left.isSelected() == false) && (m_right.isSelected() == true))
		 //                 || ((m_left.isSelected() == true) && (m_right.isSelected() == true)) )

		if (m_noProblem == true) {
                    for (int i = 0; i < m_numberOfIntervals; i++) {
                        m_interval[i].setLowerLimit(Double.parseDouble(m_from[i].getText().trim() ));
			m_interval[i].setUpperLimit(Double.parseDouble(m_to[i].getText().trim() ));
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)

                    m_visAlgGraph.setLeftOpen(m_left.isSelected());
                    m_visAlgGraph.setRightOpen(m_right.isSelected());

                    m_visAlgGraph.setVisAlgIntervals(m_interval);

		    m_tgvw.refresh(m_interval, this);

                    for (int i = 0; i < m_numberOfIntervals; i++) {
                        m_from[i].setEditable(false);
	                m_to[i].setEditable(false);
	            }//end for (int i = 0; i < m_numberOfIntervals; i++)

                    m_left.setEnabled(false);
                    m_right.setEnabled(false);

                    dispose();
		}//end if (m_noProblem == true)
	    }//end try
            catch (Exception e) {
                JOptionPane op = new JOptionPane();
		op.showMessageDialog(this,
                                     "Missing/ invalid interval- input!",
                                     "Error",
		                     JOptionPane.WARNING_MESSAGE);
            }//end catch
	}//end if (keyCode == KeyEvent.VK_ENTER)

        else if (keyCode == KeyEvent.VK_ESCAPE) {
            if (m_parent instanceof Intervalconfiguration) {
                dispose();
                Intervalconfiguration tmp = (Intervalconfiguration) m_parent;
                tmp.dispose();
                tmp.getParentOfIC().show();
            }//end if (m_parent instanceof Intervalconfiguration)
            else {
                dispose();
            }//end else
	}//end else if (keyCode == KeyEvent.VK_ESCAPE)
    }//end public void keyPressed(KeyEvent evt)

    /**
     * Dud- method
     */
    public void keyReleased(KeyEvent evt)
    {
    }

    /**
     * Dud- method
     */
    public void keyTyped(KeyEvent evt)
    {
    }


    // ------- from interface ActionListener ---------------

    /**
     * This function checks whether one of the Color- Buttons or the OK- Button
     * was pressed.
     * 'Color' opens ColorSelect- frame. 'OK' calls several tests in order to
     * assure the correctness of the given intervals.
     * @param evt   ActionEvent.
     */
    public void actionPerformed(ActionEvent evt) {
        try {
            if (evt.getActionCommand().equals("Color...")) {
                for (int i = 0; i < m_numberOfIntervals; i++) {
                    if (evt.getSource() == m_colorButton[i]) {

                        //Check for basic mistakes in interval- configuration.
			//The case "from == to" leads to a contradiction for
			//open and semi- open intervals , but not for compact
			//intervals. Because of this it must be ignored.
			//It will separately be examined below.

                        double f = Double.parseDouble(m_from[i].getText().trim());
                        double t = Double.parseDouble(m_to[i].getText().trim());

			if (f > t || (f < 0 || t > 1)) {
                            JOptionPane op = new JOptionPane();
			    op.showMessageDialog(this,
                                                 "Invalid interval- limits!",
                                                 "Error",
			                         JOptionPane.WARNING_MESSAGE);
			    break;
			}//end if (f > t || (f < 0 || t > 1))
			else {
                            Color cTmp = JColorChooser.showDialog(this,
                                                                  "Colorselection",
                                                                  m_tempColor[i]);

                            if (cTmp != null) {
                                m_tempColor[i] = cTmp;
                                m_interval[i].setColor(cTmp);
                                m_showColor[i].setBackground(cTmp);
                            }//end if (cTmp != null)
			    break;
			}//end else
		    }//end if (evt.getSource() == m_colorButton[i])
		}//end for (int i = 0; i < m_numberOfIntervals; i++)
	    }//end if (evt.getActionCommand().equals("Color..."))

            else if (evt.getActionCommand().equals("Cancel")) {
                if (m_parent instanceof Intervalconfiguration) {
                    dispose();
                    Intervalconfiguration tmp = (Intervalconfiguration) m_parent;
                    tmp.dispose();
                    tmp.getParentOfIC().show();
                }//end if (m_parent instanceof Intervalconfiguration)
                else {
                    dispose();
                }//end else
            }//end else if (evt.getActionCommand().equals("Cancel"))

            else if (evt.getActionCommand().equals("New intervals...")) {
                Intervalconfiguration ic = new Intervalconfiguration(m_visAlgGraph,m_tgvw, this);
                dispose();
                ic.show();
            }//end else if (evt.getActionCommand().equals("New intervals..."))

            else if (evt.getActionCommand().equals("OK")) {
                //Compact interval
		if (    (m_left.isSelected() == false)
                        && (m_right.isSelected() == false)) {
                    m_noProblem = true;

		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        double f = Double.parseDouble(m_from[i].getText().trim());
                        double t = Double.parseDouble(m_to[i].getText().trim());

                        if (f > t || (f < 0 || t > 1 )) {
                            JOptionPane op = new JOptionPane();
			    op.showMessageDialog(this,
                                                 "Invalid interval- limits! ("
                                                 + (i + 1) + ")",
                                                 "Error",
			                         JOptionPane.WARNING_MESSAGE);
			    return;
			}//end if (f > t || (f < 0 || t > 1 ))
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)

		    intervalcheck:
		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        for (int j = 0; j < m_numberOfIntervals; j++) {
                            if (j != i) {
                                double fI = Double.parseDouble(m_from[i].getText().trim());
                                double fJ = Double.parseDouble(m_from[j].getText().trim());
                                double tJ = Double.parseDouble(m_to[j].getText().trim());

                                if ((fI >= fJ) && (fI <= tJ)) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Interval- intersection! ("
                                                         + (i + 1) + "," + (j + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
                	            m_noProblem = false;
				    break intervalcheck;
		    		}//end if ((fI >= fJ) && (fI <= tJ))
			    }//end if (j != i)
			}//end for (int j = 0; j < m_numberOfIntervals; j++)
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)
		}//end if (    (m_left.isSelected() == false)
                 //            && (m_right.isSelected() == false))

		//Open or semi- open interval
		else if (    ((m_left.isSelected() == true) && (m_right.isSelected() == false))
		             || ((m_left.isSelected() == false) && (m_right.isSelected() == true))
		             || ((m_left.isSelected() == true) && (m_right.isSelected() == true)) ) {

                    //Open and semi- open intervals are both covered by the
		    //following code. But the differentiation is important
		    //for the colorize- method.

		    m_noProblem = true;

		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        double f = Double.parseDouble(m_from[i].getText().trim());
                        double t = Double.parseDouble(m_to[i].getText().trim());

                        if (f > t || (f < 0 || t > 1 )) {
                            JOptionPane op = new JOptionPane();
			    op.showMessageDialog(this,
                                                 "Invalid interval- limits! ("
                                                 + (i + 1) + ")",
                                                 "Error",
			                         JOptionPane.WARNING_MESSAGE);
			    return;
			}//end if (f > t || (f < 0 || t > 1 ))
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)

		    intervalcheck:
		    for (int i = 0; i < m_numberOfIntervals; i++) {
                        for (int j = 0; j < m_numberOfIntervals; j++) {
                            double fI = Double.parseDouble(m_from[i].getText().trim());
                            double fJ = Double.parseDouble(m_from[j].getText().trim());
                            double tJ = Double.parseDouble(m_to[j].getText().trim());

                            if (j == i) {
                                if (fJ == tJ) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Empty interval! ("
                                                         + (i + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
				    m_noProblem = false;
				    break intervalcheck;
				}//end if (fJ == tJ)
			    }//end if (j == i)

			    else if (j != i) {
                                if ((fI >= fJ) && (fI < tJ)) {
                                    JOptionPane op = new JOptionPane();
				    op.showMessageDialog(this,
                                                         "Interval- intersection! ("
                                                         + (i + 1) + "," + (j + 1) + ")",
                                                         "Error",
				                         JOptionPane.WARNING_MESSAGE);
				    m_noProblem = false;
				    break intervalcheck;
				}//end if ((fI >= fJ) && (fI < tJ))
			    }//end else if (j != i)
			}//end for (int j = 0; j < m_numberOfIntervals; j++)
		    }//end for (int i = 0; i < m_numberOfIntervals; i++)
		}//end else if (    ((m_left.isSelected() == true) && (m_right.isSelected() == false))
		 //                 || ((m_left.isSelected() == false) && (m_right.isSelected() == true))
		 //                 || ((m_left.isSelected() == true) && (m_right.isSelected() == true)) )

		if (m_noProblem == true) {
                    for (int i = 0; i < m_numberOfIntervals; i++) {
                        m_interval[i].setLowerLimit(Double.parseDouble(m_from[i].getText().trim() ));
			m_interval[i].setUpperLimit(Double.parseDouble(m_to[i].getText().trim() ));
                    }//end for (int i = 0; i < m_numberOfIntervals; i++)

                    m_visAlgGraph.setLeftOpen(m_left.isSelected());
                    m_visAlgGraph.setRightOpen(m_right.isSelected());

                    m_visAlgGraph.setVisAlgIntervals(m_interval);

		    m_tgvw.refresh(m_interval, this);

                    for (int i = 0; i < m_numberOfIntervals; i++) {
                        m_from[i].setEditable(false);
	                m_to[i].setEditable(false);
	            }//end for (int i = 0; i < m_numberOfIntervals; i++)

                    m_left.setEnabled(false);
                    m_right.setEnabled(false);

                    dispose();
		}//end if (m_noProblem == true)
	    }//end else if (evt.getActionCommand().equals("OK"))
	}//end try
	catch (Exception e) {
            JOptionPane op = new JOptionPane();
	    op.showMessageDialog(this,
                                 "Missing/ invalid interval- input!",
                                 "Error",
		                 JOptionPane.WARNING_MESSAGE);
	}//end catch
    }//end public void actionPerformed(ActionEvent evt)

}//end public class IntervalconfigurationFrame extends JFrame
 //                                            implements ActionListener, KeyListener