////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TSPGraphViewerWindow.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.modules.tspGraphViewerWindow;

import visalg.basics.ObjectReader;
import visalg.graphics.*;
import visalg.modules.TSPGraphViewer;
import visalg.types.ModuleChangeEvent;
import visalg.types.VisAlgGraph.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.io.File;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * This class creates a new TSPGraphViewer- Window.
 */
public class TSPGraphViewerWindow extends ModuleWindow
                                  implements ActionListener, MenuListener,
                                             MouseMotionListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
      *
      */
    private TSPGraphViewer m_tspGraphViewer;
    private Dimension m_dimension;
    private JMenu m_tspMenu;
    private JMenuItem m_back;
    private JMenuItem m_forward;
    private JMenuItem m_config;
    private Object m_obj = new Object();
    private Intervalconfiguration m_intConfig;
    private IntervalconfigurationFrame m_configFrame;

    private VisAlgGraph m_visAlgGraph;
    private int m_delID;
    private int m_delX;
    private int m_delY;
    private boolean m_drag;
    private int m_flag;
    private Vector m_vec;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
    * constructor:
    */
    public TSPGraphViewerWindow(String name, TSPGraphViewer tgv, Rectangle bounds,
                                ModuleWindowContainer mwc) {
        super(name, tgv, bounds, mwc);
        m_tspGraphViewer = tgv;
        m_dimension = getSize();

        m_visAlgGraph = (VisAlgGraph) m_tspGraphViewer.getDataModule().getData();

        VisAlgInterval[] tmp_interval = (VisAlgInterval[]) m_visAlgGraph.getVisAlgIntervals();
        m_obj       = tmp_interval;

	m_drag  = false;
	m_vec   = new Vector();
	m_flag  = 0;

        m_visAlgGraph.setNodeToEdges();

        setBackground(Color.white);

        setFont(new Font("SansSerif", 0, 10));
	addMouseListener( new MouseAdapter() {
                public void mouseClicked(MouseEvent evt) {
                }

                public void mousePressed(MouseEvent evt) {
                    Object[] v;
                    VisAlgNode van;

                    int x = evt.getX();
                    int y = evt.getY();

                    int temporary = m_visAlgGraph.nodeFoundAtXY(x, y);

                    if (temporary >= 0) {
                        setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));

                        v = m_visAlgGraph.getVisAlgNodes();

                        if (v[temporary] instanceof VisAlgNode) {
                            van = (VisAlgNode) v[temporary];
                        }//end if (v.get(temporary) instanceof VisAlgNode)
                        else {
                            return;
                        }//end else

                        m_visAlgGraph.setMouseDifferenceToNodeX(x - van.getXForDrawing());
                        m_visAlgGraph.setMouseDifferenceToNodeY(y - van.getYForDrawing());

                        m_delID = temporary;
			m_delX  = van.getAbsoluteX();
			m_delY  = van.getAbsoluteY();
                    }//end if (temporary >= 0)
                    else {
                        setCursor(Cursor.getDefaultCursor());
                    }//end else

                    m_visAlgGraph.setMousePressed(true);
                    repaint();
		}//end public void mousePressed(MouseEvent evt)

		public void mouseReleased(MouseEvent evt) {
                    if (m_drag == true) {
                        Object[] v;
                        VisAlgNode van;

                        v = m_visAlgGraph.getVisAlgNodes();

                        if (v[m_delID] instanceof VisAlgNode) {
                            van = (VisAlgNode) v[m_delID];
                        }//end if (v.get(temporary) instanceof VisAlgNode)
                        else {
                            return;
                        }//end else

                        int x = van.getAbsoluteX();
			int y = van.getAbsoluteY();

			if (m_flag == m_vec.size()) {
                            m_vec.add(new Delta(m_delID, (x - m_delX), (y - m_delY) ));
			    m_flag++;
			}//end if (m_flag == m_vec.size())

			else if (m_flag < m_vec.size()) {
                            m_vec.set(m_flag, new Delta(m_delID, (x - m_delX),
                                                        (y - m_delY) ));
			    m_flag++;

			    int i = m_vec.size() - m_flag;

			    while (i > 0) {
                                m_vec.remove(m_flag);
				i--;
			    }//end while (i > 0)
			}//end else if (m_flag < m_vec.size())
		    }//end if (m_drag == true)

		    m_drag = false;

                    if (m_visAlgGraph.nodeFoundAtXY(evt.getX(), evt.getY()) >= 0) {
                        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    }//end if (m_visAlgGraph.nodeFoundAtXY(evt.getX(),
                     //                                    evt.getY()) >= 0)
                    else {
                        setCursor(Cursor.getDefaultCursor());
                    }//end else

                    m_visAlgGraph.setMousePressed(false);

                    repaint();
		}//end public void mouseReleased(MouseEvent evt)
	});//end MouseAdapter()

	addMouseMotionListener(this);

        addComponentListener(new ComponentAdapter() {
                public void componentResized(ComponentEvent evt) {
                    m_visAlgGraph.determineDrawCoordinates( (int) m_dimension.getWidth(),
                                                            (int) m_dimension.getHeight());
                }//end public void componentResized(ComponentEvent evt)

                public void componentShown(ComponentEvent evt) {
                    m_visAlgGraph.determineDrawCoordinates( (int) m_dimension.getWidth(),
                                                            (int) m_dimension.getHeight());
                }//end public void componentShown(ComponentEvent evt)
        });//end ComponentAdapter()

        Object[] v = m_visAlgGraph.getVisAlgEdges();
        if (v != null) {
            VisAlgEdge vae;

	    for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++) {
                if (v[i] instanceof VisAlgEdge) {
                    vae = (VisAlgEdge) v[i];
                    vae.colorize(tmp_interval, m_visAlgGraph.isLeftOpen(),
                                 m_visAlgGraph.isRightOpen());
                }//end if (v[i] instanceof VisAlgEdge)
	    }//end for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++)
        } // end if (v == null)
    }//end public TSPGraphViewerWindow(String name, TSPGraphViewer tgv, Rectangle bounds,
     //                                ModuleWindowContainer mwc)

    /**
     * This function closes all open JFrames belonging to the TSPGraph- project.
     */
    public void onClose() {
        if (m_configFrame != null) {
            m_configFrame.dispose();
        }//end if (m_configFrame != null)

        if (m_intConfig != null) {
            m_intConfig.dispose();
        }//end if (m_intConfig != null)
    }//end public void onClose()

    /**
     * Adds the 'TSP Graph'- menu into the VisAlg- menubar.
     */
    public void onFocusGained() {
        m_tspMenu = new JMenu("TSP Graph");
        m_tspMenu.setMnemonic('T');

	JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic('F');
        fileMenu.addMenuListener(this);
	JMenu editMenu = new JMenu("Edit");
        editMenu.setMnemonic('E');
	editMenu.addMenuListener(this);

        m_config = new JMenuItem("Modify intervalconfiguration...", 'M');
        m_config.addActionListener(this);

	JMenuItem saveNode  = new JMenuItem("Export to TSPLib- file", 'L');
	saveNode.addActionListener(this);
	JMenuItem saveNodeAs  = new JMenuItem("Export to new TSPLib- file...", 'I');
	saveNodeAs.addActionListener(this);

        JMenuItem saveGraph  = new JMenuItem("Export to TSPGraph- file", 'G');
	saveGraph.addActionListener(this);
	JMenuItem saveGraphAs  = new JMenuItem("Export to new TSPGraph- file...", 'R');
	saveGraphAs.addActionListener(this);

        JMenuItem saveTex  = new JMenuItem("Export to TEX- file", 'T');
	saveTex.addActionListener(this);
	JMenuItem saveTexAs  = new JMenuItem("Export to new TEX- file...", 'E');
	saveTexAs.addActionListener(this);

        JMenuItem abtItm = new JMenuItem("About", 'A');
        abtItm.addActionListener(this);

	JMenuItem tnv = new JMenuItem("Toggle Name View", 'N');
	tnv.addActionListener(this);

	m_back = new JMenuItem("Back", 'B');
	m_back.addActionListener(this);

	m_forward = new JMenuItem("Forward", 'F');
	m_forward.addActionListener(this);

        // constructs the file- menu
        fileMenu.add(m_config);
        fileMenu.addSeparator();
	fileMenu.add(saveNode);
	fileMenu.add(saveNodeAs);
        fileMenu.addSeparator();
	fileMenu.add(saveGraph);
	fileMenu.add(saveGraphAs);
        fileMenu.addSeparator();
        fileMenu.add(saveTex);
        fileMenu.add(saveTexAs);

        editMenu.add(tnv);
        editMenu.addSeparator();
	editMenu.add(m_back);
	editMenu.add(m_forward);

	m_tspMenu.add(fileMenu);
	m_tspMenu.add(editMenu);
        m_tspMenu.addSeparator();
        m_tspMenu.add(abtItm);

        getModule().getModuleManager().getMenuBarContainer().addMenu(m_tspMenu);
    }//end public void onFocusGained()

    /**
     * Removes the 'TSP Graph'- menu from the VisAlg- menubar.
     */
    public void onFocusLost() {
        getModule().getModuleManager().getMenuBarContainer().removeMenu(m_tspMenu);
    }//end public void onFocusLost()

    /**
     * This module does not react on change events.
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {};


    /**
     * This function paints the current status of the VisAlgGraph.
     * @param g Graphics
     */
    public void paintComponent(Graphics g) {
        Object[] v;
        VisAlgNode van;
        VisAlgEdge vae;

        super.paintComponent(g);
        m_dimension = getSize();

        m_visAlgGraph.determineDrawCoordinates( (int) m_dimension.getWidth(),
                                                (int) m_dimension.getHeight());

        if ((v = m_visAlgGraph.getVisAlgEdges()) != null) {
	    for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++) {
                if (v[i] instanceof VisAlgEdge) {
                    vae = (VisAlgEdge) v[i];
                    vae.draw(g);
                }//end if (v.get(i) instanceof VisAlgNode)
	    }//end for (int i = 0; i < edges.length; i++)
        }//end if (v != null)

        v = m_visAlgGraph.getVisAlgNodes();

        for (int i = 0; i < m_visAlgGraph.numberOfNodes(); i++) {

            if (v[i] instanceof VisAlgNode) {
                van = (VisAlgNode) v[i];
            }//end if (v.get(i) instanceof VisAlgNode)
            else {
                return;
            }//end else

            if (    m_visAlgGraph.getCurrentNode() == i
                    && !m_visAlgGraph.isMousePressed()) {
                van.paintSelectedNode(g, true, m_visAlgGraph.isShowing(), m_dimension);
            }//end if (    m_visAlgGraph.getCurrentNode() == i
             //            && !m_visAlgGraph.isMousePressed())
            else {
                van.paintSelectedNode(g, false, m_visAlgGraph.isShowing(), m_dimension);
            }//end else

            if (    m_visAlgGraph.getCurrentNode() == i
                    && m_visAlgGraph.isMousePressed()) {
                van.paintActivatedNode(g, true);
            }//end if (    m_visAlgGraph.getCurrentNode() == i
             //            && m_visAlgGraph.isMousePressed())
            else {
                van.paintActivatedNode(g, false);
            }//end else
        } //for (int i = 0; i < m_visAlgGraph.numberOfNodes(); i++)
    }//end public void paintComponent(Graphics g)

    /**
     * This function updates the appropriate data, if the intervals have been
     * changed.
     * @param intervals   actual VisAlgInterval[]
     * @param lo          left open
     * @param ro          right open
     * @param icf         IntervalconfigurationFrame, that changed the intervals
     */
    public void refresh(VisAlgInterval[] intervals, IntervalconfigurationFrame icf) {

        VisAlgInterval[] tmp_vai = new VisAlgInterval[intervals.length];

        for (int i = 0; i < intervals.length; i++) {
            tmp_vai[i] = new VisAlgInterval(intervals[i].getLowerLimit(),
                                            intervals[i].getUpperLimit(),
                                            new Color(intervals[i].getColor().getRGB()));
        }//end for (int i = 0; i < intervals.length; i++)

        m_obj         = tmp_vai;
        m_configFrame = icf;

        Object[] v = m_visAlgGraph.getVisAlgEdges();
        if (v != null) {
            VisAlgEdge vae;

	    for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++) {
                if (v[i] instanceof VisAlgEdge) {
                    vae = (VisAlgEdge) v[i];
                    vae.colorize(tmp_vai, m_visAlgGraph.isLeftOpen(),
                                 m_visAlgGraph.isRightOpen());
                }//end if (v[i] instanceof VisAlgEdge)
	    }//end for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++)
        }// end if (v != null)

        repaint();
    }//end public void refresh(VisAlgInterval[] intervals, boolean lo, boolean ro,
     //                        IntervalconfigurationFrame icf)

    /**
     * This function checks, if m_flag has reached the first vector- element
     *
     * @return true m_flag is on the first element
     * @return false else
     */
    public boolean flagOnFirst() {
        if (m_flag == 0) {
            return true;
	}//end if (m_flag == 0)
	else {
            return false;
	}//end else
    }//end public boolean flagOnFirst()

    /**
     * This function checks, if m_flag has reached the last vector- element
     *
     * @return true m_flag is on the last element
     * @return false else
     */
    public boolean flagOnLast() {
        if (m_flag == m_vec.size()) {
            return true;
	}//end if (m_flag == m_vec.size())
	else {
            return false;
	}//end else
    }//end public boolean flagOnLast()

    /**
     * This function checks, whether the VisAlgGraph- Object contains any edges
     * or not.
     *
     * @return true, if VisAlgGraph- Object contains any edges
     * @return false, if VisAlgGraph- Object does not contain any edges
     */
    public boolean noEdges() {
        if (m_visAlgGraph.numberOfEdges() == -1) {
            return true;
        }//end if (m_visAlgGraph.numberOfEdges() == -1)
        else {
            return false;
        }//end else
    }//end public boolean noEdges()

    /**
     * This function checks whether the refresh()- method has been called at
     * least one time.
     *
     * @return true, if refresh() has been called
     * @return false, if refresh() has not yet been called
     */
    public boolean isRefreshed() {
        if (m_configFrame == null) {
            return false;
        }//end if (m_configFrame == null)
        else {
            return true;
        }//end else
    }//end public boolean isRefreshed()

    /**
     * This function returns the VisAlgInterval[]- Object stored in m_obj.
     *
     * @return m_obj VisAlgInterval[]
     */
    public VisAlgInterval[] getIntervals() {
        return (VisAlgInterval[]) m_obj;
    }//end public VisAlgInterval[] getIntervals()

    /**
     * This function sets the value of the m_intConfig- variable.
     * @param ic   Intervalconfiguration- Object
     */
    public void setIntervalconfiguration(Intervalconfiguration ic) {
        m_intConfig = ic;
    }//end public void setIntervalconfiguration(Intervalconfiguration ic)

    /**
     * This function sets the value of the m_configFrame- variable.
     * @param icf   IntervalconfigurationFrame- Object
     */
    public void setIntervalconfigurationFrame(IntervalconfigurationFrame icf) {
        m_configFrame = icf;
    }//end public void setIntervalconfigurationFrame(IntervalconfigurationFrame icf)


    // ------- from interface MouseMotionListener ----------

    /**
     * This function checks whether one of the nodes is found at the current
     * mouse- coordinates and changes the cursor, if necessary.
     * Then it calls the repaint()- method.
     * @param evt MouseEvent
     */
    public void mouseMoved(MouseEvent evt) {
        if (m_visAlgGraph.nodeFoundAtXY(evt.getX(), evt.getY()) >= 0) {
            setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        }//end if (visAlgGraph.nodeFoundAtXY(evt.getX(), evt.getY()) >= 0)
        else {
            setCursor(Cursor.getDefaultCursor());
        }//end else

        repaint();
    }//end public void mouseMoved(MouseEvent evt)

    /**
     * This function calculates the new node- coordinates and calls the
     * repaint()- method.
     * @param evt   MouseEvent
     */
    public void mouseDragged(MouseEvent evt) {
        Object[] v;
        VisAlgNode van;

        int x       = evt.getX();
        int y       = evt.getY();
        int current = m_visAlgGraph.getCurrentNode();
        Dimension d = getSize();

        int drawX;
        int drawY;

        if (current >= 0) {
            m_drag = true;

            drawX = x - m_visAlgGraph.getMouseDifferenceToNodeX();
            drawY = y - m_visAlgGraph.getMouseDifferenceToNodeY();

            drawX = (drawX < 20) ? 20 : drawX;
            drawY = (drawY < 20) ? 20 : drawY;

            drawX = (drawX >(int) m_dimension.getWidth() - 20)
                    ? ((int) m_dimension.getWidth() - 20)
                    : drawX;

            drawY = (drawY >(int) m_dimension.getHeight() - 40)
                    ? ((int) m_dimension.getHeight() - 40)
                    : drawY;

            v = m_visAlgGraph.getVisAlgNodes();

            if (v[current] instanceof VisAlgNode) {
                van = (VisAlgNode) v[current];
            }//end if (v.get(current) instanceof VisAlgNode)
            else {
                return;
            }//end else

            van.setXForDrawing(drawX);
            van.setYForDrawing(drawY);
            m_visAlgGraph.determineAbsoluteXYOnSize(drawX,
                                                    drawY,
                                                    (int) m_dimension.getWidth(),
                                                    (int) m_dimension.getHeight());
        }//end if (current >= 0)
        repaint();
    }//end public void mouseDragged(MouseEvent evt)


    // ------- from interface ActionListener ---------------

    /**
     * This function checks whether one of the "edit"- menu- items was
     * activated.
     * @param evt ActionEvent
     */
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() instanceof JMenuItem) {
            String arg = evt.getActionCommand();
            VisAlgNode       outNode;
            VisAlgEdge       outEdge;
            Object[]         outNodes = m_visAlgGraph.getVisAlgNodes();
            Object[]         outEdges = m_visAlgGraph.getVisAlgEdges();
            Object outIntvl = m_visAlgGraph.getVisAlgIntervals();

            if (arg.equals("Toggle Name View")) {

                if (m_visAlgGraph.isShowing()) {
                    VisAlgNode van;

                    for (int i = m_visAlgGraph.numberOfNodes() - 1;i >= 0 ;i--) {
                        van = m_visAlgGraph.getNodeAt(i);

                    // checks and if error - set back the old width and height
                        if (van == null) {
                            for (int j = i + 1;j < m_visAlgGraph.numberOfNodes(); j++) {
                                van.setHeight(24);
                                van.setWidth(24);
                            }
                            System.err.println("Not able to toggle name view");
                            return;
                        } // end if (van == null)

                        van.setHeight(14);
                        van.setWidth(10);
                    } // end for (i = numberOfNodes() - 1;i >= 0 ;i--)
                    m_visAlgGraph.setShowing(false);
                } // end if (m_visAlgGraph.isShowing())
                else {
                    VisAlgNode van;

                    for (int i = m_visAlgGraph.numberOfNodes() - 1;i >= 0 ;i--) {
                        van = m_visAlgGraph.getNodeAt(i);

                    // checks and if error - set back the old width and height
                        if (van == null) {
                            for (int j = i + 1;j < m_visAlgGraph.numberOfNodes(); j++) {
                                van.setHeight(14);
                                van.setWidth(10);
                            }
                            System.err.println("Not able to toggle name view");
                            return;
                        } // end if (van == null)

                        van.setHeight(24);
                        van.setWidth(24);
                    } // end for (i = numberOfNodes() - 1;i >= 0 ;i--)
                    m_visAlgGraph.setShowing(true);
                } // else -belongs to- if (m_visAlgGraph.isShowing())

                repaint();
            } // end if (arg.equals("Toggle Name View"))

	    else if (arg.equals("Back")) {
                if (flagOnFirst() == false) {
                    Object[] v;
                    VisAlgNode van;
                    v = m_visAlgGraph.getVisAlgNodes();

                    m_flag--;
		    Graphics g = getGraphics();
		    Delta del = (Delta)m_vec.get(m_flag);

                    if (v[del.getID()] instanceof VisAlgNode) {
                        van = (VisAlgNode) v[del.getID()];
                    }//end if (v.get(temporary) instanceof VisAlgNode)
                    else {
                        return;
                    }//end else

		    van.setAbsoluteX(van.getAbsoluteX() - del.getDeltaX() );
		    van.setAbsoluteY(van.getAbsoluteY() - del.getDeltaY() );

                    m_visAlgGraph.determineDrawCoordinates( (int) m_dimension.getWidth(),
                                                            (int) m_dimension.getHeight());

		    g.dispose();
		    repaint();
                }//end if (flagOnFirst() == false)
	    }//end if (arg.equals("Back"))

	    else if (arg.equals("Forward")) {
                if (flagOnLast() == false) {
                    Object[] v;
                    VisAlgNode van;
                    v = m_visAlgGraph.getVisAlgNodes();

                    Graphics g = getGraphics();
		    Delta del = (Delta)m_vec.get(m_flag);

                    if (v[del.getID()] instanceof VisAlgNode) {
                        van = (VisAlgNode) v[del.getID()];
                    }//end if (v.get(temporary) instanceof VisAlgNode)
                    else {
                        return;
                    }//end else

		    van.setAbsoluteX(van.getAbsoluteX() + del.getDeltaX() );
		    van.setAbsoluteY(van.getAbsoluteY() + del.getDeltaY() );

                    m_visAlgGraph.determineDrawCoordinates( (int) m_dimension.getWidth(),
                                                            (int) m_dimension.getHeight());

		    g.dispose();
		    m_flag++;
		    repaint();
                }//end if (flagOnLast() == false)
	    }//end else if (arg.equals("Forward"))

            else if (arg.equals("Modify intervalconfiguration...")) {
                if (m_configFrame != null) {
                    m_configFrame.dispose();
                }//end if (m_configFrame != null)
                IntervalconfigurationFrame icf = new IntervalconfigurationFrame(m_visAlgGraph.numberOfIntervals(),
                                                                                m_visAlgGraph, this, this);
                icf.show();
            }//end else if (arg.equals("Intervalconfiguration")

            else if (arg.equals("About")) {
                String about = "This module was developed by\n";
                about += "(in alphabetical order) :\n\n";
                about += "Bonato, Thorsten\n";
                about += "Braun, Christian T.\n";
                about += "Schoebel, Jens\n\n";
                about += "during the \"Software- Praktikum\" in 2001.";

                JOptionPane.showMessageDialog(null,
                                              about,
                                              "About TSPGraph",
                                              JOptionPane.PLAIN_MESSAGE);
            } // else if (arg.equals("About"))

            // ckecks if the file has no reference the
            // "Export to new TSPLib- file..." will be called.
            if (arg.equals("Export to TSPLib- file") && m_visAlgGraph.getTSPLibFile() == null) {
                arg = "Export to new TSPLib- file...";
            }

	    if (arg.equals("Export to new TSPLib- file...")) {
                JFileChooser d = new JFileChooser();

                d.setDialogTitle("Export to TSPLib- file");
                d.setCurrentDirectory(new File("." + File.separator + "import"));
                d.setMultiSelectionEnabled(false);

                // Checks for data, if nothing to save, it returns
                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to TSPLib- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }

                if (d.showSaveDialog(null) == JFileChooser.CANCEL_OPTION) {
		    return;
	        }// end if

                m_visAlgGraph.setTSPLibFile(d.getSelectedFile());

                // called if save- dialog is closed and not canceled
                if (m_visAlgGraph.getTSPLibFile() == null) {
                    return;
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTSPLibFile()));
                    int temporary = m_visAlgGraph.numberOfNodes();

                    // output the comment lines, necessary for tsp.

                    // NAME
                    out.writeBytes("NAME : " + m_visAlgGraph.getTSPLibFile().getName() + "\n");
                    // COMMENT
                    out.writeBytes("COMMENT : made with TSP- Graph on VisAlg\n");
                    // TYPE
                    out.writeBytes("TYPE : TSP\n");
                    // DIMENSION
                    out.writeBytes("DIMENSION : " + temporary + "\n");
                    // EDGE_WEIGHT_TYPE
                    out.writeBytes("EDGE_WEIGHT_TYPE : EUC_2D\n");
                    // NODE_COORD_SECTION
                    out.writeBytes("NODE_COORD_SECTION\n");

                    for (int i = 0; i < temporary; i++){

                        if (outNodes[i] instanceof VisAlgNode) {
                            outNode = (VisAlgNode) outNodes[i];

                            out.writeBytes(outNode.getName() +  " "
                                           + outNode.getAbsoluteX() + " "
                                           + outNode.getAbsoluteY() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPLib- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPLibFile().delete();
                            return;
                        }//end else
                    }// end for (int i = m_visAlgGraph.numberOfNodes(); i < temporary; i++)
                    out.writeBytes("EOF");
                    out.flush();
	            out.close();
	        }
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TSPLib- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTSPLibFile(null);
	        }
	    }//end if (arg.equals("Export to new TSPLib- file..."))

	    if (arg.equals("Export to TSPLib- file")) {
                // Checks for data, if nothing to save, it returns
                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to TSPLib- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTSPLibFile()));
                    int temporary = m_visAlgGraph.numberOfNodes();

                    // output the comment line, necessary for tsp.

                    // NAME
                    out.writeBytes("NAME : " + m_visAlgGraph.getTSPLibFile().getName() + "\n");
                    // COMMENT
                    out.writeBytes("COMMENT : made with TSPGraph on VisAlg\n");
                    // TYPE
                    out.writeBytes("TYPE : TSP\n");
                    // DIMENSION
                    out.writeBytes("DIMENSION : " + temporary + "\n");
                    // EDGE_WEIGHT_TYPE
                    out.writeBytes("EDGE_WEIGHT_TYPE : EUC_2D\n");
                    // NODE_COORD_SECTION
                    out.writeBytes("NODE_COORD_SECTION\n");

                    for (int i = 0; i < temporary; i++){

                        if (outNodes[i] instanceof VisAlgNode) {
                            outNode = (VisAlgNode) outNodes[i];

                            out.writeBytes(outNode.getName() +  " "
                                           + outNode.getAbsoluteX() + " "
                                           + outNode.getAbsoluteY() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPLib- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPLibFile().delete();
                            return;
                        }//end else
                    }// end for (int i = m_visAlgGraph.numberOfNodes(); i < temporary; i++)
                    out.writeBytes("EOF");
                    out.flush();
	            out.close();
	        }
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TSPLib- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTSPLibFile(null);
	        }
	    }//end if (arg.equals("Export to new TSPLib- file..."))

            // ckecks if the file has no reference the
            // "Export to new TSPGraph- file..." will be called.
            if (arg.equals("Export to TSPGraph- file") && m_visAlgGraph.getTSPGraphFile() == null) {
                arg = "Export to new TSPGraph- file...";
            }

	    if (arg.equals("Export to new TSPGraph- file...")) {
                JFileChooser d = new JFileChooser();

                d.setDialogTitle("Export to TSPGraph- file");
                d.setCurrentDirectory(new File("." + File.separator + "import"));
                d.setMultiSelectionEnabled(false);

                // Checks for data, if nothing to save, it returns
                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to TSPGraph- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }

                if (d.showSaveDialog(null) == JFileChooser.CANCEL_OPTION) {
		    return;
	        }// end if

                m_visAlgGraph.setTSPGraphFile(d.getSelectedFile());

                // called if save- dialog is closed and not canceled
                if (m_visAlgGraph.getTSPGraphFile() == null) {
                    return;
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTSPGraphFile()));
                    int temporary = m_visAlgGraph.numberOfNodes();

                    // specifies converter
                    out.writeBytes(ObjectReader.converterMagicNumber
                                   + "visalg.converter.TSPGraphConverter\n");

                    // NODE_SECTION
                    out.writeBytes("NODE_SECTION\n");

                    // puts out the nodes
                    for (int i = 0; i < temporary; i++){

                        if (outNodes[i] instanceof VisAlgNode) {
                            outNode = (VisAlgNode) outNodes[i];

                            out.writeBytes(outNode.getName() +  " "
                                           + outNode.getAbsoluteX() + " "
                                           + outNode.getAbsoluteY() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPGraph- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPGraphFile().delete();
                            return;
                        }//end else
                    }// end for (int i = m_visAlgGraph.numberOfNodes(); i < temporary; i++)
                    // EDGE_SECTION
                    out.writeBytes("EDGE_SECTION\n");

                    // puts out the edges
                    temporary = m_visAlgGraph.numberOfEdges();
                    for (int i = 0;
                         i < temporary;
                         i++) {

                        if (outEdges[i] instanceof VisAlgEdge) {
                            outEdge = (VisAlgEdge) outEdges[i];

                            out.writeBytes(m_visAlgGraph.getNodeAt(outEdge.getStartID()).getName() +  " "
                                           + m_visAlgGraph.getNodeAt(outEdge.getEndID()).getName() + " "
                                           + outEdge.getWeight() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPGraph- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPGraphFile().delete();
                            return;
                        }//end else
                    }// for (int i = 0; i < temporary && m_visAlgGraph.numberOfEdges() != -1; i++)

                    // EDGE_VIS_SECTION
                    out.writeBytes("EDGE_VIS_SECTION\n");
                    out.writeBytes("INTERVAL_TYPE : ");

                    if (m_visAlgGraph.isLeftOpen()) {

                        if (m_visAlgGraph.isRightOpen()) {
                            out.writeBytes("OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                        else {
                            out.writeBytes("LEFT_OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                    } // if (m_visAlgGraph.isLeftOpen())
                    else {

                        if (m_visAlgGraph.isRightOpen()) {
                            out.writeBytes("RIGHT_OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                        else {
                            out.writeBytes("COMPACT\n");
                        } // if (m_visAlgGraph.isRightOpen())
                    } // if (m_visAlgGraph.isLeftOpen())

                    if (outIntvl instanceof VisAlgInterval[]) {
                        VisAlgInterval[] outI;
                        outI = (VisAlgInterval[]) outIntvl;

                        for (int i = 0; i < outI.length; i++) {
                            out.writeBytes(outI[i].getLowerLimit() + " "
                                           + outI[i].getUpperLimit() + " "
                                           + outI[i].getColor().getRed() + " "
                                           + outI[i].getColor().getGreen() + " "
                                           + outI[i].getColor().getBlue() + "\n"
                            );
                        }
                    } // if (outIntvl instanceof VisAlgInterval[])

                    if (outIntvl instanceof VisAlgInterval) {
                        VisAlgInterval outI;
                        outI = (VisAlgInterval) outIntvl;

                        out.writeBytes(outI.getLowerLimit() + " "
                                       + outI.getUpperLimit() + " "
                                       + outI.getColor().getRed() + " "
                                       + outI.getColor().getGreen() + " "
                                       + outI.getColor().getBlue() + "\n"
                        );
                    } // if (outIntvl instanceof VisAlgInterval)

                    out.writeBytes("EOF");
                    out.flush();
	            out.close();
	        } // try
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TSPGraph- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTSPLibFile(null);
	        } // catch
	    }//end if (arg.equals("Export to new TSPGraph- file..."))

	    if (arg.equals("Export to TSPGraph- file")) {
                // Checks for data, if nothing to save, it returns
                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to TSPGraph- file",
                                                  JOptionPane.ERROR_MESSAGE);
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTSPGraphFile()));
                    int temporary = m_visAlgGraph.numberOfNodes();

                    // specifies converter
                    out.writeBytes(ObjectReader.converterMagicNumber
                                   + "visalg.converter.TSPGraphConverter\n");

                    // NODE_SECTION
                    out.writeBytes("NODE_SECTION\n");

                    // puts out the nodes
                    for (int i = 0; i < temporary; i++){

                        if (outNodes[i] instanceof VisAlgNode) {
                            outNode = (VisAlgNode) outNodes[i];

                            out.writeBytes(outNode.getName() +  " "
                                           + outNode.getAbsoluteX() + " "
                                           + outNode.getAbsoluteY() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPGraph- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPGraphFile().delete();
                            return;
                        }//end else
                    }// end for (int i = m_visAlgGraph.numberOfNodes(); i < temporary; i++)
                    // EDGE_SECTION
                    out.writeBytes("EDGE_SECTION\n");

                    // puts out the edges
                    temporary = m_visAlgGraph.numberOfEdges();
                    for (int i = 0;
                         i < temporary;
                         i++) {

                        if (outEdges[i] instanceof VisAlgEdge) {
                            outEdge = (VisAlgEdge) outEdges[i];

                            out.writeBytes(m_visAlgGraph.getNodeAt(outEdge.getStartID()).getName() +  " "
                                           + m_visAlgGraph.getNodeAt(outEdge.getEndID()).getName() + " "
                                           + outEdge.getWeight() + "\n");
                        } //end if (outEdges[i] instanceof VisAlgEdge)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TSPGraph- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTSPGraphFile().delete();
                            return;
                        }//end else
                    }// for (int i = 0; i < temporary && m_visAlgGraph.numberOfEdges() != -1; i++)

                    // EDGE_VIS_SECTION
                    out.writeBytes("EDGE_VIS_SECTION\n");
                    out.writeBytes("INTERVAL_TYPE : ");

                    if (m_visAlgGraph.isLeftOpen()) {

                        if (m_visAlgGraph.isRightOpen()) {
                            out.writeBytes("OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                        else {
                            out.writeBytes("LEFT_OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                    } // if (m_visAlgGraph.isLeftOpen())
                    else {

                        if (m_visAlgGraph.isRightOpen()) {
                            out.writeBytes("RIGHT_OPEN\n");
                        } // if (m_visAlgGraph.isRightOpen())
                        else {
                            out.writeBytes("COMPACT\n");
                        } // if (m_visAlgGraph.isRightOpen())
                    } // if (m_visAlgGraph.isLeftOpen())

                    if (outIntvl instanceof VisAlgInterval[]) {
                        VisAlgInterval[] outI;
                        outI = (VisAlgInterval[]) outIntvl;

                        for (int i = 0; i < outI.length; i++) {
                            out.writeBytes(outI[i].getLowerLimit() + " "
                                           + outI[i].getUpperLimit() + " "
                                           + outI[i].getColor().getRed() + " "
                                           + outI[i].getColor().getGreen() + " "
                                           + outI[i].getColor().getBlue() + "\n"
                            );
                        }
                    } // if (outIntvl instanceof VisAlgInterval[])

                    if (outIntvl instanceof VisAlgInterval) {
                        VisAlgInterval outI;
                        outI = (VisAlgInterval) outIntvl;

                        out.writeBytes(outI.getLowerLimit() + " "
                                       + outI.getUpperLimit() + " "
                                       + outI.getColor().getRed() + " "
                                       + outI.getColor().getGreen() + " "
                                       + outI.getColor().getBlue() + "\n"
                        );
                    } // if (outIntvl instanceof VisAlgInterval)

                    out.writeBytes("EOF");
                    out.flush();
	            out.close();
	        } // try
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TSPGraph- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTSPLibFile(null);
	        } // catch
	    }//end if (arg.equals("Export to new TSPLib- file..."))

            // ckecks if the file has no reference the
            // "Export to new TSPLib- file..." will be called.
           if (arg.equals("Export to TEX- file") && m_visAlgGraph.getTexFile() == null) {
                arg = "Export to new TEX- file...";
           }

	    if (arg.equals("Export to new TEX- file...")) {
                VisAlgNode van;
                VisAlgEdge vae;

                int X;
                int Y;
                // xy[0] = minX, xy[1] = maxX, xy[2] = minY, xy[3] = maxY
                int[] xy = m_visAlgGraph.getMinMaxXY();

                JFileChooser d = new JFileChooser();

                d.setDialogTitle("Export to TEX- file");
                d.setCurrentDirectory(new File("." + File.separator + "import"));
                d.setMultiSelectionEnabled(false);

                // Checks for data, if nothing to save, it returns
                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to TEX- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }

                if (d.showSaveDialog(null) == JFileChooser.CANCEL_OPTION) {
		    return;
	        }// end if

                m_visAlgGraph.setTexFile(d.getSelectedFile());

                // called if save- dialog is closed and not canceled
                if (m_visAlgGraph.getTexFile() == null) {
                    return;
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTexFile()));

                    // defines the picture in the TEX- file
                    out.writeBytes("\\documentclass[]{article}\n");
                    out.writeBytes("\\usepackage{a4wide}\n");
                    out.writeBytes("\\usepackage{pstcol,pst-node}\n");
                    out.writeBytes("\\pagestyle{empty}\n");
                    out.writeBytes("\\begin{document}\n");
                    out.writeBytes("\\psset{xunit=1mm,yunit=1mm}\n");
                    out.writeBytes("\\pspicture(0,0)(120,120)\n");

                    // sets the nodes and names
                    for (int i = 0; i < m_visAlgGraph.numberOfNodes(); i++) {
                        if (m_visAlgGraph.getNodeAt(i) instanceof VisAlgNode) {
                            van = m_visAlgGraph.getNodeAt(i);
                        } // if (m_vAG.getNodeAt(i) instanceof VAN)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TEX- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTexFile().delete();
                            return;
                        } // else: if (m_vAG.getNodeAt(i) instanceof VAN)

                        // determins the X- and Y- Value from the absolutes
                        X = (int) (100 * (van.getAbsoluteX() - xy[0])
                                  / (double) (xy[1] - xy[0]) + 10);

                        Y = (int) (110 - (100 * (van.getAbsoluteY() - xy[2])
                                   / (double) (xy[3] - xy[2])));

                        // dot with r = .5 mm
                        out.writeBytes("\\cnode*(" + X + "," + Y + "){0.5mm}{"
                                       + van.getName() + "}\n");

                        // name under the dot
                        out.writeBytes("\\rput(" + (X - 1) + ","
                                       + (Y + 1) + ".5){{\\tiny "
                                       + van.getName() + "}}\n");
                    } // for (int i = 0; i < m_vAG.numberOfNodes(); i--)

                    // notification on the lower right
                    out.writeBytes("\\rput[r](115,115){{\\tiny made with TSPGraph on VisAlg 1.1}}\n");

                    // sets line- thickness
                    out.writeBytes("\\psset{linewidth=.4pt}\n");

                    // sets the lines with start- and end- node and color
                    for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++) {
                        if (m_visAlgGraph.getEdgeAt(i) instanceof VisAlgEdge) {
                            vae = m_visAlgGraph.getEdgeAt(i);
                        } // if (m_vAG.getEdgeAt(i) instanceof VAN)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TEX- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTexFile().delete();
                            return;
                        } // else: if (m_vAG.getNodeAt(i) instanceof VAN)

                        // sets color
                        out.writeBytes("\\newrgbcolor{color" + (i + 1) + "}{"
                                       + vae.getColor().getRed() / (double) 256
                                       + " "
                                       + vae.getColor().getGreen() / (double) 256
                                       + " "
                                       + vae.getColor().getBlue() / (double) 256
                                       + "}\n");

                        // draws the line
                        out.writeBytes("\\ncline[linecolor=color" + (i + 1) + "]{"
                                       + m_visAlgGraph.getNodeAt(vae.getStartID()).getName()
                                       + "}{"
                                       + m_visAlgGraph.getNodeAt(vae.getEndID()).getName()
                                       + "}\n");
                    } // for (int i = 0; i < m_vAG.numberOfEdges(); i++)

                    // ends the TEX- file
                    out.writeBytes("\\endpspicture\n");
                    out.writeBytes("\\end{document}");

                    out.flush();
                    out.close();
                } // try
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TEX- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTexFile(null);
                } // catch
            } // if (arg.equals("Export to new TEX- file..."))

	    if (arg.equals("Export to TEX- file")) {
                VisAlgNode van;
                VisAlgEdge vae;

                int X;
                int Y;
                // xy[0] = minX, xy[1] = maxX, xy[2] = minY, xy[3] = maxY
                int[] xy = m_visAlgGraph.getMinMaxXY();

                if (outNodes == null) {
                    JOptionPane.showMessageDialog(null,
                                                  "No data for saving available!",
                                                  "Export to Tex- file",
                                                  JOptionPane.ERROR_MESSAGE);
                }

                try {
                    DataOutputStream out = new DataOutputStream(new FileOutputStream(m_visAlgGraph.getTexFile()));

                    // defines the picture in the TEX- file
                    out.writeBytes("\\documentclass[]{article}\n");
                    out.writeBytes("\\usepackage{a4wide}\n");
                    out.writeBytes("\\usepackage{pstcol,pst-node}\n");
                    out.writeBytes("\\pagestyle{empty}\n");
                    out.writeBytes("\\begin{document}\n");
                    out.writeBytes("\\psset{xunit=1mm,yunit=1mm}\n");
                    out.writeBytes("\\pspicture(0,0)(120,120)\n");

                    // sets the nodes and names
                    for (int i = 0; i < m_visAlgGraph.numberOfNodes(); i++) {
                        if (m_visAlgGraph.getNodeAt(i) instanceof VisAlgNode) {
                            van = m_visAlgGraph.getNodeAt(i);
                        } // if (m_vAG.getNodeAt(i) instanceof VAN)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TEX- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTexFile().delete();
                            return;
                        } // else: if (m_vAG.getNodeAt(i) instanceof VAN)

                        // determins the X- and Y- Value from the absolutes
                        X = (int) (100 * (van.getAbsoluteX() - xy[0])
                                   / (double) (xy[1] - xy[0])
                                   + 10);

                        Y = (int) (110 - (100 * (van.getAbsoluteY() - xy[2])
                                   / (double) (xy[3] - xy[2])));

                        // dot with r = .5 mm
                        out.writeBytes("\\cnode*(" + X + "," + Y + "){0.5mm}{"
                                       + van.getName() + "}\n");

                        // name under the dot
                        out.writeBytes("\\rput(" + (X - 1) + ","
                                       + (Y + 1) + ".5){{\\tiny "
                                       + van.getName() + "}}\n");
                    } // for (int i = 0; i < m_vAG.numberOfNodes(); i--)

                    // notification on the lower right
                    out.writeBytes("\\rput[r](115,115){{\\tiny made with TSPGraph on VisAlg 1.1}}\n");

                    // sets line- thickness
                    out.writeBytes("\\psset{linewidth=.4pt}\n");

                    // sets the lines with start- and end- node and color
                    for (int i = 0; i < m_visAlgGraph.numberOfEdges(); i++) {
                        if (m_visAlgGraph.getEdgeAt(i) instanceof VisAlgEdge) {
                            vae = m_visAlgGraph.getEdgeAt(i);
                        } // if (m_vAG.getEdgeAt(i) instanceof VAN)
                        else {
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong instance while saving data!",
                                                          "Export to TEX- file",
                                                          JOptionPane.ERROR_MESSAGE);
                            out.close();
                            m_visAlgGraph.getTexFile().delete();
                            return;
                        } // else: if (m_vAG.getNodeAt(i) instanceof VAN)

                        // sets color
                        out.writeBytes("\\newrgbcolor{color" + (i + 1) + "}{"
                                       + vae.getColor().getRed() / (double) 256
                                       + " "
                                       + vae.getColor().getGreen() / (double) 256
                                       + " "
                                       + vae.getColor().getBlue() / (double) 256
                                       + "}\n");

                        // draws the line
                        out.writeBytes("\\ncline[linecolor=color" + (i + 1) + "]{"
                                       + m_visAlgGraph.getNodeAt(vae.getStartID()).getName()
                                       + "}{"
                                       + m_visAlgGraph.getNodeAt(vae.getEndID()).getName()
                                       + "}\n");
                    } // for (int i = 0; i < m_vAG.numberOfEdges(); i++)

                    // ends the TEX- file
                    out.writeBytes("\\endpspicture\n");
                    out.writeBytes("\\end{document}");

                    out.flush();
                    out.close();
                } // try
                catch (IOException e) {
                    JOptionPane.showMessageDialog(null,
                                                  "Error while saving data!",
                                                  "Export to TEX- file",
                                                  JOptionPane.ERROR_MESSAGE);
                    m_visAlgGraph.setTexFile(null);
                } // catch
            } // if (arg.equals("Export to TEX- file"))
	}//end if (evt.getSource() instanceof JMenuItem)
    }//end public void actionPerformed(ActionEvent evt)


    // ------- from interface MenuListener -----------------

    /**
     * This function disables the menu- Item, if the action would throw an
     * exception.
     * @param evt   MenuEvent
     */
    public void menuSelected(MenuEvent evt) {
        m_back.setEnabled(!flagOnFirst());
	m_forward.setEnabled(!flagOnLast());
        m_config.setEnabled(!noEdges());
    }//end public void menuSelected(MenuEvent evt)

    /**
     * Dud- method
     */
    public void menuDeselected(MenuEvent evt) {
    }

    /**
     * Dud- method
     */
    public void menuCanceled(MenuEvent evt) {
    }
}//end public class TSPGraphViewerWindow extends ModuleWindow
 //                                      implements ActionListener, MenuListener,
 //                                                 MouseMotionListener