////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ModuleChangeEvent.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Wed Mar 07 18:42:00 2000
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
///////////////////////////////////////////////////////////////////////////////

package visalg.types;

import java.util.Hashtable;

import visalg.basics.Module;

/**
 * This class encapsulates the event, that is passed to every
 * ModuleListener, if a property of the module has changed. The ModuleChanged
 * object will be constructed including information about the source, which is
 * the module itself, and a HashTable containing the property information.
 * Every changed property has a key and a value. The key is a String, through
 * which the property is accessed, and the value is the new value of this
 * property.
 */
public class ModuleChangeEvent {

    /**
     * The changed properties.
     */
    private Hashtable m_table;

    /**
     * The source of the change.
     */
    private Module  m_source;

    /**
    * Initialises the event with just the source, no specific property will be
    * given. This constructor is used for simply notifying listeners, that a
    * change has occurred, not exactly what type.
    *
    * @param source The module, where a change has occurred
    */
    public ModuleChangeEvent(Module source)
    {
        m_source = source;
        m_table = null;
    }//ModuleChangeEvent(Module source)

    /**
    * Initialises the event with both, the source module and the changed
    * properties.
    *
    * @param source The module, where a change has occurred
    * @param prop All changed properties and the new values
    */
    public ModuleChangeEvent(Module source, Hashtable prop)
    {
      m_source = source;
      m_table = prop;
    }//ModuleChangeEvent(Module, HashTable)

    /**
     * Returns the module, that causes the event.
     *
     * @return The source module
     */
    public Module getSource()
    {
        return m_source;
    }//getSource()

    /**
     * Returns the properties, that have been changed.
     *
     * @return The changed properties
    */
    public Hashtable getProperties()
    {
        return m_table;
    }//getProperties()
}