////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgData.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 14:55:01 2000
//  Update count     : 3
//  CVS Info         : $Id: VisAlgData.java,v 1.2 2002/03/18 12:28:53 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		    Changes/Extensions
//  ----	    ----		    ------------------
//  03.03.01    vigelius        revised documentation
//  17.05.01    vigelius        added registerViewerModules(..)
//  29.10.01    vigelius        added cloneData()
//
///////////////////////////////////////////////////////////////////////////////


package visalg.types;


import java.io.Serializable;

import javax.swing.JComponent;
import java.awt.Graphics;

import visalg.basics.ModuleManager;
import visalg.basics.DataModule;

/**
 * This interface must be implemented by all data classes in VisAlg. Data should
 * also be capable of painting itself.
*/
public interface VisAlgData extends Serializable, Cloneable {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Should return a convenient JComponent, that can be used, to pain the
     * data, e.g. JLabel, for a number, ...
     *
     * @return A JComponent
     */
    public JComponent getComponent();

    /**
     * The data should be able to paint itself on a give graphics context.
     * A number, for example, could just the API method
     * Graphics.drawString(String,int,int).
     *
     * @param g The graphics context used for the painting
     * @param x,y The position for the painting
     */
    public void paintComponent(Graphics g, int x, int y);

    /**
     * The data must create all viewer module objects and register them at the
     * associated ModuleManager.
     *
     * @param mm The ModuleManager
     * @param dm The associated DataModule
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm);

    /**
     * The data must be able to clone itself. It is sufficient in most cases
     * to use the standard clone() method of java.object by calling up
     * super.clone(). However, if this
     * field-to-field copy is somewhat too less, classes must take care of
     * implementing an own clone() method.
     */
    public VisAlgData cloneData();

    /**
     * The data must be able to print itself to the stdOut.
     */
    public void printData();
} // interface VisAlgData
