////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgEdge.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.types.VisAlgGraph;


import visalg.types.VisAlgGraph.*;

import java.awt.*;


/**
 * This class defines the VisAlgEdge- Objects used in this project.
 */
public class VisAlgEdge {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    private int m_startID;
    private int m_endID;
    private VisAlgNode m_start;
    private VisAlgNode m_end;
    private double m_weight;
    private Color m_color = null;


    // -----------------------------------------------------
    // ------- M e t h o d s  ( p u b l i c ) --------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public VisAlgEdge() {
    }

    public VisAlgEdge(VisAlgNode s, VisAlgNode e, double w) {
        m_start  = s;
        m_end    = e;
        m_weight = w;
    }//end public VisAlgEdge(int s, int e, double w)

    /**
     * This function returns the value of the m_startID- variable.
     *
     * @return m_startID
     */
    public int getStartID() {
        return m_startID;
    }//end public int getStartID()

    /**
     * This function sets the value of the m_startID- variable.
     * @param s   Start- ID.
     */
    public void setStartID(int s) {
        m_startID = s;
    }//end public void setStartID(int s)

    /**
     * This function returns the value of the m_endID- variable.
     *
     * @return m_endID
     */
    public int getEndID() {
        return m_endID;
    }//end public int getEndID()

    /**
     * This function sets the value of the m_endID- variable.
     * @param s   End- ID
     */
    public void setEndID(int e) {
        m_endID = e;
    }//end public void setEndID(int e)

    /**
     * This function sets the value of the m_start- variable.
     * @param s   Start.
     */
    public void setStart(VisAlgNode s) {
        m_start = s;
    }//end public void setStart(int s)

    /**
     * This function returns the value of the m_start- variable.
     *
     * @return m_start
     */
    public VisAlgNode getStart() {
        return m_start;
    }//end public int getStart()

    /**
     * This function sets the value of the m_end- variable.
     * @param e   End.
     */
    public void setEnd(VisAlgNode e) {
        m_end = e;
    }//end public void setEnd(int e)

    /**
     * This function returns the value of the m_end- variable.
     *
     * @return m_end
     */
    public VisAlgNode getEnd() {
        return m_end;
    }//end public int getEnd()

    /**
     * This function sets the value of the m_weight- variable.
     * @param w   Weight.
     */
    public void setWeight(double w) {
        m_weight = w;
    }//end public void setWeight(double w)

    /**
     * This function returns the value of the m_weight- variable.
     *
     * @return m_weight
     */
    public double getWeight() {
        return m_weight;
    }//end public double getWeight()

    /**
     * This function returns the color of the edge- line.
     *
     * @return m_color
     */
    public Color getColor() {
        return m_color;
    }//end public double getWeight()

    /**
     * This function draws the given edge.
     * @param g   Graphics
     */
    public void draw(Graphics g) {
        if (m_color == null) {
            g.setColor(Color.black);
        }//end if (m_color == null)

        g.setColor(m_color);
        g.drawLine(m_start.getXForDrawing(), m_start.getYForDrawing(),
                   m_end.getXForDrawing()  , m_end.getYForDrawing() );
        g.setColor(Color.black);
    }//end public void draw(Graphics g)

    /**
     * This function sets the color of the given edge.
     * @param in   VisAlgInterval[]
     * @param lo   left open
     * @param ro   right open
     */
    public void colorize(VisAlgInterval[] in, boolean lo, boolean ro) {
        if ((lo == false) && (ro == false)) {
            for (int i = 0; i < in.length; i++) {
                if (    (m_weight >= in[i].getLowerLimit())
		        && (m_weight <= in[i].getUpperLimit()) ) {
                    m_color = in[i].getColor();
		    return;
		}//end if (    (m_weight >= in[i].getLowerLimit())
		 //            && (m_weight <= in[i].getUpperLimit()) )
	    }//end for (int i = 0; i < in.length; i++)
        }//end if ((lo == false) && (ro == false))

	if ((lo == true) && (ro == false)) {
            for (int i = 0; i < in.length; i++) {
                if (in[i].getLowerLimit() == 0) {
                    if (    (m_weight >=  in[i].getLowerLimit())
                            && (m_weight <= in[i].getUpperLimit()) ) {
                        m_color = in[i].getColor();
                        return;
                    }//end if (    (m_weight >  in[i].getLowerLimit())
                     //            && (m_weight <= in[i].getUpperLimit()) )
                }//end if (in[i].getLowerLimit() == 0)
                else {
                    if (    (m_weight >  in[i].getLowerLimit())
                            && (m_weight <= in[i].getUpperLimit()) ) {
                        m_color = in[i].getColor();
                        return;
                    }//end if (    (m_weight >  in[i].getLowerLimit())
                     //            && (m_weight <= in[i].getUpperLimit()) )
                }//end else
	    }//end for (int i = 0; i < in.length; i++)
	}//end if ((lo == true) && (ro == false))

	if ((lo == false) && (ro == true)) {
            for (int i = 0; i < in.length; i++) {
                if (in[i].getUpperLimit() == 1) {
                    if (    (m_weight >= in[i].getLowerLimit())
                            && (m_weight <=  in[i].getUpperLimit()) ) {
                        m_color = in[i].getColor();
                        return;
                    }//end if (    (m_weight >= in[i].getLowerLimit())
                     //            && (m_weight <  in[i].getUpperLimit()) )
                }//end if (in[i].getUpperLimit() == 1)
                else {
                    if (    (m_weight >= in[i].getLowerLimit())
                            && (m_weight <  in[i].getUpperLimit()) ) {
                        m_color = in[i].getColor();
                        return;
                    }//end if (    (m_weight >= in[i].getLowerLimit())
                     //            && (m_weight <  in[i].getUpperLimit()) )
                }//end else
	    }//end for (int i = 0; i < in.length; i++)
	}//end if ((lo == false) && (ro == true))

	if ((lo == true) && (ro == true)) {
            for (int i = 0; i < in.length; i++) {
                if (    (m_weight > in[i].getLowerLimit())
		        && (m_weight < in[i].getUpperLimit()) ) {
                    m_color = in[i].getColor();
		    return;
		}//end if (    (m_weight > in[i].getLowerLimit())
		 //            && (m_weight < in[i].getUpperLimit()) )
	    }//end for (int i = 0; i < in.length; i++)
	}//end if ((lo == true) && (ro == true))

        //This point will only be reached, if the edge is no element
        //of one of the intervals.
        m_color = Color.white;
    }//end public void colorize(Interval[] in, boolean lo, boolean ro)

}  // class VisAlgEdge