////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgNode.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.types.VisAlgGraph;


import visalg.types.VisAlgGraph.*;

import java.awt.*;


/**
 * This class defines the VisAlgNode- Objects used in this project.
 */
public class VisAlgNode {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     *
     */
    private int m_height = 24;
    private int m_width  = 24;
    private int m_absoluteX;
    private int m_absoluteY;
    private int m_xForDrawing;
    private int m_yForDrawing;
    private String m_nodeName;


    // -----------------------------------------------------
    // ------- M e t h o d s  ( p u b l i c ) --------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public VisAlgNode() {
    }

    public VisAlgNode(String s, int x, int y) {
        m_nodeName    = s;
        m_absoluteX   = x;
        m_absoluteY   = y;
    }//end public VisAlgNode(String s, int x, int y)

    /**
     * This function sets the value of the m_absoluteX- variable.
     * @param x   AbsoluteX.
     */
    public void setAbsoluteX(int x) {
        m_absoluteX = x;
    }//end public void setAbsoluteX(int x)

    /**
     * This function returns the value of the m_absoluteX- variable.
     *
     * @return m_absoluteX
     */
    public int getAbsoluteX() {
        return m_absoluteX;
    }//end public int getAbsoluteX()

    /**
     * This function sets the value of the m_absoluteY- variable.
     * @param y   AbsoluteY.
     */
    public void setAbsoluteY(int y) {
        m_absoluteY = y;
    }//end public void setAbsoluteY(int y)

    /**
     * This function returns the value of the m_absoluteY- variable.
     *
     * @return m_absoluteY
     */
    public int getAbsoluteY() {
        return m_absoluteY;
    }//end public int getAbsoluteY()

    /**
     * This function sets the string of the m_nodeName- variable.
     * @param n   Nodename.
     */
    public void setName(String n) {
        m_nodeName = n;
    }//end public void setName(String n)

    /**
     * This function returns the string of the m_nodeName- variable.
     *
     * @return m_nodeName
     */
    public String getName() {
        return m_nodeName;
    }//end public String getName()

    /**
     * This function sets the value of the m_xForDrawing- variable.
     * @param x   X- value for drawing.
     */
    public void setXForDrawing(int x) {
        m_xForDrawing = x;
    }//end public void setXForDrawing(int x)

    /**
     * This function returns the value of the m_xForDrawing- variable.
     *
     * @return m_xForDrawing
     */
    public int getXForDrawing() {
        return m_xForDrawing;
    }//end public int getXForDrawing()

    /**
     * This function sets the value of the m_yForDrawing- variable.
     * @param y   Y- value for drawing.
     */
    public void setYForDrawing(int y) {
        m_yForDrawing = y;
    }//end public void setYForDrawing(int y)

    /**
     * This function returns the value of the m_yForDrawing- variable.
     *
     * @return m_yForDrawing
     */
    public int getYForDrawing() {
        return m_yForDrawing;
    }//end public int getYForDrawing()

    /**
     * This function sets the value of the m_height- variable.
     * @param h   Height.
     */
    public void setHeight(int h) {
        m_height = h;
    }//end public void setHeight(int h)

    /**
     * This function returns the value of the m_height- variable.
     *
     * @return m_height
     */
    public int getHeight() {
        return m_height;
    }//end public int getHeight()

    /**
     * This function sets the value of the m_width- variable.
     * @param w   Width.
     */
    public void setWidth(int w) {
        m_width = w;
    }//end public void setWidth(int w)

    /**
     * This function returns the value of the m_width- variable.
     *
     * @return m_width
     */
    public int getWidth() {
       return m_width;
    }//end public int getWidth()

    /**
     * This function draws a node according to its status ( selected or not ).
     * @param g          Graphics
     * @param selected   Node- Selection- Status
     * @param sizeY      Window- height
     */
    public void paintSelectedNode(Graphics g, boolean selected, boolean showing, Dimension dim) {
        int sizeY = (int) dim.getHeight();
        int sizeX = (int) dim.getWidth();
        int width  = 8;

        FontMetrics fontMetrics = g.getFontMetrics();
        String outS = "Node " + m_nodeName + " is selected.";

        if (showing) {
            width = fontMetrics.stringWidth(m_nodeName);
            g.setColor(Color.black);
            g.drawString(m_nodeName, m_xForDrawing - width / 2,
                         m_yForDrawing - 4);
        }

        if(selected) {
            g.drawRect(m_xForDrawing - (width / 2) - 3,
                       m_yForDrawing - m_height  + 7 ,
                       width + 6,
                       m_height);

            int i = 25;
            do {
                g.setFont(new Font("SansSerif", 0, i));
                fontMetrics = g.getFontMetrics();
                i--;
            } while(fontMetrics.stringWidth(outS) >= (sizeX - 20)
                    && (i >= 10));

            g.drawString(outS, 10, sizeY - 10);
            g.setFont(new Font("SansSerif", 0, 10));
        }//end if(selected)
    }//end public void paintSelectedNode(Graphics g, boolean selected, boolean showing, int sizeY)

    /**
     * This function draws a node according to its status ( activated or not ).
     * @param g           Graphics
     * @param activated   Node- Status
     */
    public void paintActivatedNode(Graphics g, boolean activated) {
        if (activated) {
            g.setColor(Color.lightGray);
        }//end if (activated)
        else {
            g.setColor(Color.black);
        }//end else
        g.fillOval(m_xForDrawing - 3, m_yForDrawing - 3, 7, 7);
     }//end public void paintActivatedNode(Graphics g, boolean activated)

}//end public class VisAlgNode