////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgInt.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 17:04:17 2000
//  Update count     : 4
//  CVS Info         : $Id: VisAlgInt.java,v 1.2 2002/03/18 12:28:53 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  03.03.01    vigelius        revised documentation
//  29.10.01    vigelius        included empty constructor
//
///////////////////////////////////////////////////////////////////////////////


package visalg.types;


import javax.swing.JComponent;
import javax.swing.JLabel;
import java.awt.Graphics;

import visalg.basics.ModuleManager;
import visalg.basics.DataModule;
import visalg.modules.ArrayViewer;
import visalg.modules.MatrixArrayViewer;

/**
 * This class represents a VisAlg integer datum. It can be compared to eachother
 * and is capable of painting itself.
 */
public class VisAlgInt implements VisAlgComparableData {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * The number this data contains.
     * @serial
     */
    private int i;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Constructor with no start value. This is needed for creating
     * a reference object for vectors.
     */
    public VisAlgInt() {}

    /**
     * Constructor with input of type int.
     *
     * @param value The initial value of this datum
     */
    public VisAlgInt(int value) {
	i = value;
    }//VisAlgInt(int)

    /**
     * Constructor with input of type Integer.
     *
     * @param value The initial value of this datum
     */
    public VisAlgInt(Integer value) {
	i = value.intValue();
    }//VisAlgInt(Integer)

    /**
     * Returns the value as int.
     *
     * @return The datum
     */
    public int intValue() {
	return i;
    }//intValue()

    /**
     * Returns the value as string.
     *
     * @return The datum
     */
    public String toString() {
	return "" + i;
    }//toString

    /**
     * Clones the data.
     */
    public VisAlgData cloneData() {
        VisAlgInt newInt;

        newInt = new VisAlgInt(i);

        return newInt;
   }

    // ----------- from interface Comparable ------------------

    /**
     * Compares the specified object to this one.
     * @todo If objects are not of the same type throw ClassCastException.
     * @param o object to be compared to
     * @return difference o-i
     */
    public int compareTo(Object o) {
	// Not the same type
	if (!(o instanceof VisAlgInt))
	    return 0;

	return ((VisAlgInt)o).intValue() - i;
    }//compareTo(Object)


    // ----------- from interface VisAlgComparableData ------------------

    /**
     * Returns the absolute value of this datum.
     * @return The absolute value
     */
    public int compareToZero() {
	return i;
    }//compareToZero()


    // ----------- from interface VisAlgData ------------------

    /**
     * This function returns a label which shows the value of the
     * integer contained.
     *
     * @return The label
     */
    public JComponent getComponent() {
	JLabel label = new JLabel(toString());
	label.setHorizontalAlignment(JLabel.RIGHT);
	return label;
    }//getComponent()

    /**
     * This function paints the value of the contained integer as
     * string into the given Graphics component.
     *
     * @param g The graphics context
     * @param x,y The position
     */
    public void paintComponent(Graphics g, int x, int y) {
	g.drawString(String.valueOf(i), x, y);
    }//paintComponent(Graphics,int,int)

    /**
     * Registers ArrayViewer and MatrixArrayViewer as available ViewerModules.
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm) {
        mm.addModule(new ArrayViewer(mm, dm));
        mm.addModule(new MatrixArrayViewer(mm, dm));
    }//registerViewerModules(ModuleManager mm, DataModule dm)

    /**
     * Prints the Integer Value to the stdOut.
     */
    public void printData() {
        System.out.println(i);
    }//printData
    // ----------- from class Object ------------------

    /**
     * Returns a copy of this instance.
     *
     * @return A field-to-field copy of this instance
     */
    public Object clone() {
	try {
	    VisAlgInt vai = (VisAlgInt)super.clone();
	    vai.i = i;
	    return vai;
	} catch (CloneNotSupportedException e) {
	    return null;
	}
    }//clone()
} // class VisAlgInt