////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgMatrix.java
//  Description      :
//  Author           : Tom Kraus, Martin Nowak, Dominik Maidner
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Fri Apr 7 16:20:00 2000
//  Last modified by : ahr
//  Last modified on : Wed May  3 15:04:59 2000
//  Update count     : 3
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        T.Kraus,M.Nowak Methods SwapRow and SwapColumn added
// 29.10.01     vigelius        added convenient cloneData() method
//
///////////////////////////////////////////////////////////////////////////////


package visalg.types;

import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.Graphics;

import visalg.basics.ModuleManager;
import visalg.basics.DataModule;

import visalg.modules.MatrixTool;
/**
* VisAlgMatrix is a class for all kinds of matrices with elements
* of elementary data types
*/
public class VisAlgMatrix implements VisAlgData{

    // ---------------------------------
    // ------- V a r i a b l e s -------
    // ---------------------------------

    /**
     * @serial
     */
    public int rows, columns;

    /**
     * @serial
     */
    private Number[][] values;

    // -------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) ---------
    // -------------------------------------------------

    /**
     * Constructor
     */
    public VisAlgMatrix(int m, int n) {
	rows=m;
	columns=n;
	values=new Number[m][n];
    }

    /**
     * getValue returns the matrix element at position (i,j) as a
     * Number-Object.
     */
    public Number getValue(int m, int n) {
	return values[m][n];
    }

    /**
     * setValue gives the matrix element (i,j) the value of the
     * Number-Object "value".
     */
    public void setValue(int m, int n, Number value) {
	values[m][n]=value;
    }

	/**
	 * Method swaps two rows with each other
	*/
	public void SwapRows(int Kom1, int Kom2){
		for(int j=0;j<columns;j++){
			Number value_temp = getValue(Kom1,j);
			setValue(Kom1,j,getValue(Kom2,j));
			setValue(Kom2,j,value_temp);
		}
	}

	/**
	 * Method swaps two columns with each other
	*/
	public void SwapColumns(int Kon1, int Kon2){
		for(int i=0;i<rows;i++){
			Number value_temp = getValue(i,Kon1);
			setValue(i,Kon1,getValue(i,Kon2));
			setValue(i,Kon2,value_temp);
		}
	}

    /**
     * Clones the data. For this data type, a new object of type
     * VisAlgMatrix is created and the matrix elements are copied.
     */
    public VisAlgData cloneData() {
        VisAlgMatrix newMatrix = new VisAlgMatrix(rows, columns);

        for (int i=0; i<rows; i++) {
            for (int j=0; j<columns; j++) {
                newMatrix.values[i][j] = this.values[i][j];
            }
        }

        return newMatrix;
    }

    // ----------- from interface VisAlgData ------------------

    public JComponent getComponent(){return new JPanel();};

    public void paintComponent(Graphics g, int x, int y){};

    /**
     * The associated viewer module is MatrixTool.
     *
     * @see visalg.modules.MatrixTool
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm) {
        mm.addModule(new MatrixTool(mm, dm));
    }//registerViewerModules()

    /**
     * Prints the data to the stdOut.
     */
    public void printData() {
        System.out.println("Data type : VisAlgMatrix");
        System.out.println();
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < columns; j++) {
                System.out.print("" + values[i][j] + "\t");
            }
            System.out.println("");
        }
    }//printData()
} // class VisAlgMatrix
